@@ -17,10 +17,10 @@ CLASS_FACTORY::~CLASS_FACTORY() {
   SERVER::instance()->release();
 }
 
-STDMETHODIMP 
+STDMETHODIMP
 CLASS_FACTORY::QueryInterface(REFIID riid, void** ppv) {
   HRESULT ret = E_NOINTERFACE;
-  *ppv = 0;
+  *ppv = nullptr;
 
   if(IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory)) {
     *ppv = static_cast<CLASS_FACTORY*>(this);
@@ -33,15 +33,15 @@ CLASS_FACTORY::QueryInterface(REFIID riid, void** ppv) {
   return ret;
 }
 
-STDMETHODIMP_(ULONG) 
+STDMETHODIMP_(ULONG)
 CLASS_FACTORY::AddRef() {
   return InterlockedIncrement((LPLONG)&_ref_count);
 }
 
-STDMETHODIMP_(ULONG) 
+STDMETHODIMP_(ULONG)
 CLASS_FACTORY::Release() {
   ULONG ret = 0L;
-  
+
   if(InterlockedDecrement((LPLONG)&_ref_count) != 0)
     ret = _ref_count;
   else
@@ -50,25 +50,25 @@ CLASS_FACTORY::Release() {
   return ret;
 }
 
-STDMETHODIMP 
+STDMETHODIMP
 CLASS_FACTORY::CreateInstance(IUnknown* outer, REFIID refiid, void** obj) {
   HRESULT ret = CLASS_E_NOAGGREGATION;
   *obj = 0;
 
   // Shell extensions typically don't support aggregation (inheritance)
-  if(outer == 0) {
+  if(outer == nullptr) {
     DIFF_EXT* ext = new DIFF_EXT();
-  
-    if(ext == 0)
-      ret = E_OUTOFMEMORY;    
+
+    if(ext == nullptr)
+      ret = E_OUTOFMEMORY;
     else
       ret = ext->QueryInterface(refiid, obj);
   }
-  
+
   return ret;
 }
 
-STDMETHODIMP 
+STDMETHODIMP
 CLASS_FACTORY::LockServer(BOOL) {
   return NOERROR;
 }
