@@ -4,12 +4,15 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 #include "kdiff3fileitemaction.h"
 
-#include "../src/fileaccess.h"
 #include "../src/Utils.h"
+#include "../src/TypeUtils.h"
+
+#include <memory>
 
 #include <QAction>
 #include <QMenu>
@@ -24,12 +27,12 @@
 #include <KPluginLoader>
 #include <KProcess>
 
-static QStringList* s_pHistory = nullptr;
+std::unique_ptr<QStringList> s_pHistory;
 
 class KDiff3PluginHistory
 {
-    KConfig* m_pConfig;
-    KConfigGroup* m_pConfigGroup;
+    std::unique_ptr<KConfig> m_pConfig;
+    std::unique_ptr<KConfigGroup> m_pConfigGroup;
 
   public:
     KDiff3PluginHistory()
@@ -38,9 +41,9 @@ class KDiff3PluginHistory
         if(s_pHistory == nullptr)
         {
             //std::cout << "New History: " << instanceName << std::endl;
-            s_pHistory = new QStringList;
-            m_pConfig = new KConfig("kdiff3fileitemactionrc", KConfig::SimpleConfig);
-            m_pConfigGroup = new KConfigGroup(m_pConfig, "KDiff3Plugin");
+            s_pHistory = std::make_unique<QStringList>();
+            m_pConfig = std::make_unique<KConfig>("kdiff3fileitemactionrc", KConfig::SimpleConfig);
+            m_pConfigGroup = std::make_unique<KConfigGroup>(m_pConfig.get(), "KDiff3Plugin");
             *s_pHistory = m_pConfigGroup->readEntry("HistoryStack", QStringList());
         }
     }
@@ -50,11 +53,8 @@ class KDiff3PluginHistory
         //std::cout << "Delete History" << std::endl;
         if(s_pHistory && m_pConfigGroup)
             m_pConfigGroup->writeEntry("HistoryStack", *s_pHistory);
-        delete s_pHistory;
-        delete m_pConfigGroup;
-        delete m_pConfig;
+
         s_pHistory = nullptr;
-        m_pConfig = nullptr;
     }
 };
 
@@ -63,8 +63,8 @@ KDiff3PluginHistory s_history;
 K_PLUGIN_FACTORY_WITH_JSON(KDiff3FileItemActionFactory, "kdiff3fileitemaction.json", registerPlugin<KDiff3FileItemAction>();)
 #include "kdiff3fileitemaction.moc"
 
-KDiff3FileItemAction::KDiff3FileItemAction(QObject* pParent, const QVariantList& /*args*/)
-    : KAbstractFileItemActionPlugin(pParent)
+KDiff3FileItemAction::KDiff3FileItemAction(QObject* pParent, const QVariantList& /*args*/):
+    KAbstractFileItemActionPlugin(pParent)
 {
 }
 
@@ -101,7 +101,7 @@ QList<QAction*> KDiff3FileItemAction::actions(const KFileItemListProperties& fil
 
     if(m_list.count() == 1)
     {
-        int historyCount = s_pHistory ? s_pHistory->count() : 0;
+        QtSizeType historyCount = s_pHistory ? s_pHistory->count() : 0;
 
         actionText = i18n("Compare with %1", (historyCount > 0 ? s_pHistory->first() : QString()));
         pAction = new QAction(actionText, this);
@@ -141,7 +141,7 @@ QList<QAction*> KDiff3FileItemAction::actions(const KFileItemListProperties& fil
                 pHistoryMenu->addAction(pAction);
             }
 
-            pAction = new QAction(i18n("Clear list"), this);
+            pAction = new QAction(i18nc("Contexualmenu option to cleat comparison list", "Clear list"), this);
             connect(pAction, &QAction::triggered, this, &KDiff3FileItemAction::slotClearList);
             pActionMenu->addAction(pAction);
             pAction->setEnabled(historyCount > 0);
@@ -149,7 +149,7 @@ QList<QAction*> KDiff3FileItemAction::actions(const KFileItemListProperties& fil
     }
     else if(m_list.count() == 2)
     {
-        pAction = new QAction(i18n("Compare"), this);
+        pAction = new QAction(i18nc("Contexualmenu option ", "Compare"), this);
         connect(pAction, &QAction::triggered, this, &KDiff3FileItemAction::slotCompareTwoFiles);
         pActionMenu->addAction(pAction);
     }
@@ -176,9 +176,10 @@ void KDiff3FileItemAction::slotCompareWith()
 {
     if(m_list.count() > 0 && s_pHistory && !s_pHistory->empty())
     {
-        QStringList args;
-        args << s_pHistory->first();
-        args << Utils::urlToString(m_list.first());
+        QStringList args = {
+            s_pHistory->first(),
+            Utils::urlToString(m_list.first())
+        };
         KProcess::startDetached("kdiff3", args);
     }
 }
@@ -188,9 +189,10 @@ void KDiff3FileItemAction::slotCompareWithHistoryItem()
     const QAction* pAction = dynamic_cast<const QAction*>(sender());
     if(!m_list.isEmpty() && pAction)
     {
-        QStringList args;
-        args << pAction->data().toString();
-        args << Utils::urlToString(m_list.first());
+        QStringList args = {
+            pAction->data().toString(),
+            Utils::urlToString(m_list.first())
+        };
         KProcess::startDetached("kdiff3", args);
     }
 }
@@ -199,9 +201,10 @@ void KDiff3FileItemAction::slotCompareTwoFiles()
 {
     if(m_list.count() == 2)
     {
-        QStringList args;
-        args << Utils::urlToString(m_list.first());
-        args << Utils::urlToString(m_list.last());
+        QStringList args = {
+            Utils::urlToString(m_list.first()),
+            Utils::urlToString(m_list.last())
+        };
         KProcess::startDetached("kdiff3", args);
     }
 }
@@ -210,10 +213,11 @@ void KDiff3FileItemAction::slotCompareThreeFiles()
 {
     if(m_list.count() == 3)
     {
-        QStringList args;
-        args << Utils::urlToString(m_list.at(0));
-        args << Utils::urlToString(m_list.at(1));
-        args <<Utils::urlToString(m_list.at(2));
+        QStringList args = {
+            Utils::urlToString(m_list.at(0)),
+            Utils::urlToString(m_list.at(1)),
+            Utils::urlToString(m_list.at(2))
+        };
         KProcess::startDetached("kdiff3", args);
     }
 }
@@ -222,10 +226,11 @@ void KDiff3FileItemAction::slotMergeWith()
 {
     if(m_list.count() > 0 && s_pHistory && !s_pHistory->empty())
     {
-        QStringList args;
-        args << s_pHistory->first();
-        args << Utils::urlToString(m_list.first());
-        args << ("-o" + Utils::urlToString(m_list.first()));
+        QStringList args = {
+            s_pHistory->first(),
+            Utils::urlToString(m_list.first()),
+            ("-o" + Utils::urlToString(m_list.first()))
+        };
         KProcess::startDetached("kdiff3", args);
     }
 }
@@ -234,11 +239,12 @@ void KDiff3FileItemAction::slotMergeThreeWay()
 {
     if(m_list.count() > 0 && s_pHistory && s_pHistory->count() >= 2)
     {
-        QStringList args;
-        args << s_pHistory->at(1);
-        args << s_pHistory->at(0);
-        args << Utils::urlToString(m_list.first());
-        args << ("-o" + Utils::urlToString(m_list.first()));
+        QStringList args = {
+            s_pHistory->at(1),
+            s_pHistory->at(0),
+            Utils::urlToString(m_list.first()),
+            ("-o" + Utils::urlToString(m_list.first()))
+        };
         KProcess::startDetached("kdiff3", args);
     }
 }
