@@ -3,7 +3,8 @@
  *
  * SPDX-FileCopyrightText: 2019-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 #include "FileNameLineEdit.h"
 
 #include "fileaccess.h"
@@ -16,7 +17,7 @@
 
 void FileNameLineEdit::dropEvent(QDropEvent* event)
 {
-    Q_ASSERT(event->mimeData()->hasUrls());//Debugging aid in case FileNameLineEdit::dragEnterEvent is changed to accept other types.
+    assert(event->mimeData()->hasUrls());//Debugging aid in case FileNameLineEdit::dragEnterEvent is changed to accept other types.
 
     qCDebug(kdiffMain) << "Enter FileNameLineEdit::dropEvent";
     QList<QUrl> lst = event->mimeData()->urls();
