@@ -3,7 +3,8 @@
  *
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 #ifndef LINEREF_H
 #define LINEREF_H
 
@@ -20,9 +21,10 @@ class LineRef
     typedef qint32 LineType;
 
     static constexpr LineType invalid = -1;
-    inline LineRef() = default;
-    inline LineRef(const LineType i) noexcept { mLineNumber = i; }
-
+    constexpr inline LineRef() = default;
+    //cppcheck-suppress noExplicitConstructor
+    constexpr inline LineRef(const LineType i) noexcept { mLineNumber = i; }
+    //cppcheck-suppress noExplicitConstructor
     inline LineRef(const qint64 i) noexcept
     {
         if(i <= TYPE_MAX(LineType))
@@ -30,53 +32,56 @@ class LineRef
         else
             mLineNumber = invalid;
     }
+
     inline operator LineType() const noexcept { return mLineNumber; }
+
     inline LineRef& operator=(const LineType lineIn) noexcept
     {
         mLineNumber = lineIn;
         return *this;
     }
-    inline LineRef& operator+=(const LineType& inLine)
+
+    inline LineRef& operator+=(const LineType& inLine) noexcept
     {
         mLineNumber += inLine;
         return *this;
     };
 
-    LineRef& operator++()
+    LineRef& operator++() noexcept
     {
         ++mLineNumber;
         return *this;
     };
 
-    const LineRef operator++(int)
+    const LineRef operator++(int) noexcept
     {
         LineRef line(*this);
         ++mLineNumber;
         return line;
     };
 
-    LineRef& operator--()
+    LineRef& operator--() noexcept
     {
         --mLineNumber;
         return *this;
     };
 
-    const LineRef operator--(int)
+    const LineRef operator--(int) noexcept
     {
         LineRef line(*this);
         --mLineNumber;
         return line;
     };
     inline void invalidate() noexcept { mLineNumber = invalid; }
-    inline bool isValid() const noexcept { return mLineNumber != invalid; }
+    [[nodiscard]] inline bool isValid() const noexcept { return mLineNumber != invalid; }
 
   private:
-    LineType mLineNumber = invalid;
+    SafeInt<LineType> mLineNumber = invalid;
 };
 
 /*
     This is here because its easy to unknowingly break these conditions. The resulting
-    compiler errors are a bit cryptic if you aren't famialar with the C++ language.
+    compiler errors are numerous and may be a bit cryptic if you aren't famialar with the C++ language.
     Also some IDEs with clangd or ccls integration can automatically check static_asserts
     without doing a full compile.
 */
@@ -90,4 +95,6 @@ static_assert(std::is_convertible<int, LineRef>::value, "Can not convert int to
 typedef LineRef::LineType LineCount;
 typedef LineRef::LineType LineIndex;
 
+//Break in an obvious way if way cann't get LineCounts from Qt supplied ints without overflow issues.
+static_assert(sizeof(LineCount) >= sizeof(QtNumberType)); //Generally assumed by KDiff3
 #endif
