@@ -4,38 +4,25 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
-
+ */
+// clang-format on
 
 #include "MergeFileInfos.h"
 
 #include "DirectoryInfo.h"
 #include "directorymergewindow.h"
 #include "fileaccess.h"
+#include "Logging.h"
 #include "progress.h"
 
+#include <map>
+#include <vector>
+
 #include <QString>
 
 #include <KLocalizedString>
 
-MergeFileInfos::MergeFileInfos()
-{
-    m_bEqualAB = false;
-    m_bEqualAC = false;
-    m_bEqualBC = false;
-    m_pParent = nullptr;
-    m_bOperationComplete = false;
-    m_bSimOpComplete = false;
-    m_eMergeOperation = eNoOperation;
-    m_eOpStatus = eOpStatusNone;
-    m_ageA = eNotThere;
-    m_ageB = eNotThere;
-    m_ageC = eNotThere;
-    m_bConflictingAges = false;
-    m_pFileInfoA = nullptr;
-    m_pFileInfoB = nullptr;
-    m_pFileInfoC = nullptr;
-}
+MergeFileInfos::MergeFileInfos() = default;
 
 MergeFileInfos::~MergeFileInfos()
 {
@@ -191,19 +178,26 @@ QString MergeFileInfos::fullNameDest() const
 
 bool MergeFileInfos::compareFilesAndCalcAges(QStringList& errors, QSharedPointer<Options> const &pOptions, DirectoryMergeWindow* pDMW)
 {
-    std::map<QDateTime, int> dateMap;
+    enum class FilesFound
+    {
+        aOnly,
+        bAndA,
+        all
+    };
+
+    std::map<QDateTime, FilesFound> dateMap;
 
     if(existsInA())
     {
-        dateMap[getFileInfoA()->lastModified()] = 0;
+        dateMap[getFileInfoA()->lastModified()] = FilesFound::aOnly;
     }
     if(existsInB())
     {
-        dateMap[getFileInfoB()->lastModified()] = 1;
+        dateMap[getFileInfoB()->lastModified()] = FilesFound::bAndA;
     }
     if(existsInC())
     {
-        dateMap[getFileInfoC()->lastModified()] = 2;
+        dateMap[getFileInfoC()->lastModified()] = FilesFound::all;
     }
 
     if(pOptions->m_bDmFullAnalysis)
@@ -290,15 +284,16 @@ bool MergeFileInfos::compareFilesAndCalcAges(QStringList& errors, QSharedPointer
     if(isDirA() != isDirC()) m_bEqualAC = false;
     if(isDirB() != isDirC()) m_bEqualBC = false;
 
-    Q_ASSERT(eNew == 0 && eMiddle == 1 && eOld == 2);
+    assert(eNew == 0 && eMiddle == 1 && eOld == 2);
 
     // The map automatically sorts the keys.
     int age = eNew;
-    std::map<QDateTime, int>::reverse_iterator i;
+    std::map<QDateTime, FilesFound>::reverse_iterator i;
     for(i = dateMap.rbegin(); i != dateMap.rend(); ++i)
     {
-        int n = i->second;
-        if(n == 0 && getAgeA() == eNotThere)
+        FilesFound n = i->second;
+
+        if(n == FilesFound::aOnly && getAgeA() == eNotThere)
         {
             setAgeA((e_Age)age);
             ++age;
@@ -313,7 +308,7 @@ bool MergeFileInfos::compareFilesAndCalcAges(QStringList& errors, QSharedPointer
                 ++age;
             }
         }
-        else if(n == 1 && getAgeB() == eNotThere)
+        else if(n == FilesFound::bAndA && getAgeB() == eNotThere)
         {
             setAgeB((e_Age)age);
             ++age;
@@ -328,7 +323,7 @@ bool MergeFileInfos::compareFilesAndCalcAges(QStringList& errors, QSharedPointer
                 ++age;
             }
         }
-        else if(n == 2 && getAgeC() == eNotThere)
+        else if(n == FilesFound::all && getAgeC() == eNotThere)
         {
             setAgeC((e_Age)age);
             ++age;
@@ -386,27 +381,40 @@ bool MergeFileInfos::fastFileComparison(
     status = "";
     bError = true;
 
+    qCDebug(kdiffMergeFileInfo) << "Entering MergeFileInfos::fastFileComparison";
     if(fi1.isNormal() != fi2.isNormal())
     {
+        qCDebug(kdiffMergeFileInfo) << "Have: \'" << fi2.fileName() << "\' , isNormal = " << fi2.isNormal();
+
         status = i18n("Unable to compare non-normal file with normal file.");
         return false;
     }
 
     if(!fi1.isNormal())
     {
+        qCInfo(kdiffMergeFileInfo) << "Skipping not a normal file.";
         bError = false;
         return false;
     }
 
     if(!pOptions->m_bDmFollowFileLinks)
     {
+        qCInfo(kdiffMergeFileInfo) << "Have: \'" << fi2.fileName() << "\' , isSymLink = " << fi2.isSymLink();
+        /*
+            "git difftool --dir-diff"
+            sets up directory comparsions with symlinks to the current work tree being compared with real files.
+
+            m_bAllowMismatch is a compatibility work around.
+        */
         if(fi1.isSymLink() != fi2.isSymLink())
         {
+            qCDebug(kdiffMergeFileInfo) << "Rejecting comparison of link to file. With the same relitive path.";
             status = i18n("Mix of links and normal files.");
             return bEqual;
         }
         else if(fi1.isSymLink() && fi2.isSymLink())
         {
+            qCDebug(kdiffMergeFileInfo) << "Comparison of link to link. OK.";
             bError = false;
             bEqual = fi1.readLink() == fi2.readLink();
             status = i18n("Link: ");
@@ -416,6 +424,7 @@ bool MergeFileInfos::fastFileComparison(
 
     if(fi1.size() != fi2.size())
     {
+        qCInfo(kdiffMergeFileInfo) << "Sizes differ.";
         bError = false;
         bEqual = false;
         status = i18n("Size. ");
@@ -423,6 +432,8 @@ bool MergeFileInfos::fastFileComparison(
     }
     else if(pOptions->m_bDmTrustSize)
     {
+        qCInfo(kdiffMergeFileInfo) << "Same size. Trusting result.";
+
         bEqual = true;
         bError = false;
         return bEqual;
@@ -462,7 +473,7 @@ bool MergeFileInfos::fastFileComparison(
         status = fi2.errorString();
         return bEqual;
     }
-
+    qCInfo(kdiffMergeFileInfo) << "Comparing files...";
     pp.setInformation(i18n("Comparing file..."), 0, false);
     typedef qint64 t_FileSize;
     t_FileSize fullSize = fi1.size();
