@@ -4,7 +4,8 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 #ifndef MERGEFILEINFO_H
 #define MERGEFILEINFO_H
 
@@ -12,10 +13,9 @@
 #include "diff.h"
 #include "fileaccess.h"
 
+#include <QSharedPointer>
 #include <QString>
 
-//class DirectoryInfo;
-
 enum e_MergeOperation
 {
     eTitleId,
@@ -109,12 +109,10 @@ class MergeFileInfos
     Q_REQUIRED_RESULT QString fullNameC() const;
     Q_REQUIRED_RESULT QString fullNameDest() const;
 
-    Q_REQUIRED_RESULT static inline QSharedPointer<DirectoryInfo> getDirectoryInfo() { return gDirInfo; }
-
-    Q_REQUIRED_RESULT inline QString getDirNameA() const { return getDirectoryInfo()->dirA().prettyAbsPath(); }
-    Q_REQUIRED_RESULT inline QString getDirNameB() const { return getDirectoryInfo()->dirB().prettyAbsPath(); }
-    Q_REQUIRED_RESULT inline QString getDirNameC() const { return getDirectoryInfo()->dirC().prettyAbsPath(); }
-    Q_REQUIRED_RESULT inline QString getDirNameDest() const { return getDirectoryInfo()->destDir().prettyAbsPath(); }
+    Q_REQUIRED_RESULT inline QString getDirNameA() const { return gDirInfo->dirA().prettyAbsPath(); }
+    Q_REQUIRED_RESULT inline QString getDirNameB() const { return gDirInfo->dirB().prettyAbsPath(); }
+    Q_REQUIRED_RESULT inline QString getDirNameC() const { return gDirInfo->dirC().prettyAbsPath(); }
+    Q_REQUIRED_RESULT inline QString getDirNameDest() const { return gDirInfo->destDir().prettyAbsPath(); }
 
     Q_REQUIRED_RESULT inline TotalDiffStatus& diffStatus() { return m_totalDiffStatus; }
 
@@ -131,7 +129,7 @@ class MergeFileInfos
     Q_REQUIRED_RESULT inline bool isEqualAB() const { return m_bEqualAB; }
     Q_REQUIRED_RESULT inline bool isEqualAC() const { return m_bEqualAC; }
     Q_REQUIRED_RESULT inline bool isEqualBC() const { return m_bEqualBC; }
-    bool compareFilesAndCalcAges(QStringList& errors, QSharedPointer<Options> const &pOptions, DirectoryMergeWindow* pDMW);
+    bool compareFilesAndCalcAges(QStringList& errors, QSharedPointer<Options> const& pOptions, DirectoryMergeWindow* pDMW);
 
     void updateAge();
 
@@ -148,8 +146,8 @@ class MergeFileInfos
 
     Q_REQUIRED_RESULT inline bool isThreeWay() const
     {
-        if(getDirectoryInfo() == nullptr) return false;
-        return getDirectoryInfo()->dirC().isValid();
+        if(gDirInfo == nullptr) return false;
+        return gDirInfo->dirC().isValid();
     }
     Q_REQUIRED_RESULT inline bool existsEveryWhere() const { return existsInA() && existsInB() && (existsInC() || !isThreeWay()); }
 
@@ -162,33 +160,33 @@ class MergeFileInfos
     Q_REQUIRED_RESULT bool conflictingAges() const { return m_bConflictingAges; }
 
   private:
-    bool fastFileComparison(FileAccess& fi1, FileAccess& fi2, bool& bError, QString& status, const QSharedPointer<const Options> &pOptions);
+    bool fastFileComparison(FileAccess& fi1, FileAccess& fi2, bool& bError, QString& status, const QSharedPointer<const Options>& pOptions);
     inline void setAgeA(const e_Age inAge) { m_ageA = inAge; }
     inline void setAgeB(const e_Age inAge) { m_ageB = inAge; }
     inline void setAgeC(const e_Age inAge) { m_ageC = inAge; }
 
-    MergeFileInfos* m_pParent;
+    MergeFileInfos* m_pParent = nullptr;
     QList<MergeFileInfos*> m_children;
 
-    FileAccess* m_pFileInfoA;
-    FileAccess* m_pFileInfoB;
-    FileAccess* m_pFileInfoC;
+    FileAccess* m_pFileInfoA = nullptr;
+    FileAccess* m_pFileInfoB = nullptr;
+    FileAccess* m_pFileInfoC = nullptr;
 
     TotalDiffStatus m_totalDiffStatus;
 
-    e_MergeOperation m_eMergeOperation;
-    e_OperationStatus m_eOpStatus;
-    e_Age m_ageA;
-    e_Age m_ageB;
-    e_Age m_ageC;
+    e_MergeOperation m_eMergeOperation = eNoOperation;
+    e_OperationStatus m_eOpStatus = eOpStatusNone;
+    e_Age m_ageA = eNotThere;
+    e_Age m_ageB = eNotThere;
+    e_Age m_ageC = eNotThere;
 
-    bool m_bOperationComplete;
-    bool m_bSimOpComplete;
+    bool m_bOperationComplete = false;
+    bool m_bSimOpComplete = false;
 
-    bool m_bEqualAB;
-    bool m_bEqualAC;
-    bool m_bEqualBC;
-    bool m_bConflictingAges; // Equal age but files are not!
+    bool m_bEqualAB = false;
+    bool m_bEqualAC = false;
+    bool m_bEqualBC = false;
+    bool m_bConflictingAges = false; // Equal age but files are not!
 };
 
 QTextStream& operator<<(QTextStream& ts, MergeFileInfos& mfi);
