@@ -1,15 +1,20 @@
 /*
  * KDiff3 - Text Diff And Merge Tool
- * 
+ *
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 #ifndef OPTIONITEMS_H
 #define OPTIONITEMS_H
 
 #include "common.h"
+#include "options.h"
+#include "TypeUtils.h"
+
+#include <boost/signals2.hpp>
 
 #include <QComboBox>
 #include <QLabel>
@@ -23,41 +28,43 @@
 class OptionItemBase
 {
   public:
-    explicit OptionItemBase(const QString& saveName)
-    {
-        m_saveName = saveName;
-        m_bPreserved = false;
-    }
+    explicit OptionItemBase(const QString& saveName);
     virtual ~OptionItemBase() = default;
+
     virtual void setToDefault() = 0;
     virtual void setToCurrent() = 0;
 
     virtual void apply() = 0;
-    
+
     virtual void write(ValueMap*) const = 0;
     virtual void read(ValueMap*) = 0;
-    void doPreserve()
+
+    void preserve()
     {
         if(!m_bPreserved)
         {
             m_bPreserved = true;
-            preserve();
+            preserveImp();
         }
     }
-    void doUnpreserve()
+
+    void unpreserve()
     {
         if(m_bPreserved)
         {
-            unpreserve();
+            unpreserveImp();
         }
     }
-    QString getSaveName() const { return m_saveName; }
+
+    bool accept(const QString& key, const QString& val);
+
+    [[nodiscard]] QString getSaveName() const { return m_saveName; }
   protected:
-    virtual void preserve() = 0;
-    virtual void unpreserve() = 0;
+    virtual void preserveImp() = 0;
+    virtual void unpreserveImp() = 0;
     bool m_bPreserved;
     QString m_saveName;
-
+    std::list<boost::signals2::scoped_connection> connections;
     Q_DISABLE_COPY(OptionItemBase)
 };
 
@@ -89,10 +96,10 @@ class Option : public OptionItemBase
 
     void setToDefault() override {};
     void setToCurrent() override {};
-    const T& getDefault() const { return m_defaultVal; };
-    const T getCurrent() const { return *m_pVar; };
+    [[nodiscard]] const T& getDefault() const { return m_defaultVal; };
+    [[nodiscard]] const T getCurrent() const { return *m_pVar; };
 
-    virtual void setCurrent(const T inValue) { *m_pVar = inValue; }
+    virtual void setCurrent(const T& inValue) { *m_pVar = inValue; }
 
     void apply() override {};
     virtual void apply(const T& inValue) { *m_pVar = inValue; }
@@ -101,9 +108,9 @@ class Option : public OptionItemBase
     void read(ValueMap* config) override { *m_pVar = config->readEntry(m_saveName, m_defaultVal); }
 
   protected:
-    void preserve() override { m_preservedVal = *m_pVar; }
-    void unpreserve() override { *m_pVar = m_preservedVal; }
-    T* m_pVar;
+    void preserveImp() override { m_preservedVal = *m_pVar; }
+    void unpreserveImp() override { *m_pVar = m_preservedVal; }
+    T* m_pVar = nullptr;
     T m_preservedVal;
     T m_defaultVal;
 
@@ -126,7 +133,7 @@ class OptionNum : public Option<T>
     {
     }
 
-    void setCurrent(const T inValue) override
+    void setCurrent(const T& inValue) override
     {
         Option<T>::setCurrent(inValue);
     }
@@ -136,7 +143,7 @@ class OptionNum : public Option<T>
         //QString::setNum does not use locale formatting instead it always use QLocale::C.
         return QLocale().toString(inValue);
     }
-    const QString getString() const
+    [[nodiscard]] const QString getString() const
     {
         //QString::setNum does not use locale formatting instead it always use QLocale::C.
         return QLocale().toString(Option<T>::getCurrent());
@@ -162,16 +169,16 @@ class OptionCodec : public OptionString
   public:
     using OptionString::Option;
 
-    void setCurrent(const QString name) override { OptionString::setCurrent(name); };
+    void setCurrent(const QString& name) override { OptionString::setCurrent(name); };
     void setCurrent(const QByteArray& name) { OptionString::setCurrent(QString::fromLatin1(name)); }
-    const QString& defaultName() const { return mDefaultName; }
+    [[nodiscard]] const QString& defaultName() const { return mDefaultName; }
 
-    void saveDefaultIndex(const int i) { defaultIndex = i; };
-    int getDefaultIndex() const { return defaultIndex; }
+    void saveDefaultIndex(const SafeInt<QtNumberType> i) { defaultIndex = i; };
+    [[nodiscard]] QtNumberType getDefaultIndex() const { return defaultIndex; }
 
   private:
     const QString mDefaultName = QLatin1String(QTextCodec::codecForLocale()->name());
-    int defaultIndex = 0;
+    QtNumberType defaultIndex = 0;
     Q_DISABLE_COPY(OptionCodec)
 };
 
