@@ -1,3 +1,4 @@
+// clang-format off
 /*
  *  This file is part of KDiff3.
  *
@@ -5,12 +6,14 @@
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
 */
+// clang-format on
 #ifndef OVERVIEW_H
 #define OVERVIEW_H
 
 #include "LineRef.h"       // for LineRef
 #include "TypeUtils.h"     // for QtNumberType
 
+#include <QSharedPointer>
 #include <QString>         // for QString
 #include <QPixmap>
 #include <QWidget>
