@@ -4,7 +4,8 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 /* Features of class SourceData:
 - Read a file (from the given URL) or accept data via a string.
@@ -32,10 +33,16 @@ Optimizations: Skip unneeded steps.
 
 #include "CommentParser.h"
 #include "diff.h"
-#include "kdiff3.h"
+#include "LineRef.h"
 #include "Logging.h"
 #include "Utils.h"
 
+#include <algorithm>         // for min
+#include <memory>
+#include <vector>            // for vector
+
+#include <QtGlobal>
+
 #include <QProcess>
 #include <QScopedPointer>
 #include <QString>
@@ -46,13 +53,6 @@ Optimizations: Skip unneeded steps.
 
 #include <KLocalizedString>
 
-void SourceData::setupConnections()
-{
-#ifndef AUTOTEST
-    connections.push_back(KDiff3App::encodingChanged.connect(boost::bind(&SourceData::setEncoding, this, boost::placeholders::_1)));
-#endif
-}
-
 void SourceData::reset()
 {
     mFromClipBoard = false;
@@ -97,7 +97,7 @@ bool SourceData::isValid() const
     return isEmpty() || hasData();
 }
 
-void SourceData::setOptions(const QSharedPointer<Options> &pOptions)
+void SourceData::setOptions(const QSharedPointer<Options>& pOptions)
 {
     m_pOptions = pOptions;
 }
@@ -157,28 +157,31 @@ const QString SourceData::setData(const QString& data)
     {
         m_aliasName = i18n("From Clipboard");
         mFromClipBoard = true;
-        //m_fileAccess = FileAccess(); // Insure m_fileAccess is not valid
     }
 
-    return QLatin1String("");
+    return u8"";
 }
 
-const QVector<LineData>* SourceData::getLineDataForDiff() const
+const std::shared_ptr<LineDataVector>& SourceData::getLineDataForDiff() const
 {
     if(m_lmppData.m_pBuf == nullptr)
-        return m_normalData.m_v.size() > 0 ? &m_normalData.m_v : nullptr;
+    {
+        return m_normalData.m_v;
+    }
     else
-        return m_lmppData.m_v.size() > 0 ? &m_lmppData.m_v : nullptr;
+    {
+        return m_lmppData.m_v;
+    }
 }
 
-const QVector<LineData>* SourceData::getLineDataForDisplay() const
+const std::shared_ptr<LineDataVector>& SourceData::getLineDataForDisplay() const
 {
-    return m_normalData.m_v.size() > 0 ? &m_normalData.m_v : nullptr;
+    return m_normalData.m_v;
 }
 
-LineRef SourceData::getSizeLines() const
+LineCount SourceData::getSizeLines() const
 {
-    return m_normalData.lineCount();
+    return SafeInt<LineCount>(m_normalData.lineCount());
 }
 
 qint64 SourceData::getSizeBytes() const
@@ -188,7 +191,7 @@ qint64 SourceData::getSizeBytes() const
 
 const char* SourceData::getBuf() const
 {
-    return m_normalData.m_pBuf;
+    return m_normalData.m_pBuf.get();
 }
 
 const QString& SourceData::getText() const
@@ -225,12 +228,8 @@ bool SourceData::isBinaryEqualWith(const QSharedPointer<SourceData>& other) cons
 */
 void SourceData::FileData::reset()
 {
-    if(m_pBuf != nullptr)
-    {
-        delete[](char*) m_pBuf;
-        m_pBuf = nullptr;
-    }
-    m_v.clear();
+    m_pBuf.reset();
+    m_v->clear();
     mDataSize = 0;
     mLineCount = 0;
     m_bIsText = false;
@@ -246,30 +245,26 @@ bool SourceData::FileData::readFile(FileAccess& file)
         return true;
     }
 
-    //FileAccess fa(filename);
-
     if(!file.isNormal())
         return true;
 
     mDataSize = file.sizeForReading();
-    char* pBuf;
-    m_pBuf = pBuf = new char[mDataSize + 100]; // Alloc 100 byte extra: Safety hack, not nice but does no harm.
-                                               // Some extra bytes at the end of the buffer are needed by
-                                               // the diff algorithm. See also GnuDiff::diff_2_files().
-    bool bSuccess = file.readFile(pBuf, mDataSize);
+    m_pBuf = std::make_unique<char[]>(mDataSize + 100); // Alloc 100 byte extra: Safety hack, not nice but does no harm.
+                                                        // Some extra bytes at the end of the buffer are needed by
+                                                        // the diff algorithm. See also GnuDiff::diff_2_files().
+    bool bSuccess = file.readFile(m_pBuf.get(), mDataSize);
     if(!bSuccess)
     {
-        delete[] pBuf;
         m_pBuf = nullptr;
         mDataSize = 0;
     }
     else
     {
         //null terminate buffer
-        pBuf[mDataSize + 1] = 0;
-        pBuf[mDataSize + 2] = 0;
-        pBuf[mDataSize + 3] = 0;
-        pBuf[mDataSize + 4] = 0;
+        m_pBuf[mDataSize + 1] = 0;
+        m_pBuf[mDataSize + 2] = 0;
+        m_pBuf[mDataSize + 3] = 0;
+        m_pBuf[mDataSize + 4] = 0;
     }
     return bSuccess;
 }
@@ -288,14 +283,12 @@ bool SourceData::FileData::readFile(const QString& filename)
         return true;
 
     mDataSize = fa.sizeForReading();
-    char* pBuf;
-    m_pBuf = pBuf = new char[mDataSize + 100]; // Alloc 100 byte extra: Safety hack, not nice but does no harm.
-                                               // Some extra bytes at the end of the buffer are needed by
-                                               // the diff algorithm. See also GnuDiff::diff_2_files().
-    bool bSuccess = fa.readFile(pBuf, mDataSize);
+    m_pBuf = std::make_unique<char[]>(mDataSize + 100); // Alloc 100 byte extra: Safety hack, not nice but does no harm.
+                                                        // Some extra bytes at the end of the buffer are needed by
+                                                        // the diff algorithm. See also GnuDiff::diff_2_files().
+    bool bSuccess = fa.readFile(m_pBuf.get(), mDataSize);
     if(!bSuccess)
     {
-        delete[] pBuf;
         m_pBuf = nullptr;
         mDataSize = 0;
     }
@@ -315,7 +308,7 @@ bool SourceData::FileData::writeFile(const QString& filename)
     }
 
     FileAccess fa(filename);
-    bool bSuccess = fa.writeFile(m_pBuf, mDataSize);
+    bool bSuccess = fa.writeFile(m_pBuf.get(), mDataSize);
     return bSuccess;
 }
 
@@ -323,11 +316,10 @@ bool SourceData::FileData::writeFile(const QString& filename)
 void SourceData::FileData::copyBufFrom(const FileData& src) //TODO: Remove me.
 {
     reset();
-    char* pBuf;
     mDataSize = src.mDataSize;
-    m_pBuf = pBuf = new char[mDataSize + 100];
-    Q_ASSERT(src.m_pBuf != nullptr);
-    memcpy(pBuf, src.m_pBuf, mDataSize);
+    m_pBuf = std::make_unique<char[]>(mDataSize + 100);
+    assert(src.m_pBuf != nullptr);
+    memcpy(m_pBuf.get(), src.m_pBuf.get(), mDataSize);
 }
 
 QTextCodec* SourceData::detectEncoding(const QString& fileName, QTextCodec* pFallbackCodec)
@@ -338,7 +330,7 @@ QTextCodec* SourceData::detectEncoding(const QString& fileName, QTextCodec* pFal
         char buf[400];
 
         qint64 size = f.read(buf, sizeof(buf));
-        qint64 skipBytes = 0;
+        FileOffset skipBytes = 0;
         QTextCodec* pCodec = detectEncoding(buf, size, skipBytes);
         if(pCodec)
             return pCodec;
@@ -361,7 +353,7 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
         //Routine result of directory compare finding a file that isn't in all locations.
         if(!m_fileAccess.isValid()) return;
 
-        Q_ASSERT(!m_fileAccess.isDir());
+        assert(!m_fileAccess.isDir());
         if(!m_fileAccess.isNormal())
         {
             mErrors.append(i18n("%1 is not a normal file.", m_fileAccess.prettyAbsPath()));
@@ -523,7 +515,7 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
                 m_pOptions->m_LineMatchingPreProcessorCmd = "";
                 if(!m_lmppData.readFile(fileNameIn2))
                 {
-                    mErrors.append(i18n("Failed to read file: %1", fileNameIn2));
+                    mErrors.append(i18nc("Read error message. %1 = filepath", "Failed to read file: %1", fileNameIn2));
                     return;
                 }
             }
@@ -542,18 +534,18 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
 
     if(!m_lmppData.preprocess(pEncoding2, true))
     {
-        mErrors.append(i18n("File %1 too large to process. Skipping.", fileNameIn1));
+        mErrors.append(i18nc("Read error message. %1 = filepath", "File %1 too large to process. Skipping.", fileNameIn1));
         return;
     }
 
-    Q_ASSERT(m_lmppData.isText());
+    assert(m_lmppData.isText());
     //TODO: Needed?
     if(m_lmppData.lineCount() < m_normalData.lineCount())
     {
         // Preprocessing command may result in smaller data buffer so adjust size
         for(qint64 i = m_lmppData.lineCount(); i < m_normalData.lineCount(); ++i)
         { // Set all empty lines to point to the end of the buffer.
-            m_lmppData.m_v.push_back(LineData(m_lmppData.m_unicodeBuf, m_lmppData.m_unicodeBuf->length()));
+            m_lmppData.m_v->push_back(LineData(m_lmppData.m_unicodeBuf, m_lmppData.m_unicodeBuf->length()));
         }
 
         m_lmppData.mLineCount = m_normalData.lineCount();
@@ -563,13 +555,13 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
     if(m_pOptions->ignoreComments() && hasData())
     {
         qint64 vSize = std::min(m_normalData.lineCount(), m_lmppData.lineCount());
-        Q_ASSERT(vSize < TYPE_MAX(qint32));
+        assert(vSize < TYPE_MAX(qint32));
         //Perform explcit cast to insure well defined behavior comparing 32-bit to a 64-bit value
         for(qint32 i = 0; (qint64)i < vSize; ++i)
         {
             //TODO: Phase this out. We should not be messing with these flags outside the parser.
-            m_normalData.m_v[i].setPureComment(m_lmppData.m_v[i].isPureComment());
-            m_normalData.m_v[i].setSkipable(m_lmppData.m_v[i].isSkipable());
+            (*m_normalData.m_v)[i].setPureComment((*m_lmppData.m_v)[i].isPureComment());
+            (*m_normalData.m_v)[i].setSkipable((*m_lmppData.m_v)[i].isSkipable());
         }
     }
 }
@@ -586,37 +578,39 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
     QString line;
     QChar curChar, prevChar = 0;
     LineCount lineCount = 0;
-    qint64 lastOffset = 0;
-    qint64 skipBytes = 0;
+    QtSizeType lastOffset = 0;
+    FileOffset skipBytes = 0;
     QScopedPointer<CommentParser> parser(new DefaultCommentParser());
 
     // detect line end style
     QVector<e_LineEndStyle> vOrigDataLineEndStyle;
     m_eLineEndStyle = eLineEndStyleUndefined;
 
-    QTextCodec* pCodec = detectEncoding(m_pBuf, mDataSize, skipBytes);
+    QTextCodec* pCodec = detectEncoding(m_pBuf.get(), mDataSize, skipBytes);
     if(pCodec != pEncoding)
         skipBytes = 0;
 
     if(mDataSize - skipBytes > TYPE_MAX(QtNumberType))
         return false;
 
-    const QByteArray ba = QByteArray::fromRawData(m_pBuf + skipBytes, (int)(mDataSize - skipBytes));
-    QTextStream ts(ba, QIODevice::ReadOnly); //Don't use text mode we need to see the actual line ending.
+    const QByteArray ba = QByteArray::fromRawData(m_pBuf.get() + skipBytes, (QtSizeType)(mDataSize - skipBytes));
+    QTextStream ts(ba, QIODevice::ReadOnly); // Don't use text mode we need to see the actual line ending.
     ts.setCodec(pEncoding);
     ts.setAutoDetectUnicode(false);
 
     m_bIncompleteConversion = false;
     m_unicodeBuf->clear();
-    Q_ASSERT(m_unicodeBuf->length() == 0);
 
+    assert(m_unicodeBuf->length() == 0);
+
+    mHasEOLTermination = false;
     bool skipNextRead = false;
     while(!ts.atEnd())
     {
         line.clear();
         if(lineCount >= TYPE_MAX(LineCount) - 5)
         {
-            m_v.clear();
+            m_v->clear();
             return false;
         }
 
@@ -627,15 +621,15 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
         else
             skipNextRead = false;
 
-        quint32 firstNonwhite = 0;
-        bool    foundNonWhite = false;
+        QtSizeType  firstNonwhite = 0;
+        bool        foundNonWhite = false;
 
         //QTextStream::readLine doesn't tell us about line endings.
         while(curChar != '\n' && curChar != '\r')
         {
             if(curChar.isNull() || curChar.isNonCharacter())
             {
-                m_v.clear();
+                m_v->clear();
                 return true;
             }
 
@@ -656,14 +650,13 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
             curChar = ts.read(1).unicode()[0];
         }
 
-
         switch(curChar.unicode())
         {
             case '\n':
                 vOrigDataLineEndStyle.push_back(eLineEndStyleUnix);
                 break;
             case '\r':
-                if(lastOffset < mDataSize)
+                if((FileOffset)lastOffset < mDataSize)
                 {
                     prevChar = curChar;
                     curChar = ts.read(1).unicode()[0];
@@ -686,7 +679,7 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
             parser->removeComment(line);
 
         ++lineCount;
-        m_v.push_back(LineData(m_unicodeBuf, lastOffset, line.length(), firstNonwhite, parser->isSkipable(), parser->isPureComment()));
+        m_v->push_back(LineData(m_unicodeBuf, lastOffset, line.length(), firstNonwhite, parser->isSkipable(), parser->isPureComment()));
         //The last line may not have an EOL mark. In that case don't add one to our buffer.
         m_unicodeBuf->append(line);
         if(curChar == '\n' || curChar == '\r')
@@ -697,21 +690,22 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
 
         lastOffset = m_unicodeBuf->length();
     }
+
     /*
         Process trailing new line as if there were a blank non-terminated line after it.
-        But do nothing to the data buffer since this a phantom line needed for internal purposes.
+        But do nothing to the data buffer since this is a phantom line needed for internal purposes.
     */
     if(curChar == '\n' || curChar == '\r')
     {
+        mHasEOLTermination = true;
         ++lineCount;
 
         parser->processLine("");
-        m_v.push_back(LineData(m_unicodeBuf, lastOffset, 0, 0, parser->isSkipable(), parser->isPureComment()));
+        m_v->push_back(LineData(m_unicodeBuf, lastOffset, 0, 0, parser->isSkipable(), parser->isPureComment()));
     }
-    //Check if we have two identical offsets at the end. Indicates a bug in the read loop.
-    Q_ASSERT(m_v.size() < 2 || m_v[m_v.size() - 1].getOffset() != m_v[m_v.size() - 2].getOffset());
+    assert(m_v->size() < 2 || (*m_v)[m_v->size() - 1].getOffset() != (*m_v)[m_v->size() - 2].getOffset());
 
-    m_v.push_back(LineData(m_unicodeBuf, lastOffset));
+    m_v->push_back(LineData(m_unicodeBuf, lastOffset));
 
     m_bIsText = true;
 
@@ -747,11 +741,11 @@ bool SourceData::convertFileEncoding(const QString& fileNameIn, QTextCodec* pCod
 
 QTextCodec* SourceData::getEncodingFromTag(const QByteArray& s, const QByteArray& encodingTag)
 {
-    int encodingPos = s.indexOf(encodingTag);
+    QtSizeType encodingPos = s.indexOf(encodingTag);
     if(encodingPos >= 0)
     {
-        int apostrophPos = s.indexOf('"', encodingPos + encodingTag.length());
-        int apostroph2Pos = s.indexOf('\'', encodingPos + encodingTag.length());
+        QtSizeType apostrophPos = s.indexOf('"', encodingPos + encodingTag.length());
+        QtSizeType apostroph2Pos = s.indexOf('\'', encodingPos + encodingTag.length());
         char apostroph = '"';
         if(apostroph2Pos >= 0 && (apostrophPos < 0 || apostroph2Pos < apostrophPos))
         {
@@ -759,7 +753,7 @@ QTextCodec* SourceData::getEncodingFromTag(const QByteArray& s, const QByteArray
             apostrophPos = apostroph2Pos;
         }
 
-        int encodingEnd = s.indexOf(apostroph, apostrophPos + 1);
+        QtSizeType encodingEnd = s.indexOf(apostroph, apostrophPos + 1);
         if(encodingEnd >= 0) // e.g.: <meta charset="utf-8"> or <?xml version="1.0" encoding="ISO-8859-1"?>
         {
             QByteArray encoding = s.mid(apostrophPos + 1, encodingEnd - (apostrophPos + 1));
@@ -774,7 +768,7 @@ QTextCodec* SourceData::getEncodingFromTag(const QByteArray& s, const QByteArray
     return nullptr;
 }
 
-QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, qint64& skipBytes)
+QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, FileOffset& skipBytes)
 {
     if(size >= 2)
     {
@@ -805,14 +799,14 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, qint64& ski
         We don't need the whole file here just the header.
     */
     if(size <= 5000)
-        s = QByteArray(buf, (int)size);
+        s = QByteArray(buf, (QtSizeType)size);
     else
         s = QByteArray(buf, 5000);
 
-    int xmlHeaderPos = s.indexOf("<?xml");
+    QtSizeType xmlHeaderPos = s.indexOf("<?xml");
     if(xmlHeaderPos >= 0)
     {
-        int xmlHeaderEnd = s.indexOf("?>", xmlHeaderPos);
+        QtSizeType xmlHeaderEnd = s.indexOf("?>", xmlHeaderPos);
         if(xmlHeaderEnd >= 0)
         {
             QTextCodec* pCodec = getEncodingFromTag(s.mid(xmlHeaderPos, xmlHeaderEnd - xmlHeaderPos), "encoding=");
@@ -822,10 +816,10 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, qint64& ski
     }
     else // HTML
     {
-        int metaHeaderPos = s.indexOf("<meta");
+        QtSizeType metaHeaderPos = s.indexOf("<meta");
         while(metaHeaderPos >= 0)
         {
-            int metaHeaderEnd = s.indexOf(">", metaHeaderPos);
+            QtSizeType metaHeaderEnd = s.indexOf(">", metaHeaderPos);
             if(metaHeaderEnd >= 0)
             {
                 QTextCodec* pCodec = getEncodingFromTag(s.mid(metaHeaderPos, metaHeaderEnd - metaHeaderPos), "charset=");
@@ -847,7 +841,8 @@ QTextCodec* SourceData::dectectUTF8(const QByteArray& data)
     QTextCodec* utf8 = QTextCodec::codecForName("UTF-8");
 
     QTextCodec::ConverterState state;
-    const QString validText = utf8->toUnicode(data.constData(), data.size(), &state);
+    utf8->toUnicode(data.constData(), data.size(), &state);
+
     if(state.invalidChars == 0)
         return utf8;
 
