@@ -5,15 +5,22 @@
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
 */
+// clang-format on
 
 #include "common.h"
+#include "TypeUtils.h"
+
+#include <map>
+#include <utility>            // for pair
+
 #include <QColor>
+#include <QFont>
+#include <QPoint>
+#include <QLatin1String>
 #include <QSize>
 #include <QStringList>
+#include <QStringLiteral>
 #include <QTextStream>
-#include <map>
-#include <qfont.h>
-#include <qpoint.h>
 
 ValueMap::ValueMap() = default;
 
@@ -47,7 +54,7 @@ void ValueMap::load(QTextStream& ts)
     while(!ts.atEnd())
     {                              // until end of file...
         QString s = ts.readLine(); // line of text excluding '\n'
-        int pos = s.indexOf('=');
+        QtSizeType pos = s.indexOf('=');
         if(pos > 0) // seems not to have a tag
         {
             QString key = s.left(pos);
@@ -65,7 +72,7 @@ QString safeStringJoin(const QStringList& sl, char sepChar, char metaChar)
     // If a string contains the separator character, it will be replaced with "\,".
     // Any occurrences of "\" (one backslash) will be replaced with "\\" (2 backslashes)
 
-    Q_ASSERT(sepChar != metaChar);
+    assert(sepChar != metaChar);
 
     QString sep;
     sep += sepChar;
@@ -91,11 +98,11 @@ QString safeStringJoin(const QStringList& sl, char sepChar, char metaChar)
 // Split a string that was joined with safeStringJoin
 QStringList safeStringSplit(const QString& s, char sepChar, char metaChar)
 {
-    Q_ASSERT(sepChar != metaChar);
+    assert(sepChar != metaChar);
     QStringList sl;
     // Miniparser
-    int i = 0;
-    int len = s.length();
+    QtSizeType i = 0;
+    QtSizeType len = s.length();
     QString b;
     for(i = 0; i < len; ++i)
     {
@@ -127,22 +134,22 @@ QStringList safeStringSplit(const QString& s, char sepChar, char metaChar)
 
 void ValueMap::writeEntry(const QString& k, const QFont& v)
 {
-    m_map[k] = v.family() + QLatin1String(",") + QString::number(v.pointSize()) + QLatin1String(",") + (v.bold() ? QLatin1String("bold") : QLatin1String("normal"));
+    m_map[k] = v.family() + u8"," + QString::number(v.pointSize()) + u8"," + (v.bold() ? QStringLiteral("bold") : QStringLiteral("normal"));
 }
 
 void ValueMap::writeEntry(const QString& k, const QColor& v)
 {
-    m_map[k].setNum(v.red()) + QLatin1String(",") + QString().setNum(v.green()) + QLatin1String(",") + QString().setNum(v.blue());
+    m_map[k].setNum(v.red()) + u8"," + QString().setNum(v.green()) + u8"," + QString().setNum(v.blue());
 }
 
 void ValueMap::writeEntry(const QString& k, const QSize& v)
 {
-    m_map[k].setNum(v.width()) + QLatin1String(",") + QString().setNum(v.height());
+    m_map[k].setNum(v.width()) + u8"," + QString().setNum(v.height());
 }
 
 void ValueMap::writeEntry(const QString& k, const QPoint& v)
 {
-    m_map[k].setNum(v.x()) + QLatin1String(",") + QString().setNum(v.y());
+    m_map[k].setNum(v.x()) + u8"," + QString().setNum(v.y());
 }
 
 void ValueMap::writeEntry(const QString& k, int v)
