@@ -1,19 +1,24 @@
 /*
  * KDiff3 - Text Diff And Merge Tool
- * 
+ *
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 #ifndef COMMON_H
 #define COMMON_H
 
-#include <QAtomicInt>
-#include <QString>
 #include <algorithm>
 #include <map>
 
+#include <QAtomicInt>
+#include <QString>
+
+#define KF_VERSION_CHECK(major, minor, patch) ((major<<16)|(minor<<8)|(patch))
+#define KF_VERSION KF_VERSION_CHECK(KF_VERSION_MAJOR, KF_VERSION_MINOR, KF_VERSION_PATCH)
+
 template <class T>
 T min3( T d1, T d2, T d3 )
 {
@@ -28,12 +33,20 @@ T max3( T d1, T d2, T d3 )
 
 inline int getAtomic(QAtomicInt& ai)
 {
-   return ai.load();
+#if (QT_VERSION < QT_VERSION_CHECK(5, 14, 0))
+    return ai.load();
+#else
+    return ai.loadRelaxed();
+#endif
 }
 
 inline qint64 getAtomic(QAtomicInteger<qint64>& ai)
 {
-   return ai.load();
+#if (QT_VERSION < QT_VERSION_CHECK(5, 14, 0))
+    return ai.load();
+#else
+    return ai.loadRelaxed();
+#endif
 }
 
 class QFont;
