@@ -1,3 +1,4 @@
+// clang-format off
 /*
  *  This file is part of KDiff3.
  *
@@ -5,31 +6,41 @@
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
 */
+// clang-format on
 
 #ifndef DIFF_H
 #define DIFF_H
 
 #include "common.h"
-#include "fileaccess.h"
 #include "LineRef.h"
-#include "SourceData.h"
 #include "Logging.h"
+#include "TypeUtils.h"
 
-#include <QList>
+#include <list>
+#include <memory>
+#include <vector>
+
+#include <QSharedPointer>
+#include <QString>
+#include <QStringList>
 #include <QVector>
 
+class LineData;
+
 class Options;
 
-//e_SrcSelector must be sequential with no gaps between Min and Max.
+using LineDataVector=std::vector<LineData>;
+
+//e_SrcSelector must be sequential integers with no gaps between Min and Max.
 enum class e_SrcSelector
 {
-   Min = -1,
-   Invalid=-1,
-   None=0,
-   A = 1,
-   B = 2,
-   C = 3,
-   Max=C
+    Min = -1,
+    Invalid = -1,
+    None = 0,
+    A = 1,
+    B = 2,
+    C = 3,
+    Max = C
 };
 
 inline e_SrcSelector nextSelector(e_SrcSelector selector)
@@ -45,28 +56,26 @@ inline e_SrcSelector nextSelector(e_SrcSelector selector)
         default:
             return e_SrcSelector::Invalid;
     }
-
-    return e_SrcSelector::Invalid;
 }
 
 enum class e_MergeDetails
 {
-   eDefault,
-   eNoChange,
-   eBChanged,
-   eCChanged,
-   eBCChanged,         // conflict
-   eBCChangedAndEqual, // possible conflict
-   eBDeleted,
-   eCDeleted,
-   eBCDeleted,         // possible conflict
+    eDefault,
+    eNoChange,
+    eBChanged,
+    eCChanged,
+    eBCChanged,         // conflict
+    eBCChangedAndEqual, // possible conflict
+    eBDeleted,
+    eCDeleted,
+    eBCDeleted,         // possible conflict
 
-   eBChanged_CDeleted, // conflict
-   eCChanged_BDeleted, // conflict
-   eBAdded,
-   eCAdded,
-   eBCAdded,           // conflict
-   eBCAddedAndEqual    // possible conflict
+    eBChanged_CDeleted, // conflict
+    eCChanged_BDeleted, // conflict
+    eBAdded,
+    eCAdded,
+    eBCAdded,           // conflict
+    eBCAddedAndEqual    // possible conflict
 };
 
 enum ChangeFlag
@@ -89,6 +98,7 @@ class Diff
 
     qint64 mDiff1 = 0;
     qint64 mDiff2 = 0;
+
   public:
     Diff() = default;//We use defualt initialization force compiler to generate a default constructor
     Diff(LineCount eq, const qint64 inDiff1, const qint64 inDiff2)
@@ -98,6 +108,8 @@ class Diff
         mDiff2 = inDiff2;
     }
 
+    [[nodiscard]] bool isEmpty() const { return nofEquals == 0 && mDiff1 == 0 && mDiff2 == 0;}
+
     Q_REQUIRED_RESULT inline qint32 numberOfEquals() const { return nofEquals; };
 
     Q_REQUIRED_RESULT inline qint64 diff1() const { return mDiff1; };
@@ -108,28 +120,41 @@ class Diff
     inline void adjustNumberOfEquals(const qint64 delta) { nofEquals += delta; }
     inline void adjustDiff1(const qint64 delta) { mDiff1 += delta; }
     inline void adjustDiff2(const qint64 delta) { mDiff2 += delta; }
+
+    bool operator==(const Diff& b) const
+    {
+        return nofEquals == b.nofEquals && mDiff1 == b.mDiff1 && mDiff2 == b.mDiff2;
+    };
+
+    bool operator!=(const Diff& b) const
+    {
+        return !(*this == b);
+    };
 };
 
-class DiffList : public std::list<Diff>
+class DiffList: public std::list<Diff>
 {
   public:
-    void runDiff(const QVector<LineData>* p1, const qint32 index1, LineRef size1, const QVector<LineData>* p2, const qint32 index2, LineRef size2, const QSharedPointer<Options>& pOptions);
+    using std::list<Diff>::list;
+    void calcDiff(const QString& line1, const QString& line2, const int maxSearchRange);
+    void runDiff(const std::shared_ptr<LineDataVector> &p1, const size_t index1, LineRef size1, const std::shared_ptr<LineDataVector> &p2, const size_t index2, LineRef size2, const QSharedPointer<Options>& pOptions);
+    void verify(const LineRef size1, const LineRef size2);
 };
 
 class LineData
 {
   private:
     QSharedPointer<QString> mBuffer;
-    //QString pLine;
-    QtNumberType mFirstNonWhiteChar = 0;
-    qint64 mOffset = 0;
-    QtNumberType mSize = 0;
+    QtSizeType mFirstNonWhiteChar = 0;
+    //This tracks the offset with-in our unicode buffer not the file offset
+    QtSizeType mOffset = 0;
+    QtSizeType mSize = 0;
     bool bContainsPureComment = false;
     bool bSkipable = false;//TODO: Move me
 
   public:
     explicit LineData() = default; // needed for Qt internal reasons should not be used.
-    inline LineData(const QSharedPointer<QString> &buffer, const qint64 inOffset, QtNumberType inSize = 0, QtNumberType inFirstNonWhiteChar=0, bool inIsSkipable=false,const bool inIsPureComment = false)
+    inline LineData(const QSharedPointer<QString>& buffer, const QtSizeType inOffset, QtSizeType inSize = 0, QtSizeType inFirstNonWhiteChar = 0, bool inIsSkipable = false, const bool inIsPureComment = false)
     {
         mBuffer = buffer;
         mOffset = inOffset;
@@ -138,7 +163,7 @@ class LineData
         bSkipable = inIsSkipable;
         mFirstNonWhiteChar = inFirstNonWhiteChar;
     }
-    Q_REQUIRED_RESULT inline int size() const { return mSize; }
+    Q_REQUIRED_RESULT inline QtSizeType size() const { return mSize; }
     Q_REQUIRED_RESULT inline qint32 getFirstNonWhiteChar() const { return mFirstNonWhiteChar; }
 
     /*
@@ -147,9 +172,9 @@ class LineData
     Q_REQUIRED_RESULT inline const QString getLine() const { return QString::fromRawData(mBuffer->data() + mOffset, mSize); }
     Q_REQUIRED_RESULT inline const QSharedPointer<QString>& getBuffer() const { return mBuffer; }
 
-    Q_REQUIRED_RESULT inline qint64 getOffset() const { return mOffset; }
+    Q_REQUIRED_RESULT inline QtSizeType getOffset() const { return mOffset; }
     Q_REQUIRED_RESULT int width(int tabSize) const; // Calcs width considering tabs.
-    //int occurrences;
+
     Q_REQUIRED_RESULT inline bool whiteLine() const { return mFirstNonWhiteChar == 0; }
 
     Q_REQUIRED_RESULT inline bool isPureComment() const { return bContainsPureComment; }
@@ -163,26 +188,24 @@ class LineData
 
 class ManualDiffHelpList; // A list of corresponding ranges
 
+class Diff3Line;
 class Diff3LineList;
-class Diff3LineVector;
+
+using Diff3LineVector = QVector<Diff3Line*>;
 
 class DiffBufferInfo
 {
   private:
-    const QVector<LineData>* mLineDataA;
-    const QVector<LineData>* mLineDataB;
-    const QVector<LineData>* mLineDataC;
+    std::shared_ptr<LineDataVector> mLineDataA;
+    std::shared_ptr<LineDataVector> mLineDataB;
+    std::shared_ptr<LineDataVector> mLineDataC;
+    const Diff3LineList* m_pDiff3LineList = nullptr;
 
-    LineCount m_sizeA;
-    LineCount m_sizeB;
-    LineCount m_sizeC;
-    const Diff3LineList* m_pDiff3LineList;
-    const Diff3LineVector* m_pDiff3LineVector;
-public:
-    void init(Diff3LineList* d3ll, const Diff3LineVector* d3lv,
-              const QVector<LineData>* pldA, LineCount sizeA, const QVector<LineData>* pldB, LineCount sizeB, const QVector<LineData>* pldC, LineCount sizeC);
+  public:
+    void init(Diff3LineList* d3ll,
+              const std::shared_ptr<LineDataVector> &pldA, const std::shared_ptr<LineDataVector> &pldB, const std::shared_ptr<LineDataVector> &pldC);
 
-    inline const QVector<LineData>* getLineData(e_SrcSelector srcIndex) const
+    [[nodiscard]] inline std::shared_ptr<LineDataVector> getLineData(e_SrcSelector srcIndex) const
     {
         switch(srcIndex)
         {
@@ -201,8 +224,8 @@ public:
 enum class IgnoreFlag
 {
     none = 0,
-    ignoreWhiteSpace,
-    ignoreComments
+    ignoreWhiteSpace = 1 << 1,
+    ignoreComments = 1 << 2,
 };
 
 Q_DECLARE_FLAGS(IgnoreFlags, IgnoreFlag);
@@ -210,7 +233,7 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(IgnoreFlags);
 
 class Diff3Line
 {
-  private:
+  protected:
     friend class Diff3LineList;
     LineRef lineA;
     LineRef lineB;
@@ -224,32 +247,35 @@ class Diff3Line
     bool bWhiteLineB = false;
     bool bWhiteLineC = false;
 
-    bool bIsPureCommentA = false;
-    bool bIsPureCommentB = false;
-    bool bIsPureCommentC = false;
-
-    DiffList* pFineAB = nullptr; // These are NULL only if completely equal or if either source doesn't exist.
-    DiffList* pFineBC = nullptr;
-    DiffList* pFineCA = nullptr;
-
+    std::shared_ptr<DiffList> pFineAB; // These are NULL only if completely equal or if either source doesn't exist.
+    std::shared_ptr<DiffList> pFineBC;
+    std::shared_ptr<DiffList> pFineCA;
 
     qint32 mLinesNeededForDisplay = 1;    // Due to wordwrap
     qint32 mSumLinesNeededForDisplay = 0; // For fast conversion to m_diff3WrapLineVector
-
   public:
-    static QSharedPointer<DiffBufferInfo> m_pDiffBufferInfo; // For convenience
+    static QSharedPointer<DiffBufferInfo> m_pDiffBufferInfo; // Needed by this class and only this but inited directly from KDiff3App::mainInit
 
-    ~Diff3Line()
+    [[nodiscard]] inline bool hasFineDiffAB() const { return pFineAB != nullptr; }
+    [[nodiscard]] inline bool hasFineDiffBC() const { return pFineBC != nullptr; }
+    [[nodiscard]] inline bool hasFineDiffCA() const { return pFineCA != nullptr; }
+
+    [[nodiscard]] inline LineRef getLineIndex(e_SrcSelector src) const
     {
-        delete pFineAB;
-        delete pFineBC;
-        delete pFineCA;
-
-        pFineAB = nullptr;
-        pFineBC = nullptr;
-        pFineCA = nullptr;
+        switch(src)
+        {
+            case e_SrcSelector::A:
+                return getLineA();
+            case e_SrcSelector::B:
+                return getLineB();
+            case e_SrcSelector::C:
+                return getLineC();
+            default:
+                assert(false);
+                return LineRef::invalid;
+        }
     }
-    
+
     Q_REQUIRED_RESULT LineRef getLineA() const { return lineA; }
     Q_REQUIRED_RESULT LineRef getLineB() const { return lineB; }
     Q_REQUIRED_RESULT LineRef getLineC() const { return lineC; }
@@ -264,7 +290,7 @@ class Diff3Line
 
     Q_REQUIRED_RESULT inline bool isWhiteLine(e_SrcSelector src) const
     {
-        Q_ASSERT(src == e_SrcSelector::A || src == e_SrcSelector::B || src == e_SrcSelector::C);
+        assert(src == e_SrcSelector::A || src == e_SrcSelector::B || src == e_SrcSelector::C);
 
         switch(src)
         {
@@ -276,33 +302,32 @@ class Diff3Line
                 return bWhiteLineC;
             default:
                 //should never get here
-                Q_ASSERT(false);
+                assert(false);
                 return false;
         }
     }
 
     bool operator==(const Diff3Line& d3l) const
     {
-        return lineA == d3l.lineA && lineB == d3l.lineB && lineC == d3l.lineC && bAEqB == d3l.bAEqB && bAEqC == d3l.bAEqC && bBEqC == d3l.bBEqC;
+        return lineA == d3l.lineA && lineB == d3l.lineB && lineC == d3l.lineC &&
+               bAEqB == d3l.bAEqB && bAEqC == d3l.bAEqC && bBEqC == d3l.bBEqC &&
+               bWhiteLineA == d3l.bWhiteLineA && bWhiteLineB == d3l.bWhiteLineB && bWhiteLineC == d3l.bWhiteLineC;
     }
 
-    Q_REQUIRED_RESULT const LineData* getLineData(e_SrcSelector src) const
+    Q_REQUIRED_RESULT const LineData& getLineData(e_SrcSelector src) const
     {
-        Q_ASSERT(m_pDiffBufferInfo != nullptr);
+        assert(m_pDiffBufferInfo != nullptr);
+        assert(src == e_SrcSelector::A || src == e_SrcSelector::B || src == e_SrcSelector::C);
         //Use at() here not [] to avoid using really weird syntax
-        if(src == e_SrcSelector::A && lineA.isValid()) return &m_pDiffBufferInfo->getLineData(src)->at(lineA);
-        if(src == e_SrcSelector::B && lineB.isValid()) return &m_pDiffBufferInfo->getLineData(src)->at(lineB);
-        if(src == e_SrcSelector::C && lineC.isValid()) return &m_pDiffBufferInfo->getLineData(src)->at(lineC);
-
-        return nullptr;
+        if(src == e_SrcSelector::A && lineA.isValid()) return m_pDiffBufferInfo->getLineData(src)->at(lineA);
+        if(src == e_SrcSelector::B && lineB.isValid()) return m_pDiffBufferInfo->getLineData(src)->at(lineB);
+        return m_pDiffBufferInfo->getLineData(src)->at(lineC);
     }
     Q_REQUIRED_RESULT const QString getString(const e_SrcSelector src) const
     {
-        const LineData* pld = getLineData(src);
-        if(pld)
-            return pld->getLine();
-        else
-            return QString();
+        const LineData& pld = getLineData(src);
+
+        return pld.getLine();
     }
     Q_REQUIRED_RESULT LineRef getLineInFile(e_SrcSelector src) const
     {
@@ -317,42 +342,41 @@ class Diff3Line
     Q_REQUIRED_RESULT inline qint32 linesNeededForDisplay() const { return mLinesNeededForDisplay; }
 
     void setLinesNeeded(const qint32 lines) { mLinesNeededForDisplay = lines; }
-    Q_REQUIRED_RESULT bool fineDiff(bool bTextsTotalEqual, const e_SrcSelector selector, const QVector<LineData>* v1, const QVector<LineData>* v2, const IgnoreFlags eIgnoreFlags);
-    void mergeOneLine(e_MergeDetails& mergeDetails, bool& bConflict, bool& bLineRemoved, e_SrcSelector& src, bool bTwoInputs) const;
-
+    Q_REQUIRED_RESULT bool fineDiff(bool bTextsTotalEqual, const e_SrcSelector selector, const std::shared_ptr<LineDataVector> &v1, const std::shared_ptr<LineDataVector> &v2, const IgnoreFlags eIgnoreFlags);
     void getLineInfo(const e_SrcSelector winIdx, const bool isTriple, LineRef& lineIdx,
-        DiffList*& pFineDiff1, DiffList*& pFineDiff2, // return values
-        ChangeFlags& changed, ChangeFlags& changed2) const;
+                     std::shared_ptr<DiffList>& pFineDiff1, std::shared_ptr<DiffList>& pFineDiff2, // return values
+                     ChangeFlags& changed, ChangeFlags& changed2) const;
+
   private:
-    void setFineDiff(const e_SrcSelector selector, DiffList* pDiffList)
+    void setFineDiff(const e_SrcSelector selector, std::shared_ptr<DiffList>& pDiffList)
     {
-        Q_ASSERT(selector == e_SrcSelector::A || selector == e_SrcSelector::B || selector == e_SrcSelector::C);
+        assert(selector == e_SrcSelector::A || selector == e_SrcSelector::B || selector == e_SrcSelector::C);
         if(selector == e_SrcSelector::A)
         {
-            delete pFineAB;
             pFineAB = pDiffList;
         }
         else if(selector == e_SrcSelector::B)
         {
-            delete pFineBC;
             pFineBC = pDiffList;
         }
         else if(selector == e_SrcSelector::C)
         {
-            delete pFineCA;
             pFineCA = pDiffList;
         }
     }
 };
 
-class Diff3LineList : public std::list<Diff3Line>
+struct HistoryRange;
+
+class Diff3LineList: public std::list<Diff3Line>
 {
   public:
-    void findHistoryRange(const QRegExp& historyStart, bool bThreeFiles,
-                             Diff3LineList::const_iterator& iBegin, Diff3LineList::const_iterator& iEnd, int& idxBegin, int& idxEnd) const;
-    bool fineDiff(const e_SrcSelector selector, const QVector<LineData>* v1, const QVector<LineData>* v2, const IgnoreFlags eIgnoreFlags);
+    using std::list<Diff3Line>::list;
+
+    void findHistoryRange(const QRegularExpression& historyStart, bool bThreeFiles, HistoryRange& range) const;
+    bool fineDiff(const e_SrcSelector selector, const std::shared_ptr<LineDataVector> &v1, const std::shared_ptr<LineDataVector> &v2, const IgnoreFlags eIgnoreFlags);
     void calcDiff3LineVector(Diff3LineVector& d3lv);
-    void calcWhiteDiff3Lines(const QVector<LineData>* pldA, const QVector<LineData>* pldB, const QVector<LineData>* pldC, const bool bIgnoreComments);
+    void calcWhiteDiff3Lines(const std::shared_ptr<LineDataVector> &pldA, const std::shared_ptr<LineDataVector> &pldB, const std::shared_ptr<LineDataVector> &pldC, const bool bIgnoreComments);
 
     void calcDiff3LineListUsingAB(const DiffList* pDiffListAB);
     void calcDiff3LineListUsingAC(const DiffList* pDiffListAC);
@@ -360,8 +384,7 @@ class Diff3LineList : public std::list<Diff3Line>
 
     void correctManualDiffAlignment(ManualDiffHelpList* pManualDiffHelpList);
 
-    void calcDiff3LineListTrim(const QVector<LineData>* pldA, const QVector<LineData>* pldB, const QVector<LineData>* pldC, ManualDiffHelpList* pManualDiffHelpList);
-
+    void calcDiff3LineListTrim(const std::shared_ptr<LineDataVector> &pldA, const std::shared_ptr<LineDataVector> &pldB, const std::shared_ptr<LineDataVector> &pldC, ManualDiffHelpList* pManualDiffHelpList);
 
     LineCount recalcWordWrap(bool resetDisplayCount)
     {
@@ -379,25 +402,15 @@ class Diff3LineList : public std::list<Diff3Line>
         return sumOfLines;
     }
 
-    //TODO: Add safety guards to prevent list from getting too large. Same problem as with QLinkedList.
-    qint32 size() const
-    {
-        if(std::list<Diff3Line>::size() > (size_t)TYPE_MAX(qint32))//explicit cast to silence gcc
-        {
-            qCDebug(kdiffMain) << "Diff3Line: List too large. size=" << std::list<Diff3Line>::size();
-            Q_ASSERT(false); //Unsupported size
-            return 0;
-        }
-        return (qint32)std::list<Diff3Line>::size();
-    } //safe for small files same limit as exited with QLinkedList. This should ultimatly be removed.
-
     void debugLineCheck(const LineCount size, const e_SrcSelector srcSelector) const;
 
-    qint32 numberOfLines(bool bWordWrap) const
+    void dump();
+
+    [[nodiscard]] LineCount numberOfLines(bool bWordWrap) const
     {
         if(bWordWrap)
         {
-            qint32 lines;
+            LineCount lines;
 
             lines = 0;
             Diff3LineList::const_iterator i;
@@ -410,13 +423,17 @@ class Diff3LineList : public std::list<Diff3Line>
         }
         else
         {
-            return size();
+            return SafeInt<LineCount>(size());
         }
     }
 };
 
-class Diff3LineVector : public QVector<Diff3Line*>
+struct HistoryRange
 {
+    Diff3LineList::const_iterator start;
+    Diff3LineList::const_iterator end;
+    int startIdx = -1;
+    int endIdx = -1;
 };
 
 struct Diff3WrapLine
@@ -445,30 +462,28 @@ class TotalDiffStatus
         nofWhitespaceConflicts = 0;
     }
 
-    inline int getUnsolvedConflicts() const { return nofUnsolvedConflicts; }
+    [[nodiscard]] inline int getUnsolvedConflicts() const { return nofUnsolvedConflicts; }
     inline void setUnsolvedConflicts(const int unsolved) { nofUnsolvedConflicts = unsolved; }
 
-    inline int getSolvedConflicts() const { return nofSolvedConflicts; }
+    [[nodiscard]] inline int getSolvedConflicts() const { return nofSolvedConflicts; }
     inline void setSolvedConflicts(const int solved) { nofSolvedConflicts = solved; }
 
-    inline int getWhitespaceConflicts() const { return nofWhitespaceConflicts; }
+    [[nodiscard]] inline int getWhitespaceConflicts() const { return nofWhitespaceConflicts; }
     inline void setWhitespaceConflicts(const int wintespace) { nofWhitespaceConflicts = wintespace; }
 
-    inline int getNonWhitespaceConflicts() const { return getUnsolvedConflicts() + getSolvedConflicts() - getWhitespaceConflicts(); }
+    [[nodiscard]] inline int getNonWhitespaceConflicts() const { return getUnsolvedConflicts() + getSolvedConflicts() - getWhitespaceConflicts(); }
 
-    bool isBinaryEqualAC() const { return bBinaryAEqC; }
-    bool isBinaryEqualBC() const { return bBinaryBEqC; }
-    bool isBinaryEqualAB() const { return bBinaryAEqB; }
+    [[nodiscard]] bool isBinaryEqualAC() const { return bBinaryAEqC; }
+    [[nodiscard]] bool isBinaryEqualBC() const { return bBinaryBEqC; }
+    [[nodiscard]] bool isBinaryEqualAB() const { return bBinaryAEqB; }
 
     void setBinaryEqualAC(const bool equal) { bBinaryAEqC = equal; }
     void setBinaryEqualBC(const bool equal) { bBinaryBEqC = equal; }
     void setBinaryEqualAB(const bool equal) { bBinaryAEqB = equal; }
 
-
-    bool isTextEqualAC() const { return bTextAEqC; }
-    bool isTextEqualBC() const { return bTextBEqC; }
-    bool isTextEqualAB() const { return bTextAEqB; }
-
+    [[nodiscard]] bool isTextEqualAC() const { return bTextAEqC; }
+    [[nodiscard]] bool isTextEqualBC() const { return bTextBEqC; }
+    [[nodiscard]] bool isTextEqualAB() const { return bTextAEqB; }
 
     void setTextEqualAC(const bool equal) { bTextAEqC = equal; }
     void setTextEqualBC(const bool equal) { bTextBEqC = equal; }
@@ -484,7 +499,7 @@ class TotalDiffStatus
     bool bTextAEqB = false;
     int nofUnsolvedConflicts = 0;
     int nofSolvedConflicts = 0;
-    int nofWhitespaceConflicts = 0;
+    int nofWhitespaceConflicts = -1;
 };
 
 // Three corresponding ranges. (Minimum size of a valid range is one line.)
@@ -499,6 +514,26 @@ class ManualDiffHelpEntry
     LineRef lineC2;
 
   public:
+    ManualDiffHelpEntry() = default;
+    ManualDiffHelpEntry(e_SrcSelector winIdx, LineRef firstLine, LineRef lastLine)
+    {
+        if(winIdx == e_SrcSelector::A)
+        {
+            lineA1 = firstLine;
+            lineA2 = lastLine;
+        }
+        else if(winIdx == e_SrcSelector::B)
+        {
+            lineB1 = firstLine;
+            lineB2 = lastLine;
+        }
+        else
+        {
+            lineC1 = firstLine;
+            lineC2 = lastLine;
+        }
+    }
+
     LineRef& firstLine(e_SrcSelector winIdx)
     {
         return winIdx == e_SrcSelector::A ? lineA1 : (winIdx == e_SrcSelector::B ? lineB1 : lineC1);
@@ -519,40 +554,42 @@ class ManualDiffHelpEntry
 
     int calcManualDiffFirstDiff3LineIdx(const Diff3LineVector& d3lv);
 
-    void getRangeForUI(const e_SrcSelector winIdx, LineRef *rangeLine1, LineRef *rangeLine2) const {
-        if(winIdx == e_SrcSelector::A) {
+    void getRangeForUI(const e_SrcSelector winIdx, LineRef* rangeLine1, LineRef* rangeLine2) const
+    {
+        if(winIdx == e_SrcSelector::A)
+        {
             *rangeLine1 = lineA1;
             *rangeLine2 = lineA2;
         }
-        if(winIdx == e_SrcSelector::B) {
+        if(winIdx == e_SrcSelector::B)
+        {
             *rangeLine1 = lineB1;
             *rangeLine2 = lineB2;
         }
-        if(winIdx == e_SrcSelector::C) {
+        if(winIdx == e_SrcSelector::C)
+        {
             *rangeLine1 = lineC1;
             *rangeLine2 = lineC2;
         }
     }
 
-    inline const LineRef& getLine1(const e_SrcSelector winIdx) const { return winIdx == e_SrcSelector::A ? lineA1 : winIdx == e_SrcSelector::B ? lineB1 : lineC1;}
-    inline const LineRef& getLine2(const e_SrcSelector winIdx) const { return winIdx == e_SrcSelector::A ? lineA2 : winIdx == e_SrcSelector::B ? lineB2 : lineC2;}
-    bool isValidMove(int line1, int line2, e_SrcSelector winIdx1, e_SrcSelector winIdx2) const;
+    [[nodiscard]] inline const LineRef& getLine1(const e_SrcSelector winIdx) const { return winIdx == e_SrcSelector::A ? lineA1 : winIdx == e_SrcSelector::B ? lineB1 : lineC1;}
+    [[nodiscard]] inline const LineRef& getLine2(const e_SrcSelector winIdx) const { return winIdx == e_SrcSelector::A ? lineA2 : winIdx == e_SrcSelector::B ? lineB2 : lineC2;}
+    [[nodiscard]] bool isValidMove(LineRef line1, LineRef line2, e_SrcSelector winIdx1, e_SrcSelector winIdx2) const;
 };
 
 // A list of corresponding ranges
 class ManualDiffHelpList: public std::list<ManualDiffHelpEntry>
 {
-    public:
-        Q_REQUIRED_RESULT bool isValidMove(int line1, int line2, e_SrcSelector winIdx1, e_SrcSelector winIdx2) const;
-        void insertEntry(e_SrcSelector winIdx, LineRef firstLine, LineRef lastLine);
+  public:
+    Q_REQUIRED_RESULT bool isValidMove(LineRef line1, LineRef line2, e_SrcSelector winIdx1, e_SrcSelector winIdx2) const;
+    void insertEntry(e_SrcSelector winIdx, LineRef firstLine, LineRef lastLine);
 
-        void runDiff(const QVector<LineData>* p1, LineRef size1, const QVector<LineData>* p2, LineRef size2, DiffList& diffList,
-                     e_SrcSelector winIdx1, e_SrcSelector winIdx2,
-                     const QSharedPointer<Options> &pOptions);
+    void runDiff(const std::shared_ptr<LineDataVector> &p1, LineRef size1, const std::shared_ptr<LineDataVector> &p2, LineRef size2, DiffList& diffList,
+                 e_SrcSelector winIdx1, e_SrcSelector winIdx2,
+                 const QSharedPointer<Options>& pOptions);
 };
 
-void calcDiff(const QString &line1, const QString &line2, DiffList& diffList, int match, int maxSearchRange);
-
 /** Returns the number of equivalent spaces at position outPos.
 */
 inline int tabber(int outPos, int tabSize)
@@ -573,6 +610,6 @@ enum e_CoordType
     eWrapCoords
 };
 
-QString calcHistorySortKey(const QString& keyOrder, QRegExp& matchedRegExpr, const QStringList& parenthesesGroupList);
+QString calcHistorySortKey(const QString& keyOrder, QRegularExpressionMatch& matchedRegExpr, const QStringList& parenthesesGroupList);
 bool findParenthesesGroups(const QString& s, QStringList& sl);
 #endif
