@@ -4,19 +4,28 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
-
+ */
+// clang-format on
 
 #ifndef DIFFTEXTWINDOW_H
 #define DIFFTEXTWINDOW_H
 
+#include "SourceData.h"
 #include "diff.h"
 
+#include "LineRef.h"
+#include "TypeUtils.h"
+
 #include <QLabel>
-#include <QSharedPointer>  // for QSharedPointer
-#include <QString>         // for QString
+#include <QMouseEvent>
+#include <QPaintEvent>
+#include <QSharedPointer>
+#include <QString>
+#include <QWheelEvent>
 
 #include <boost/signals2.hpp>
+#include <list>
+#include <memory>
 
 class QMenu;
 class RecalcWordWrapRunnable;
@@ -27,6 +36,7 @@ class DiffTextWindowData;
 class DiffTextWindowFrame;
 class EncodingLabel;
 class RLPainter;
+class SourceData;
 
 class KDiff3App;
 
@@ -37,14 +47,14 @@ class DiffTextWindow : public QWidget
     //Using this as a scoped global
     static QScrollBar* mVScrollBar;
 
-    DiffTextWindow(DiffTextWindowFrame* pParent, const QSharedPointer<Options> &pOptions, e_SrcSelector winIdx);
+    DiffTextWindow(DiffTextWindowFrame* pParent, const QSharedPointer<Options> &pOptions, e_SrcSelector winIdx, KDiff3App &app);
     ~DiffTextWindow() override;
     void init(
         const QString& fileName,
         QTextCodec* pTextCodec,
         e_LineEndStyle eLineEndStyle,
-        const QVector<LineData>* pLineData,
-        int size,
+        const std::shared_ptr<LineDataVector> &pLineData,
+        LineCount size,
         const Diff3LineVector* pDiff3LineVector,
         const ManualDiffHelpList* pManualDiffHelpList
     );
@@ -52,22 +62,22 @@ class DiffTextWindow : public QWidget
     void setupConnections(const KDiff3App *app);
 
     void reset();
-    void convertToLinePos(int x, int y, LineRef& line, int& pos);
+    void convertToLinePos(int x, int y, LineRef& line, QtNumberType& pos);
 
-    QString getSelection();
-    int getFirstLine();
+    [[nodiscard]] QString getSelection() const;
+    [[nodiscard]] int getFirstLine() const;
     LineRef calcTopLineInFile(const LineRef firstLine);
 
-    int getMaxTextWidth();
-    LineCount getNofLines();
-    int getNofVisibleLines();
-    int getVisibleTextAreaWidth();
+    [[nodiscard]] int getMaxTextWidth() const;
+    [[nodiscard]] LineCount getNofLines() const;
+    [[nodiscard]] LineCount getNofVisibleLines() const;
+    [[nodiscard]] int getVisibleTextAreaWidth() const;
 
-    int convertLineToDiff3LineIdx(LineRef line);
-    LineRef convertDiff3LineIdxToLine(int d3lIdx);
+    LineIndex convertLineToDiff3LineIdx(LineRef line);
+    LineRef convertDiff3LineIdxToLine(LineIndex d3lIdx);
 
-    void convertD3LCoordsToLineCoords(int d3LIdx, int d3LPos, int& line, int& pos);
-    void convertLineCoordsToD3LCoords(int line, int pos, int& d3LIdx, int& d3LPos);
+    void convertD3LCoordsToLineCoords(LineIndex d3LIdx, int d3LPos, LineRef& line, int& pos);
+    void convertLineCoordsToD3LCoords(LineRef line, int pos, LineIndex& d3LIdx, int& d3LPos);
 
     void convertSelectionToD3LCoords();
 
@@ -76,24 +86,24 @@ class DiffTextWindow : public QWidget
     void getSelectionRange(LineRef* firstLine, LineRef* lastLine, e_CoordType coordType);
 
     void setPaintingAllowed(bool bAllowPainting);
-    void recalcWordWrap(bool bWordWrap, int wrapLineVectorSize, int visibleTextWidth);
-    void recalcWordWrapHelper(int wrapLineVectorSize, int visibleTextWidth, int cacheListIdx);
+    void recalcWordWrap(bool bWordWrap, QtSizeType wrapLineVectorSize, int visibleTextWidth);
+    void recalcWordWrapHelper(QtSizeType wrapLineVectorSize, int visibleTextWidth, QtSizeType cacheListIdx);
 
-    void printWindow(RLPainter& painter, const QRect& view, const QString& headerText, int line, int linesPerPage, const QColor& fgColor);
-    void print(RLPainter& painter, const QRect& r, int firstLine, int nofLinesPerPage);
+    void printWindow(RLPainter& painter, const QRect& view, const QString& headerText, int line, const LineCount linesPerPage, const QColor& fgColor);
+    void print(RLPainter& painter, const QRect& r, int firstLine, const LineCount nofLinesPerPage);
 
     static bool startRunnables();
 
-    bool isThreeWay() const;
-    const QString& getFileName() const;
+    [[nodiscard]] bool isThreeWay() const;
+    [[nodiscard]] const QString& getFileName() const;
 
-    e_SrcSelector getWindowIndex() const;
+    [[nodiscard]] e_SrcSelector getWindowIndex() const;
 
-    const QString getEncodingDisplayString() const;
-    e_LineEndStyle getLineEndStyle() const;
-    const Diff3LineVector* getDiff3LineVector() const;
+    [[nodiscard]] const QString getEncodingDisplayString() const;
+    [[nodiscard]] e_LineEndStyle getLineEndStyle() const;
+    [[nodiscard]] const Diff3LineVector* getDiff3LineVector() const;
 
-    qint32 getLineNumberWidth() const;
+    [[nodiscard]] qint32 getLineNumberWidth() const;
 
     void setSourceData(const QSharedPointer<SourceData>& inData);
 
@@ -151,7 +161,9 @@ class DiffTextWindow : public QWidget
     */
     std::list<boost::signals2::scoped_connection> connections;
 
-    DiffTextWindowData* d;
+    KDiff3App &m_app;
+    std::unique_ptr<DiffTextWindowData> d;
+
     void showStatusLine(const LineRef lineFromPos);
 
     bool canCopy() { return hasFocus() && !getSelection().isEmpty(); }
@@ -163,7 +175,7 @@ class DiffTextWindowFrame : public QWidget
 {
     Q_OBJECT
   public:
-    DiffTextWindowFrame(QWidget* pParent, const QSharedPointer<Options> &pOptions, e_SrcSelector winIdx, const QSharedPointer<SourceData> &psd);
+    DiffTextWindowFrame(QWidget* pParent, const QSharedPointer<Options> &pOptions, e_SrcSelector winIdx, const QSharedPointer<SourceData> &psd, KDiff3App &app);
     ~DiffTextWindowFrame() override;
     DiffTextWindow* getDiffTextWindow();
     void init();
@@ -184,10 +196,10 @@ class DiffTextWindowFrame : public QWidget
     void slotReturnPressed();
     void slotBrowseButtonClicked();
 
-    void slotEncodingChanged(QTextCodec* c) { Q_EMIT encodingChanged(c); }; //relay signal from encoding label
+    void slotEncodingChanged(QTextCodec* c);
 
   private:
-    DiffTextWindowFrameData* d;
+    std::unique_ptr<DiffTextWindowFrameData> d;
 };
 
 class EncodingLabel : public QLabel
