@@ -4,31 +4,27 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 #ifndef DIRECTORY_MERGE_WINDOW_H
 #define DIRECTORY_MERGE_WINDOW_H
 
 #include "common.h"
 #include "fileaccess.h"
-#include "diff.h" //TotalDiffStatus
 
 #include <QTreeWidget>
 #include <QEvent>
 
 #include <list>
-#include <map>
+#include <memory>
 
 class Options;
-class StatusInfo;
 class DirectoryMergeInfo;
-class OneDirectoryInfo;
 class QLabel;
-class QAction;
 class KToggleAction;
 class KActionCollection;
 class TotalDiffStatus;
-class DirectoryInfo;
 
 class MergeFileInfos;
 
@@ -37,13 +33,12 @@ class DirectoryMergeWindow : public QTreeView
 {
    Q_OBJECT
  public:
-   struct t_ItemInfo;
+   struct ItemInfo;
 
-   DirectoryMergeWindow(QWidget* pParent, const QSharedPointer<Options>& pOptions);
+   DirectoryMergeWindow(QWidget* pParent, const QSharedPointer<Options>& pOptions, KDiff3App &app);
    ~DirectoryMergeWindow() override;
    void setDirectoryMergeInfo(DirectoryMergeInfo* p);
    bool init(
-      const QSharedPointer<DirectoryInfo>& dirInfo,
       bool bDirectoryMerge,
       bool bReload = false
    );
@@ -65,10 +60,10 @@ class DirectoryMergeWindow : public QTreeView
    void focusOutEvent(QFocusEvent* e) override;
    void contextMenuEvent(QContextMenuEvent* e) override;
 
-   QString getDirNameA() const;
-   QString getDirNameB() const;
-   QString getDirNameC() const;
-   QString getDirNameDest() const;
+   [[nodiscard]] QString getDirNameA() const;
+   [[nodiscard]] QString getDirNameB() const;
+   [[nodiscard]] QString getDirNameC() const;
+   [[nodiscard]] QString getDirNameDest() const;
 
  public Q_SLOTS:
    void reload();
@@ -127,11 +122,11 @@ protected Q_SLOTS:
    void onExpanded();
    void currentChanged(const QModelIndex& current, const QModelIndex& previous) override; // override
 private:
-  int getIntFromIndex(const QModelIndex& index) const;
-  const QSharedPointer<Options>& getOptions() const;
+  [[nodiscard]] int getIntFromIndex(const QModelIndex& index) const;
+  [[nodiscard]] const QSharedPointer<Options>& getOptions() const;
 
   class DirectoryMergeWindowPrivate;
-  DirectoryMergeWindowPrivate* d;
+  std::unique_ptr<DirectoryMergeWindowPrivate> d;
   class DirMergeItemDelegate;
 };
 
@@ -141,11 +136,11 @@ class DirectoryMergeInfo : public QFrame
 public:
    explicit DirectoryMergeInfo(QWidget* pParent);
    void setInfo(
-      const FileAccess& dirA,
-      const FileAccess& dirB,
-      const FileAccess& dirC,
-      const FileAccess& dirDest,
-      MergeFileInfos& mfi);
+       const FileAccess& dirA,
+       const FileAccess& dirB,
+       const FileAccess& dirC,
+       const FileAccess& dirDest,
+       const MergeFileInfos& mfi);
    QTreeWidget* getInfoList() { return m_pInfoList; }
    bool eventFilter(QObject* o, QEvent* e) override;
 Q_SIGNALS:
