@@ -4,31 +4,32 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 #include "fileaccess.h"
 
 #include "common.h"
-#include "CvsIgnoreList.h"
-#include <qglobal.h>
+
 #ifndef AUTOTEST
 #include "DefaultFileAccessJobHandler.h"
 #endif
 #include "FileAccessJobHandler.h"
+#include "IgnoreList.h"
 #include "Logging.h"
+#include "ProgressProxy.h"
 #include "Utils.h"
-#include "progress.h"
 
+#include <algorithm>                      // for min
 #include <cstdlib>
 #include <sys/stat.h>
-#include <sys/types.h>
+
 #ifndef Q_OS_WIN
 #include <unistd.h>
 #endif
-#include <vector>
+#include <utility>                        // for move
 
 #include <QDir>
 #include <QFile>
-#include <QRegExp>
 #include <QTemporaryFile>
 #include <QtMath>
 
@@ -257,7 +258,7 @@ void FileAccess::reset()
 */
 void FileAccess::setFile(FileAccess* pParent, const QFileInfo& fi)
 {
-    Q_ASSERT(pParent != this);
+    assert(pParent != this);
 #ifndef AUTOTEST
     if(mJobHandler == nullptr) mJobHandler.reset(new DefaultFileAccessJobHandler(this));
 #endif
@@ -288,7 +289,7 @@ void FileAccess::setFile(const QUrl& url, bool bWantToWrite)
     if(mJobHandler == nullptr) mJobHandler.reset(new DefaultFileAccessJobHandler(this));
 #endif
     reset();
-    Q_ASSERT(parent() == nullptr || url != parent()->url());
+    assert(parent() == nullptr || url != parent()->url());
 
     m_url = url;
 
@@ -307,7 +308,7 @@ void FileAccess::setFile(const QUrl& url, bool bWantToWrite)
     {
         m_name = m_url.fileName();
 
-        if(mJobHandler->stat(2 /*all details*/, bWantToWrite))
+        if(mJobHandler->stat(bWantToWrite))
             m_bValidData = true; // After running stat() the variables are initialised
                                  // and valid even if the file doesn't exist and the stat
                                  // query failed.
@@ -377,7 +378,7 @@ void FileAccess::addPath(const QString& txt, bool reinit)
     }
     else
     {
-        QString slash = (txt.isEmpty() || txt[0] == '/') ? QLatin1String("") : QLatin1String("/");
+        QString slash = (txt.isEmpty() || txt[0] == '/') ? QString() : u8"/";
         setFile(absoluteFilePath() + slash + txt);
     }
 }
@@ -426,7 +427,7 @@ void FileAccess::setFromUdsEntry(const KIO::UDSEntry& e, FileAccess* parent)
     const QVector<uint> fields = e.fields();
     QString filePath;
 
-    Q_ASSERT(this != parent);
+    assert(this != parent);
     m_pParent = parent;
 
     for(const uint fieldId: fields)
@@ -520,7 +521,7 @@ void FileAccess::setFromUdsEntry(const KIO::UDSEntry& e, FileAccess* parent)
 
         qCDebug(kdiffFileAccess) << "Computed url is: " << m_url;
         //Verify that the scheme doesn't change.
-        Q_ASSERT(m_url.scheme() == parent->url().scheme());
+        assert(m_url.scheme() == parent->url().scheme());
     }
 
     //KIO does this when stating a remote folder.
@@ -571,6 +572,15 @@ bool FileAccess::isNormal() const
     */
     if(!mVisited && mDepth < 15 && isLocal() && isSymLink())
     {
+        /*
+            wierd psudo-name created from commandline input redirection from output of another command.
+            KIO/Qt does not handle as a normal file but presents it as such.
+        */
+        if(m_linkTarget.startsWith("pipe:"))
+        {
+            return false;
+        }
+
         FileAccess target(m_linkTarget);
 
         mVisited = true;
@@ -706,7 +716,7 @@ QString FileAccess::fileName(bool needTmp) const
 QString FileAccess::fileRelPath() const
 {
 #ifndef AUTOTEST
-    Q_ASSERT(m_pParent == nullptr || m_baseDir == m_pParent->m_baseDir);
+    assert(m_pParent == nullptr || m_baseDir == m_pParent->m_baseDir);
 #endif
     QString path;
 
@@ -737,7 +747,7 @@ QString FileAccess::fileRelPath() const
 
 FileAccess* FileAccess::parent() const
 {
-    Q_ASSERT(m_pParent != this);
+    assert(m_pParent != this);
     return m_pParent;
 }
 
@@ -749,7 +759,7 @@ QString FileAccess::prettyAbsPath() const
 
 QDateTime FileAccess::lastModified() const
 {
-    Q_ASSERT(!m_modificationTime.isNull());
+    assert(!m_modificationTime.isNull());
     return m_modificationTime;
 }
 
@@ -798,7 +808,7 @@ bool FileAccess::readFile(void* pDestBuffer, qint64 maxLength)
     }
 
     close();
-    Q_ASSERT(!realFile->isOpen() && !tmpFile->isOpen());
+    assert(!realFile->isOpen() && !tmpFile->isOpen());
     return success;
 }
 
@@ -846,12 +856,12 @@ bool FileAccess::writeFile(const void* pSrcBuffer, qint64 length)
         bool success = mJobHandler->put(pSrcBuffer, length, true /*overwrite*/);
         close();
 
-        Q_ASSERT(!realFile->isOpen() && !tmpFile->isOpen());
+        assert(!realFile->isOpen() && !tmpFile->isOpen());
 
         return success;
     }
     close();
-    Q_ASSERT(!realFile->isOpen() && !tmpFile->isOpen());
+    assert(!realFile->isOpen() && !tmpFile->isOpen());
     return false;
 }
 
@@ -877,13 +887,13 @@ bool FileAccess::removeFile()
     }
 }
 
-bool FileAccess::listDir(t_DirectoryList* pDirList, bool bRecursive, bool bFindHidden,
+bool FileAccess::listDir(DirectoryList* pDirList, bool bRecursive, bool bFindHidden,
                          const QString& filePattern, const QString& fileAntiPattern, const QString& dirAntiPattern,
-                         bool bFollowDirLinks, bool bUseCvsIgnore)
+                         bool bFollowDirLinks, IgnoreList& ignoreList)
 {
-    Q_ASSERT(mJobHandler != nullptr);
+    assert(mJobHandler != nullptr);
     return mJobHandler->listDir(pDirList, bRecursive, bFindHidden, filePattern, fileAntiPattern,
-                      dirAntiPattern, bFollowDirLinks, bUseCvsIgnore);
+                      dirAntiPattern, bFollowDirLinks, ignoreList);
 }
 
 QString FileAccess::getTempName() const
@@ -1095,15 +1105,10 @@ void FileAccess::doError()
     m_bExists = false;
 }
 
-void FileAccess::filterList(t_DirectoryList* pDirList, const QString& filePattern,
+void FileAccess::filterList(const QString& dir, DirectoryList* pDirList, const QString& filePattern,
                             const QString& fileAntiPattern, const QString& dirAntiPattern,
-                            const bool bUseCvsIgnore)
+                            IgnoreList& ignoreList)
 {
-    CvsIgnoreList cvsIgnoreList;
-    if(bUseCvsIgnore)
-    {
-        cvsIgnoreList.init(*this, pDirList);
-    }
     //TODO: Ask os for this information don't hard code it.
 #if defined(Q_OS_WIN)
     bool bCaseSensitive = false;
@@ -1112,10 +1117,10 @@ void FileAccess::filterList(t_DirectoryList* pDirList, const QString& filePatter
 #endif
 
     // Now remove all entries that should be ignored:
-    t_DirectoryList::iterator i;
+    DirectoryList::iterator i;
     for(i = pDirList->begin(); i != pDirList->end();)
     {
-        t_DirectoryList::iterator i2 = i;
+        DirectoryList::iterator i2 = i;
         ++i2;
         QString fileName = i->fileName();
 
@@ -1123,7 +1128,7 @@ void FileAccess::filterList(t_DirectoryList* pDirList, const QString& filePatter
             (!Utils::wildcardMultiMatch(filePattern, fileName, bCaseSensitive) ||
              Utils::wildcardMultiMatch(fileAntiPattern, fileName, bCaseSensitive))) ||
            (i->isDir() && Utils::wildcardMultiMatch(dirAntiPattern, fileName, bCaseSensitive)) ||
-           (bUseCvsIgnore && cvsIgnoreList.matches(fileName, bCaseSensitive)))
+           (ignoreList.matches(dir, fileName, bCaseSensitive)))
         {
             // Remove it
             pDirList->erase(i);
