@@ -1,3 +1,4 @@
+// clang-format off
 /*
  *  This file is part of KDiff3.
  *
@@ -5,7 +6,7 @@
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
 */
-
+// clang-format on
 
 // application specific includes
 #include "kdiff3.h"
@@ -23,17 +24,21 @@
 #include "mergeresultwindow.h"
 #include "RLPainter.h"
 #include "Utils.h"
-
+// Standard c/c++ includes
+#include <memory>
 #ifndef Q_OS_WIN
 #include <unistd.h>
 #endif
-
+#include <utility>
+// Anything else that isn't Qt/Frameworks
+#include <boost/signals2.hpp>
 // include files for QT
-#include <QClipboard>
 #include <QCheckBox>
+#include <QClipboard>
 #include <QCommandLineParser>
 #include <QDesktopWidget>
 #include <QDir>
+#include <QDockWidget>
 #include <QFileDialog>
 #include <QLabel>
 #include <QLayout>
@@ -66,8 +71,6 @@ bool KDiff3App::m_bTripleDiff = false;
 boost::signals2::signal<QString (), FirstNonEmpty<QString>> KDiff3App::getSelection;
 boost::signals2::signal<bool (), or> KDiff3App::allowCopy;
 boost::signals2::signal<bool (), or> KDiff3App::allowCut;
-boost::signals2::signal<bool(), and> KDiff3App::shouldContinue;
-boost::signals2::signal<void (QTextCodec*)> KDiff3App::encodingChanged;
 
 /*
     To be a constexpr the QLatin1String constructor must be given the size of the string explicitly.
@@ -79,24 +82,24 @@ KActionCollection* KDiff3App::actionCollection() const
 {
     if(m_pKDiff3Shell == nullptr)
         return m_pKDiff3Part->actionCollection();
-    else
-        return m_pKDiff3Shell->actionCollection();
+
+    return m_pKDiff3Shell->actionCollection();
 }
 
 QStatusBar* KDiff3App::statusBar() const
 {
     if(m_pKDiff3Shell == nullptr)
         return nullptr;
-    else
-        return m_pKDiff3Shell->statusBar();
+
+    return m_pKDiff3Shell->statusBar();
 }
 
 KToolBar* KDiff3App::toolBar(const QLatin1String &toolBarId) const
 {
     if(m_pKDiff3Shell == nullptr)
         return nullptr;
-    else
-        return m_pKDiff3Shell->toolBar(toolBarId);
+
+    return m_pKDiff3Shell->toolBar(toolBarId);
 }
 
 bool KDiff3App::isPart() const
@@ -117,15 +120,26 @@ bool KDiff3App::isDirComparison() const
 /*
     Don't call completeInit from here it will be called in KDiff3Shell as needed.
 */
-KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3Part)
-    : QSplitter(pParent) //previously KMainWindow
+KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3Part):
+    QMainWindow(pParent) //previously KMainWindow
 {
+    setWindowFlags(Qt::Widget);
     setObjectName(name);
     m_pKDiff3Part = pKDiff3Part;
     m_pKDiff3Shell = qobject_cast<KParts::MainWindow*>(pParent);
 
+    m_pCentralWidget = new QWidget(this);
+    QVBoxLayout* pCentralLayout = new QVBoxLayout(m_pCentralWidget);
+    pCentralLayout->setContentsMargins(0, 0, 0, 0);
+    pCentralLayout->setSpacing(0);
+    setCentralWidget(m_pCentralWidget);
+
+    m_pMainWidget = new QWidget(m_pCentralWidget);
+    m_pMainWidget->setObjectName("MainWidget");
+    pCentralLayout->addWidget(m_pMainWidget);
+    m_pMainWidget->hide();
+
     setWindowTitle("KDiff3");
-    setOpaqueResize(false); // faster resizing
     setUpdatesEnabled(false);
     KCrash::initialize();
 
@@ -177,7 +191,7 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3P
                 pDialog->setAttribute(Qt::WA_DeleteOnClose);
                 pDialog->setModal(true);
                 pDialog->setWindowTitle(title);
-                QVBoxLayout* pVBoxLayout = new QVBoxLayout(pDialog);
+                QPointer<QVBoxLayout> pVBoxLayout = new QVBoxLayout(pDialog);
                 QPointer<QTextEdit> pTextEdit = QPointer<QTextEdit>(new QTextEdit(pDialog));
                 pTextEdit->setText(s);
                 pTextEdit->setReadOnly(true);
@@ -327,24 +341,22 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3P
     }
     slotRefresh();
 
-    m_pMainSplitter = this;
-    m_pMainSplitter->setOrientation(Qt::Vertical);
-    //   setCentralWidget( m_pMainSplitter );
-    m_pDirectoryMergeSplitter = new QSplitter(m_pMainSplitter);
-    m_pDirectoryMergeSplitter->setObjectName("DirectoryMergeSplitter");
-    m_pMainSplitter->addWidget(m_pDirectoryMergeSplitter);
-    m_pDirectoryMergeSplitter->setOrientation(Qt::Horizontal);
-    m_pDirectoryMergeWindow = new DirectoryMergeWindow(m_pDirectoryMergeSplitter, m_pOptions);
-    m_pDirectoryMergeSplitter->addWidget(m_pDirectoryMergeWindow);
-    m_pDirectoryMergeInfo = new DirectoryMergeInfo(m_pDirectoryMergeSplitter);
+    m_pDirectoryMergeDock = new QDockWidget(i18n("Directory merge"), this);
+    m_pDirectoryMergeWindow = new DirectoryMergeWindow(m_pDirectoryMergeDock, m_pOptions, *this);
+    m_pDirectoryMergeDock->setWidget(m_pDirectoryMergeWindow);
+    m_pDirectoryMergeDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
+    m_pDirectoryMergeInfoDock = new QDockWidget(i18n("Merge info"), this);
+    m_pDirectoryMergeInfo = new DirectoryMergeInfo(m_pDirectoryMergeInfoDock);
+    m_pDirectoryMergeInfoDock->setWidget(m_pDirectoryMergeInfo);
+    m_pDirectoryMergeInfoDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
     m_pDirectoryMergeWindow->setDirectoryMergeInfo(m_pDirectoryMergeInfo);
-    m_pDirectoryMergeSplitter->addWidget(m_pDirectoryMergeInfo);
     //Warning: Make sure DirectoryMergeWindow::initActions is called before this point or we can crash when selectionChanged is sent.
     m_pDirectoryMergeWindow->setupConnections(this);
+    addDockWidget(Qt::LeftDockWidgetArea, m_pDirectoryMergeDock);
+    splitDockWidget(m_pDirectoryMergeDock, m_pDirectoryMergeInfoDock, Qt::Vertical);
 
     chk_connect(QApplication::clipboard(), &QClipboard::dataChanged, this, &KDiff3App::slotClipboardChanged);
     chk_connect_q(this, &KDiff3App::sigRecalcWordWrap, this, &KDiff3App::slotRecalcWordWrap);
-    connections.push_back(shouldContinue.connect(boost::bind(&KDiff3App::canContinue, this)));
     chk_connect(this, &KDiff3App::finishDrop, this, &KDiff3App::slotFinishDrop);
 
     connections.push_back(allowCut.connect(boost::bind(&KDiff3App::canCut, this)));
@@ -355,7 +367,7 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3P
     if(qApp != nullptr)
         chk_connect(qApp, &QApplication::focusChanged, this, &KDiff3App::slotFocusChanged);
 
-    delete KDiff3Shell::getParser();
+    chk_connect_a(this, &KDiff3App::updateAvailabilities, this, &KDiff3App::slotUpdateAvailabilities);
 }
 
 /*
@@ -370,8 +382,11 @@ bool KDiff3App::canCut()
 }
 
 /*
+    Please do not add logic for MerdgeResultWindow or DiffTextWindow here they have their own handlers.
     This function is only concerned with qt objects that don't support canCopy.
-    allowCopy() or's the results from all canCopy signals
+    allowCopy() or's the results from all canCopy signals sent via boost.
+
+    returns true if a QLineEdit is in focus because Qt handles these internally.
 */
 bool KDiff3App::canCopy()
 {
@@ -387,7 +402,7 @@ void KDiff3App::slotFocusChanged(QWidget* old, QWidget* now)
     Q_UNUSED(now);
     Q_UNUSED(old);
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QString& fn3)
@@ -404,7 +419,7 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
     }
 
     //should not happen now.
-    Q_ASSERT(m_bDirCompare == m_sd1->isDir());
+    assert(m_bDirCompare == m_sd1->isDir());
 
     if(m_bAutoFlag && m_bAutoMode && m_bDirCompare)
     {
@@ -418,7 +433,8 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
     else
     {
         improveFilenames();
-        m_pDirectoryMergeSplitter->hide();
+        m_pDirectoryMergeDock->hide();
+        m_pDirectoryMergeInfoDock->hide();
 
         mainInit(m_totalDiffStatus);
         if(m_bAutoMode)
@@ -457,7 +473,7 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
                 else
                     KMessageBox::error(this, i18n("Saving failed."));
             }
-            else if(m_pMergeResultWindow->getNrOfUnsolvedConflicts() == 0)
+            else if(m_pMergeResultWindow->getNumberOfUnsolvedConflicts() == 0)
             {
                 bSuccess = m_pMergeResultWindow->saveDocument(m_pMergeResultWindowTitle->getFileName(), m_pMergeResultWindowTitle->getEncoding(), m_pMergeResultWindowTitle->getLineEndStyle());
                 if(bSuccess) ::exit(0);
@@ -466,25 +482,31 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
     }
     m_bAutoMode = false;
 
-    if(!isPart() && m_pOptions->isFullScreen())
-        m_pKDiff3Shell->showFullScreen();
-    else if(!isPart() && m_pOptions->isMaximised())
-        m_pKDiff3Shell->showMaximized();
-
-    if(!isPart() && !m_pKDiff3Shell->isVisible())
+    if(!isPart())
     {
-        /*
-            This is just an intital state before qt does a proper restore.
-        */
-        QSize size = m_pOptions->getGeometry();
-        QPoint pos = m_pOptions->getPosition();
-        if(!size.isEmpty())
+        if(!m_pKDiff3Shell->isVisible() && !restoreWindow(KSharedConfig::openConfig()))
         {
-            m_pKDiff3Shell->resize(size);
+            /*
+                Set default state/geometry from config file.
+                This will no longer be updated but serves as a fallback.
+                Qt's restoreState/saveState can handle multiple screens this won't.
+            */
+            if(m_pOptions->isFullScreen())
+                m_pKDiff3Shell->showFullScreen();
+            else if(m_pOptions->isMaximised())
+                m_pKDiff3Shell->showMaximized();
 
-            QRect visibleRect = QRect(pos, size) & QApplication::desktop()->rect();
-            if(visibleRect.width() > 100 && visibleRect.height() > 100)
-                m_pKDiff3Shell->move(pos);
+            QSize size = m_pOptions->getGeometry();
+            QPoint pos = m_pOptions->getPosition();
+
+            if(!size.isEmpty())
+            {
+                m_pKDiff3Shell->resize(size);
+
+                QRect visibleRect = QRect(pos, size) & QApplication::desktop()->rect();
+                if(visibleRect.width() > 100 && visibleRect.height() > 100)
+                    m_pKDiff3Shell->move(pos);
+            }
         }
 
         m_pKDiff3Shell->show();
@@ -497,7 +519,7 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
 
     slotClipboardChanged(); // For initialisation.
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 
     if(!m_bDirCompare && m_pKDiff3Shell != nullptr)
     {
@@ -515,7 +537,7 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
             if(!m_sd3->getErrors().isEmpty())
                 text += " - " + m_sd3->getAliasName() + '\n' + m_sd3->getErrors().join('\n') + '\n';
 
-            KMessageBox::sorry(this, text, i18n("File open error"));
+            KMessageBox::error(this, text, i18n("File open error"));
 
             bFileOpenError = true;
         }
@@ -548,7 +570,7 @@ void KDiff3App::initActions(KActionCollection* ac)
     fileOpen = KStandardAction::open(this, &KDiff3App::slotFileOpen, ac);
     fileOpen->setStatusTip(i18n("Opens documents for comparison..."));
 
-    fileReload = GuiUtils::createAction<QAction>(i18n("Reload"), QKeySequence(QKeySequence::Refresh), this, &KDiff3App::slotReload, ac, QLatin1String("file_reload"));
+    fileReload = GuiUtils::createAction<QAction>(i18n("Reload"), QKeySequence(QKeySequence::Refresh), this, &KDiff3App::slotReload, ac, u8"file_reload");
 
     fileSave = KStandardAction::save(this, &KDiff3App::slotFileSave, ac);
     fileSave->setStatusTip(i18n("Saves the merge result. All conflicts must be solved!"));
@@ -560,6 +582,11 @@ void KDiff3App::initActions(KActionCollection* ac)
 #endif
     fileQuit = KStandardAction::quit(this, &KDiff3App::slotFileQuit, ac);
     fileQuit->setStatusTip(i18n("Quits the application"));
+
+    editUndo = KStandardAction::undo(this, &KDiff3App::slotEditUndo, ac);
+    editUndo->setShortcuts(QKeySequence::Undo);
+    editUndo->setStatusTip(i18n("Undo last action."));
+
     editCut = KStandardAction::cut(this, &KDiff3App::slotEditCut, ac);
     editCut->setShortcuts(QKeySequence::Cut);
     editCut->setStatusTip(i18n("Cuts the selected section and puts it to the clipboard"));
@@ -591,6 +618,7 @@ void KDiff3App::initActions(KActionCollection* ac)
 #include "xpm/down1arrow.xpm"
 #include "xpm/down2arrow.xpm"
 #include "xpm/downend.xpm"
+#include "xpm/gotoline.xpm"
 #include "xpm/iconA.xpm"
 #include "xpm/iconB.xpm"
 #include "xpm/iconC.xpm"
@@ -623,15 +651,17 @@ void KDiff3App::initActions(KActionCollection* ac)
     mGoPrevUnsolvedConflict->setToolTip(mGoPrevUnsolvedConflict->text() + includeWhitespace);
     mGoNextUnsolvedConflict = GuiUtils::createAction<QAction>(i18n("Go to Next Unsolved Conflict"), QIcon(QPixmap(nextunsolved)), i18n("Next\nUnsolved"), QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_PageDown), this, &KDiff3App::slotGoNextUnsolvedConflict, ac, "go_next_unsolved_conflict");
     mGoNextUnsolvedConflict->setToolTip(mGoNextUnsolvedConflict->text() + includeWhitespace);
-    chooseA = GuiUtils::createAction<KToggleAction>(i18n("Select Line(s) From A"), QIcon(QPixmap(iconA)), i18n("Choose\nA"), QKeySequence(Qt::CTRL + Qt::Key_1), this, &KDiff3App::slotChooseA, ac, "merge_choose_a");
-    chooseB = GuiUtils::createAction<KToggleAction>(i18n("Select Line(s) From B"), QIcon(QPixmap(iconB)), i18n("Choose\nB"), QKeySequence(Qt::CTRL + Qt::Key_2), this, &KDiff3App::slotChooseB, ac, "merge_choose_b");
-    chooseC = GuiUtils::createAction<KToggleAction>(i18n("Select Line(s) From C"), QIcon(QPixmap(iconC)), i18n("Choose\nC"), QKeySequence(Qt::CTRL + Qt::Key_3), this, &KDiff3App::slotChooseC, ac, "merge_choose_c");
-    autoAdvance = GuiUtils::createAction<KToggleAction>(i18n("Automatically Go to Next Unsolved Conflict After Source Selection"), QIcon(QPixmap(autoadvance)), i18n("Auto\nNext"), this, &KDiff3App::slotAutoAdvanceToggled, ac, "merge_autoadvance");
+    mGotoLine = GuiUtils::createAction<QAction>(i18nc("Title for menu item", "Go to Line"), QIcon(QPixmap(gotoline)), i18nc("Text used for toolbar button.", "Go\nLine"), QKeySequence(Qt::CTRL + Qt::Key_G), this, &KDiff3App::slotGoToLine, ac, "go_to_line");
+    mGoNextUnsolvedConflict->setToolTip(mGoNextUnsolvedConflict->text() + i18n(".\n Goto specified line."));
+    chooseA = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From A"), QIcon(QPixmap(iconA)), i18nc("Text used for select A toolbar button.", "Choose\nA"), QKeySequence(Qt::CTRL + Qt::Key_1), this, &KDiff3App::slotChooseA, ac, "merge_choose_a");
+    chooseB = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From B"), QIcon(QPixmap(iconB)), i18nc("Text used for select B when toolbar button.", "Choose\nB"), QKeySequence(Qt::CTRL + Qt::Key_2), this, &KDiff3App::slotChooseB, ac, "merge_choose_b");
+    chooseC = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Select Line(s) From C"), QIcon(QPixmap(iconC)), i18nc("Text used for select C toolbar button.", "Choose\nC"), QKeySequence(Qt::CTRL + Qt::Key_3), this, &KDiff3App::slotChooseC, ac, "merge_choose_c");
+    autoAdvance = GuiUtils::createAction<KToggleAction>(i18nc("Title for menu item", "Automatically Go to Next Unsolved Conflict After Source Selection"), QIcon(QPixmap(autoadvance)), i18nc("Auto goto next unsolced toolbar text.", "Auto\nNext"), this, &KDiff3App::slotAutoAdvanceToggled, ac, "merge_autoadvance");
 
-    showWhiteSpaceCharacters = GuiUtils::createAction<KToggleAction>(i18n("Show Space && Tabulator Characters"), QIcon(QPixmap(showwhitespacechars)), i18n("White\nCharacters"), this, &KDiff3App::slotShowWhiteSpaceToggled, ac, "diff_show_whitespace_characters");
-    showWhiteSpace = GuiUtils::createAction<KToggleAction>(i18n("Show White Space"), QIcon(QPixmap(showwhitespace)), i18n("White\nDeltas"), this, &KDiff3App::slotShowWhiteSpaceToggled, ac, "diff_show_whitespace");
+    showWhiteSpaceCharacters = GuiUtils::createAction<KToggleAction>(i18n("Show Space && Tabulator Characters"), QIcon(QPixmap(showwhitespacechars)), i18nc("Show whitespace toolbar text.", "White\nCharacters"), this, &KDiff3App::slotShowWhiteSpaceToggled, ac, "diff_show_whitespace_characters");
+    showWhiteSpace = GuiUtils::createAction<KToggleAction>(i18n("Show White Space"), QIcon(QPixmap(showwhitespace)), i18nc("Show whitespace changes toolbar text.", "White\nDeltas"), this, &KDiff3App::slotShowWhiteSpaceToggled, ac, "diff_show_whitespace");
 
-    showLineNumbers = GuiUtils::createAction<KToggleAction>(i18n("Show Line Numbers"), QIcon(QPixmap(showlinenumbers)), i18n("Line\nNumbers"), this, &KDiff3App::slotShowLineNumbersToggled, ac, "diff_showlinenumbers");
+    showLineNumbers = GuiUtils::createAction<KToggleAction>(i18n("Show Line Numbers"), QIcon(QPixmap(showlinenumbers)), i18nc("Show line numbers toolbar text", "Line\nNumbers"), this, &KDiff3App::slotShowLineNumbersToggled, ac, "diff_showlinenumbers");
 
     mAutoSolve = GuiUtils::createAction<QAction>(i18n("Automatically Solve Simple Conflicts"), this, &KDiff3App::slotAutoSolve, ac, "merge_autosolve");
     mUnsolve = GuiUtils::createAction<QAction>(i18n("Set Deltas to Conflicts"), this, &KDiff3App::slotUnsolve, ac, "merge_autounsolve");
@@ -682,19 +712,29 @@ void KDiff3App::initStatusBar()
         statusBar()->showMessage(i18n("Ready."));
 }
 
+void KDiff3App::saveWindow(const KSharedConfigPtr config)
+{
+    KConfigGroup group = config->group(KDIFF3_CONFIG_GROUP);
+
+    group.writeEntry("mainWindow-geometry", m_pKDiff3Shell->saveGeometry());
+    group.writeEntry("mainWindow-state", m_pKDiff3Shell->saveState());
+}
+
+bool KDiff3App::restoreWindow(const KSharedConfigPtr config)
+{
+    KConfigGroup group = config->group(KDIFF3_CONFIG_GROUP);
+
+    return (m_pKDiff3Shell->restoreGeometry(group.readEntry("mainWindow-geometry", QVariant(QByteArray())).toByteArray()) &&
+            m_pKDiff3Shell->restoreState(group.readEntry("mainWindow-state", QVariant(QByteArray())).toByteArray()));
+}
+
 void KDiff3App::saveOptions(KSharedConfigPtr config)
 {
     if(!m_bAutoMode)
     {
         if(!isPart())
         {
-            m_pOptions->setFullScreen(m_pKDiff3Shell->isFullScreen());
-            m_pOptions->setMaximised(m_pKDiff3Shell->isMaximized());
-            if(!m_pKDiff3Shell->isFullScreen() && !m_pKDiff3Shell->isMaximized() && m_pKDiff3Shell->isVisible())
-            {
-                m_pOptions->setGeometry(m_pKDiff3Shell->size());
-                m_pOptions->setPosition(m_pKDiff3Shell->pos());
-            }
+            saveWindow(config);
         }
 
         m_pOptionDialog->saveOptions(std::move(config));
@@ -709,7 +749,7 @@ bool KDiff3App::queryClose()
     {
         int result = KMessageBox::warningYesNoCancel(this,
                                                      i18n("The merge result has not been saved."),
-                                                     i18n("Warning"),
+                                                     i18nc("Error dialog caption", "Warning"),
                                                      KGuiItem(i18n("Save && Quit")),
                                                      KGuiItem(i18n("Quit Without Saving")));
         if(result == KMessageBox::Cancel)
@@ -719,7 +759,7 @@ bool KDiff3App::queryClose()
             slotFileSave();
             if(m_bOutputModified)
             {
-                KMessageBox::sorry(this, i18n("Saving the merge result failed."), i18n("Warning"));
+                KMessageBox::error(this, i18n("Saving the merge result failed."), i18nc("Error dialog caption", "Warning"));
                 return false;
             }
         }
@@ -731,7 +771,7 @@ bool KDiff3App::queryClose()
     {
         int result = KMessageBox::warningYesNo(this,
                                                i18n("You are currently doing a folder merge. Are you sure, you want to abort?"),
-                                               i18n("Warning"),
+                                               i18nc("Error dialog caption", "Warning"),
                                                KStandardGuiItem::quit(),
                                                KStandardGuiItem::cont() /* i18n("Continue Merging") */);
         if(result != KMessageBox::Yes)
@@ -845,7 +885,7 @@ void KDiff3App::slotFilePrint()
         int columns = m_bTripleDiff ? 3 : 2;
         int columnWidth = (pPaintDevice->width() - (columns - 1) * columnDistance) / columns;
 
-        QFont f = m_pOptions->m_font;
+        QFont f = m_pOptions->defaultFont();
         f.setPointSizeF(f.pointSizeF() - 1); // Print with slightly smaller font.
         painter.setFont(f);
         QFontMetrics fm = painter.fontMetrics();
@@ -863,7 +903,7 @@ void KDiff3App::slotFilePrint()
         QRect view2(1 * (columnWidth + columnDistance), view.top(), columnWidth, view.height());
         QRect view3(2 * (columnWidth + columnDistance), view.top(), columnWidth, view.height());
 
-        int linesPerPage = view.height() / fm.lineSpacing();
+        LineCount linesPerPage = view.height() / fm.lineSpacing();
 
         m_pEventLoopForPrinting = QPointer<QEventLoop>(new QEventLoop());
         if(m_pOptions->wordWrapOn())
@@ -884,7 +924,7 @@ void KDiff3App::slotFilePrint()
         bool bFirstPrintedPage = false;
 
         bool bPrintSelection = false;
-        int totalNofPages = (totalNofLines + linesPerPage - 1) / linesPerPage;
+        LineCount totalNofPages = (totalNofLines + linesPerPage - 1) / linesPerPage;
         LineRef line;
         LineRef selectionEndLine;
 
@@ -973,23 +1013,23 @@ void KDiff3App::slotFilePrint()
 
                 painter.setClipping(true);
 
-                painter.setPen(m_pOptions->m_colorA);
+                painter.setPen(m_pOptions->aColor());
                 QString headerText1 = m_sd1->getAliasName() + ", " + topLineText + ": " + QString::number(m_pDiffTextWindow1->calcTopLineInFile(line) + 1);
-                m_pDiffTextWindow1->printWindow(painter, view1, headerText1, line, linesPerPage, m_pOptions->m_fgColor);
+                m_pDiffTextWindow1->printWindow(painter, view1, headerText1, line, linesPerPage, m_pOptions->forgroundColor());
 
-                painter.setPen(m_pOptions->m_colorB);
+                painter.setPen(m_pOptions->bColor());
                 QString headerText2 = m_sd2->getAliasName() + ", " + topLineText + ": " + QString::number(m_pDiffTextWindow2->calcTopLineInFile(line) + 1);
-                m_pDiffTextWindow2->printWindow(painter, view2, headerText2, line, linesPerPage, m_pOptions->m_fgColor);
+                m_pDiffTextWindow2->printWindow(painter, view2, headerText2, line, linesPerPage, m_pOptions->forgroundColor());
 
                 if(m_bTripleDiff && m_pDiffTextWindow3 != nullptr)
                 {
-                    painter.setPen(m_pOptions->m_colorC);
+                    painter.setPen(m_pOptions->cColor());
                     QString headerText3 = m_sd3->getAliasName() + ", " + topLineText + ": " + QString::number(m_pDiffTextWindow3->calcTopLineInFile(line) + 1);
-                    m_pDiffTextWindow3->printWindow(painter, view3, headerText3, line, linesPerPage, m_pOptions->m_fgColor);
+                    m_pDiffTextWindow3->printWindow(painter, view3, headerText3, line, linesPerPage, m_pOptions->forgroundColor());
                 }
                 painter.setClipping(false);
 
-                painter.setPen(m_pOptions->m_fgColor);
+                painter.setPen(m_pOptions->forgroundColor());
                 painter.drawLine(0, view.bottom() + 3, view.width(), view.bottom() + 3);
                 QString s = bPrintCurrentPage ? QString("")
                                               : QString::number(page) + '/' + QString::number(totalNofPages);
@@ -1043,7 +1083,7 @@ void KDiff3App::slotFileQuit()
 
 void KDiff3App::slotViewToolBar()
 {
-    Q_ASSERT(viewToolBar != nullptr);
+    assert(viewToolBar != nullptr);
     slotStatusMsg(i18n("Toggling toolbar..."));
     m_pOptions->setToolbarState(viewToolBar->isChecked());
     ///////////////////////////////////////////////////////////////////
