@@ -4,7 +4,8 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 #ifndef KDIFF3_H
 #define KDIFF3_H
@@ -12,6 +13,8 @@
 #include "diff.h"
 #include "defmac.h"
 #include "combiners.h"
+#include "SourceData.h"
+#include "TypeUtils.h"
 
 #include <boost/signals2.hpp>
 
@@ -21,15 +24,17 @@
 #include <QEventLoop>
 #include <QPointer>
 #include <QScrollBar>
+#include <QSharedPointer>
 #include <QSplitter>
 
 // include files for KDE
 #include <KConfigGroup>
 #include <KMainWindow>
-#include <KParts/MainWindow>
 #include <KSharedConfig>
 #include <KToggleAction>
+
 // forward declaration of the KDiff3 classes
+class Options;
 class OptionDialog;
 
 class Overview;
@@ -44,7 +49,6 @@ class WindowTitleWidget;
 class QStatusBar;
 class QMenu;
 
-class KToggleAction;
 class KToolBar;
 class KActionCollection;
 
@@ -80,7 +84,7 @@ class ReversibleScrollBar : public QScrollBar
             QScrollBar::setValue(i);
     }
 
-    int value() const
+    [[nodiscard]] int value() const
     {
         return m_realVal;
     }
@@ -113,7 +117,7 @@ enum class InitFlag
 Q_DECLARE_FLAGS(InitFlags, InitFlag);
 Q_DECLARE_OPERATORS_FOR_FLAGS(InitFlags);
 
-class KDiff3App : public QSplitter
+class KDiff3App: public QMainWindow
 {
     Q_OBJECT
 
@@ -123,11 +127,14 @@ class KDiff3App : public QSplitter
     KDiff3App(QWidget* parent, const QString& name, KDiff3Part* pKDiff3Part);
     ~KDiff3App() override;
 
-    bool isPart() const;
+    [[nodiscard]] bool isPart() const;
 
     /** initializes the KActions of the application */
     void initActions(KActionCollection*);
 
+    bool restoreWindow(const KSharedConfigPtr config);
+    void saveWindow(const KSharedConfigPtr config);
+
     /** save general Options like all bar positions and status as well as the geometry
         and the recent file list to the configuration file */
     void saveOptions(KSharedConfigPtr);
@@ -145,18 +152,18 @@ class KDiff3App : public QSplitter
      * @see KMainWindow#closeEvent
      */
     virtual bool queryClose();
-    virtual bool isFileSaved() const;
-    virtual bool isDirComparison() const;
+    [[nodiscard]] virtual bool isFileSaved() const;
+    [[nodiscard]] virtual bool isDirComparison() const;
 
     static bool isTripleDiff() { return m_bTripleDiff; }
 
-    KActionCollection* actionCollection() const;
+    [[nodiscard]] KActionCollection* actionCollection() const;
 
     static boost::signals2::signal<QString (), FirstNonEmpty<QString>> getSelection;
     static boost::signals2::signal<bool (), or> allowCopy;
     static boost::signals2::signal<bool (), or> allowCut;
-    static boost::signals2::signal<bool (), and> shouldContinue;
-    static boost::signals2::signal<void (QTextCodec*)> encodingChanged;
+
+    bool canContinue();
 
   Q_SIGNALS:
     void createNewInstance(const QString& fn1, const QString& fn2, const QString& fn3);
@@ -213,6 +220,8 @@ public Q_SLOTS:
      * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
      */
     void slotFileQuit();
+
+    void slotEditUndo(){};
     /** put the marked text/object into the clipboard and remove
      *  it from the document
      */
@@ -261,6 +270,7 @@ public Q_SLOTS:
     void slotGoNextConflict();
     void slotGoPrevDelta();
     void slotGoNextDelta();
+    void slotGoToLine();
     void slotChooseA();
     void slotChooseB();
     void slotChooseC();
@@ -304,6 +314,8 @@ public Q_SLOTS:
     void setHScrollBarRange();
 
     void slotFocusChanged(QWidget *old, QWidget *now);
+  Q_SIGNALS:
+    void updateAvailabilities();
 
   protected:
     void setLockPainting(bool bLock);
@@ -325,12 +337,10 @@ public Q_SLOTS:
     bool doDirectoryCompare(const bool bCreateNewInstance);
     void improveFilenames();
 
-    bool canContinue();
-
     void choose(e_SrcSelector choice);
 
-    QStatusBar* statusBar() const;
-    KToolBar* toolBar(const QLatin1String &toolBarId) const;
+    [[nodiscard]] QStatusBar* statusBar() const;
+    [[nodiscard]] KToolBar* toolBar(const QLatin1String &toolBarId) const;
     void recalcWordWrap(int visibleTextWidthForPrinting = -1);
 
     bool canCut();
@@ -347,6 +357,7 @@ public Q_SLOTS:
     QPointer<QAction> filePrint;
     QPointer<QAction> fileQuit;
     QPointer<QAction> fileReload;
+    QPointer<QAction> editUndo;
     QPointer<QAction> editCut;
     QPointer<QAction> editCopy;
     QPointer<QAction> editPaste;
@@ -368,6 +379,7 @@ public Q_SLOTS:
     QPointer<QAction> mGoNextConflict;
     QPointer<QAction> mGoPrevDelta;
     QPointer<QAction> mGoNextDelta;
+    QPointer<QAction> mGotoLine;
     KToggleAction* chooseA;
     KToggleAction* chooseB;
     KToggleAction* chooseC;
@@ -399,8 +411,9 @@ public Q_SLOTS:
 
     QMenu* m_pMergeEditorPopupMenu;
 
-    QSplitter* m_pMainSplitter = nullptr;
-    QWidget* m_pMainWidget = new QWidget();// Contains vertical splitter and horiz scrollbar
+    QWidget* m_pMainWidget = nullptr; // Contains vertical splitter and horiz scrollbar
+    QWidget* m_pCentralWidget = nullptr;
+    QDockWidget* m_pMergeWindowDock = nullptr;
     QWidget* m_pMergeWindowFrame = nullptr;
     ReversibleScrollBar* m_pHScrollBar = nullptr;
 
@@ -416,8 +429,10 @@ public Q_SLOTS:
     WindowTitleWidget* m_pMergeResultWindowTitle;
     static bool m_bTripleDiff;
 
-    QSplitter* m_pDirectoryMergeSplitter = nullptr;
+    //QSplitter* m_pDirectoryMergeSplitter = nullptr;
+    QDockWidget* m_pDirectoryMergeDock = nullptr;
     DirectoryMergeWindow* m_pDirectoryMergeWindow = nullptr;
+    QDockWidget* m_pDirectoryMergeInfoDock = nullptr;
     DirectoryMergeInfo* m_pDirectoryMergeInfo;
     bool m_bDirCompare = false;
 
@@ -437,14 +452,12 @@ public Q_SLOTS:
     DiffList m_diffList12;
     DiffList m_diffList23;
     DiffList m_diffList13;
-
-    QSharedPointer<DiffBufferInfo> m_diffBufferInfo = QSharedPointer<DiffBufferInfo>::create();
     Diff3LineList m_diff3LineList;
-    Diff3LineVector m_diff3LineVector;
+    Diff3LineVector mDiff3LineVector;
     //ManualDiffHelpDialog* m_pManualDiffHelpDialog;
     ManualDiffHelpList m_manualDiffHelpList;
 
-    int m_neededLines;
+    SafeInt<QtNumberType> m_neededLines = 0;
     int m_DTWHeight;
     bool m_bOutputModified = false;
     bool m_bFileSaved = false;
@@ -466,6 +479,8 @@ public Q_SLOTS:
     int m_firstD3LIdx;                 // only needed during recalcWordWrap
     QPointer<QEventLoop> m_pEventLoopForPrinting;
 
+    bool mRunnablesStarted = false;
+
     /*
       This list exists solely to auto disconnect boost signals.
     */
