@@ -1,3 +1,4 @@
+// clang-format off
 /*
  *  This file is part of KDiff3.
  *
@@ -5,15 +6,14 @@
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
 */
+// clang-format on
 
 #include "kdiff3_shell.h"
 #include "UTF8BOMCodec.h"
 #include "version.h"
 
-#include "Logging.h"
-
-#include <stdio.h>// for fileno, stderr
-#include <stdlib.h>// for exit
+#include <stdio.h>  // for fileno, stderr
+#include <stdlib.h> // for exit
 
 #ifndef Q_OS_WIN
 #include <unistd.h>
@@ -29,13 +29,13 @@
 #include <QCommandLineParser>
 #include <QFile>
 #include <QPointer>
-#include <QStringList>
 #include <QStandardPaths>
+#include <QStringList>
 #include <QTextStream>
 
 void initialiseCmdLineArgs(QCommandLineParser* cmdLineParser)
 {
-    QString configFileName = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, "kdiff3rc");
+    const QString configFileName = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, "kdiff3rc");
     QFile configFile(configFileName);
     QString ignorableOptionsLine = "-u;-query;-html;-abort";
     if(configFile.open(QIODevice::ReadOnly))
@@ -43,10 +43,10 @@ void initialiseCmdLineArgs(QCommandLineParser* cmdLineParser)
         QTextStream ts(&configFile);
         while(!ts.atEnd())
         {
-            QString line = ts.readLine();
-            if(line.startsWith(QLatin1String("IgnorableCmdLineOptions=")))
+            const QString line = ts.readLine();
+            if(line.startsWith(u8"IgnorableCmdLineOptions="))
             {
-                int pos = line.indexOf('=');
+                const int pos = line.indexOf('=');
                 if(pos >= 0)
                 {
                     ignorableOptionsLine = line.mid(pos + 1);
@@ -56,18 +56,19 @@ void initialiseCmdLineArgs(QCommandLineParser* cmdLineParser)
         }
     }
     //support our own old preferences this is obsolete
-    QStringList sl = ignorableOptionsLine.split(',');
+    const QStringList sl = ignorableOptionsLine.split(',');
 
     if(!sl.isEmpty())
     {
         const QStringList ignorableOptions = sl.front().split(';');
-        for(QString ignorableOption : ignorableOptions)
+        for(QString ignorableOption: ignorableOptions)
         {
             ignorableOption.remove('-');
             if(!ignorableOption.isEmpty())
             {
-                if(ignorableOption.length() == 1) {
-                    cmdLineParser->addOption(QCommandLineOption({ignorableOption, QLatin1String("ignore")}, i18n("Ignored. (User defined.)")));
+                if(ignorableOption.length() == 1)
+                {
+                    cmdLineParser->addOption(QCommandLineOption({ignorableOption, u8"ignore"}, i18n("Ignored. (User defined.)")));
                 }
                 else
                 {
@@ -99,9 +100,9 @@ int main(int argc, char* argv[])
     QString appVersion(KDIFF3_VERSION_STRING);
 
     if(sizeof(void*) == 8)
-        appVersion += i18n(" (64 bit)");
+        appVersion += i18nc("Program version info.", " (64 bit)");
     else if(sizeof(void*) == 4)
-        appVersion += i18n(" (32 bit)");
+        appVersion += i18nc("Program version info.", " (32 bit)");
     const QString description = i18n("Tool for Comparison and Merge of Files and Folders");
     const QString copyright = i18n("(c) 2002-2014 Joachim Eibl, (c) 2017 Michael Reeves KF5/Qt5 port");
     const QString homePage = QStringLiteral("https://kde.org/applications/development/kdiff3");
@@ -112,36 +113,40 @@ int main(int argc, char* argv[])
 
     KAboutData::setApplicationData(aboutData);
 
-    QCommandLineParser* cmdLineParser = KDiff3Shell::getParser();
+    /*
+        The QCommandLineParser is a static scoped unique ptr. This is safe given that.
+        As the distuctor will not be fired until main exits.
+    */
+    QCommandLineParser* cmdLineParser = KDiff3Shell::getParser().get();
     cmdLineParser->setApplicationDescription(aboutData.shortDescription());
 
     aboutData.setupCommandLine(cmdLineParser);
 
     initialiseCmdLineArgs(cmdLineParser);
     // ignorable command options
-    cmdLineParser->addOption(QCommandLineOption({QLatin1String("m"), QLatin1String("merge")}, i18n("Merge the input.")));
-    cmdLineParser->addOption(QCommandLineOption({QLatin1String("b"), QLatin1String("base")}, i18n("Explicit base file. For compatibility with certain tools."), QLatin1String("file")));
-    cmdLineParser->addOption(QCommandLineOption({QLatin1String("o"), QLatin1String("output")}, i18n("Output file. Implies -m. E.g.: -o newfile.txt"), QLatin1String("file")));
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("out"), i18n("Output file, again. (For compatibility with certain tools.)"), QLatin1String("file")));
+    cmdLineParser->addOption(QCommandLineOption({u8"m", u8"merge"}, i18n("Merge the input.")));
+    cmdLineParser->addOption(QCommandLineOption({u8"b", u8"base"}, i18n("Explicit base file. For compatibility with certain tools."), u8"file"));
+    cmdLineParser->addOption(QCommandLineOption({u8"o", u8"output"}, i18n("Output file. Implies -m. E.g.: -o newfile.txt"), u8"file"));
+    cmdLineParser->addOption(QCommandLineOption(u8"out", i18n("Output file, again. (For compatibility with certain tools.)"), u8"file"));
 #ifdef ENABLE_AUTO
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("auto"), i18n("No GUI if all conflicts are auto-solvable. (Needs -o file)")));
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("noauto"), i18n("Ignore --auto and always show GUI.")));
+    cmdLineParser->addOption(QCommandLineOption(u8"auto", i18n("No GUI if all conflicts are auto-solvable. (Needs -o file)")));
+    cmdLineParser->addOption(QCommandLineOption(u8"noauto", i18n("Ignore --auto and always show GUI.")));
 #else
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("noauto"), i18n("Ignored.")));
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("auto"), i18n("Ignored.")));
+    cmdLineParser->addOption(QCommandLineOption(u8"noauto", i18n("Ignored.")));
+    cmdLineParser->addOption(QCommandLineOption(u8"auto", i18n("Ignored.")));
 #endif
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("L1"), i18n("Visible name replacement for input file 1 (base)."), QLatin1String("alias1")));
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("L2"), i18n("Visible name replacement for input file 2."), QLatin1String("alias2")));
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("L3"), i18n("Visible name replacement for input file 3."), QLatin1String("alias3")));
-    cmdLineParser->addOption(QCommandLineOption({QLatin1String("L"), QLatin1String("fname")}, i18n("Alternative visible name replacement. Supply this once for every input."), QLatin1String("alias")));
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("cs"), i18n("Override a config setting. Use once for every setting. E.g.: --cs \"AutoAdvance=1\""), QLatin1String("string")));
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("confighelp"), i18n("Show list of config settings and current values.")));
-    cmdLineParser->addOption(QCommandLineOption(QLatin1String("config"), i18n("Use a different config file."), QLatin1String("file")));
+    cmdLineParser->addOption(QCommandLineOption(u8"L1", i18n("Visible name replacement for input file 1 (base)."), u8"alias1"));
+    cmdLineParser->addOption(QCommandLineOption(u8"L2", i18n("Visible name replacement for input file 2."), u8"alias2"));
+    cmdLineParser->addOption(QCommandLineOption(u8"L3", i18n("Visible name replacement for input file 3."), u8"alias3"));
+    cmdLineParser->addOption(QCommandLineOption({u8"L", u8"fname"}, i18n("Alternative visible name replacement. Supply this once for every input."), u8"alias"));
+    cmdLineParser->addOption(QCommandLineOption(u8"cs", i18n("Override a config setting. Use once for every setting. E.g.: --cs \"AutoAdvance=1\""), u8"string"));
+    cmdLineParser->addOption(QCommandLineOption(u8"confighelp", i18n("Show list of config settings and current values.")));
+    cmdLineParser->addOption(QCommandLineOption(u8"config", i18n("Use a different config file."), u8"file"));
 
     // other command options
-    cmdLineParser->addPositionalArgument(QLatin1String("[File1]"), i18n("file1 to open (base, if not specified via --base)"));
-    cmdLineParser->addPositionalArgument(QLatin1String("[File2]"), i18n("file2 to open"));
-    cmdLineParser->addPositionalArgument(QLatin1String("[File3]"), i18n("file3 to open"));
+    cmdLineParser->addPositionalArgument(u8"[File1]", i18n("file1 to open (base, if not specified via --base)"));
+    cmdLineParser->addPositionalArgument(u8"[File2]", i18n("file2 to open"));
+    cmdLineParser->addPositionalArgument(u8"[File3]", i18n("file3 to open"));
 
     bool isAtty = true;
 
@@ -163,18 +168,20 @@ int main(int argc, char* argv[])
         */
         if(!cmdLineParser->parse(QCoreApplication::arguments()))
         {
-            QString errorMessage = cmdLineParser->errorText();
+            const QString errorMessage = cmdLineParser->errorText();
 
             KMessageBox::error(nullptr, "<html><head/><body><h2>" + errorMessage + "</h2><pre>" + i18n("See kdiff3 --help for supported options.") + "</pre></body></html>", aboutData.displayName());
             exit(1);
         }
 
-        if(cmdLineParser->isSet(QStringLiteral("version"))) {
+        if(cmdLineParser->isSet(QStringLiteral("version")))
+        {
             KMessageBox::information(nullptr,
                                     aboutData.displayName() + ' ' + aboutData.version(), aboutData.displayName());
             exit(0);
         }
-        if(cmdLineParser->isSet(QStringLiteral("help"))) {
+        if(cmdLineParser->isSet(QStringLiteral("help")))
+        {
             KMessageBox::information(nullptr, "<html><head/><body><pre>" + cmdLineParser->helpText() + "</pre></body></html>", aboutData.displayName());
 
             exit(0);
@@ -187,11 +194,9 @@ int main(int argc, char* argv[])
       Do not attempt to call show here that will be done later.
       This variable exists solely to insure the KDiff3Shell is deleted on exit.
     */
-    QPointer<KDiff3Shell> p(new KDiff3Shell());
+    const QPointer<KDiff3Shell> p(new KDiff3Shell());//QPointer will take it from here.
     Q_UNUSED(p);
 
     int retVal = QApplication::exec();
-    p.clear();
     return retVal;
 }
-
