@@ -1,3 +1,4 @@
+// clang-format off
 /*
  *  This file is part of KDiff3.
  *
@@ -5,16 +6,19 @@
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
 */
+// clang-format on
 
 #ifndef MERGER_H
 #define MERGER_H
 
 #include "diff.h"
 
+#include <memory>
+
 class Merger
 {
   public:
-    Merger(const DiffList* pDiffList1, const DiffList* pDiffList2);
+    Merger(const std::shared_ptr<DiffList>& pDiffList1, const std::shared_ptr<DiffList>& pDiffList2);
 
     /** Go one step. */
     void next();
@@ -34,15 +38,15 @@ class Merger
     {
       private:
         DiffList::const_iterator it;
-        const DiffList* pDiffList = nullptr;
+        std::shared_ptr<DiffList> pDiffList;
         Diff d;
         int idx;
 
       public:
-        MergeData(const DiffList* p, int i);
-        bool eq() const;
+        MergeData(const std::shared_ptr<DiffList>& p, int i);
+        [[nodiscard]] bool eq() const;
         void update();
-        bool isEnd() const;
+        [[nodiscard]] bool isEnd() const;
     };
 
     MergeData md1;
