@@ -4,25 +4,25 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
-#ifndef OPTION_DIALOG_H
-#define OPTION_DIALOG_H
+#ifndef OPTIONDIALOG_H
+#define OPTIONDIALOG_H
 
 #include "options.h"
 
-#include <QSharedPointer>   // for QSharedPointer
+#include <QFont>
+#include <QSharedPointer>
 #include <QStringList>
 #include <QGroupBox>
 
 #include <KPageDialog>
 #include <KSharedConfig>
 
-
 class QLabel;
 class QPlainTextEdit;
 
-class OptionItemBase;
 class OptionCheckBox;
 class OptionEncodingComboBox;
 class OptionLineEdit;
@@ -43,15 +43,13 @@ public:
 
     void setState(); // Must be called before calling exec();
 
-    void addOptionItem(OptionItemBase*);
-
     QSharedPointer<Options> getOptions() { return m_options; }
 
     static const QString s_historyEntryStartRegExpToolTip;
     static const QString s_historyEntryStartSortKeyOrderToolTip;
     static const QString s_autoMergeRegExpToolTip;
     static const QString s_historyStartRegExpToolTip;
-    
+
 protected Q_SLOTS:
     virtual void slotDefault();
     virtual void slotOk();
@@ -72,8 +70,6 @@ private:
     void setupDirectoryMergePage();
     void setupRegionalPage();
     void setupIntegrationPage();
-    void setupOtherOptions();
-
     void resetToDefaults();
 
     QSharedPointer<Options> m_options=QSharedPointer<Options>::create();
