@@ -4,14 +4,17 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 #ifndef OPTIONS_H
 #define OPTIONS_H
 
-#include <KSharedConfig>
+#include "combiners.h"
 
+#include <boost/signals2.hpp>
 #include <list>
+#include <memory>
 
 #include <QColor>
 #include <QFont>
@@ -19,65 +22,95 @@
 #include <QSize>
 #include <QStringList>
 
+#include <KSharedConfig>
+#include <qcolor.h>
+
+class ValueMap;
+
 class OptionItemBase;
 
+constexpr char KDIFF3_CONFIG_GROUP[] = "KDiff3 Options";
+
 enum e_LineEndStyle
 {
-   eLineEndStyleUnix=0,
-   eLineEndStyleDos,
-   eLineEndStyleAutoDetect,
-   eLineEndStyleUndefined, // only one line exists
-   eLineEndStyleConflict   // User must resolve manually
+    eLineEndStyleUnix = 0,
+    eLineEndStyleDos,
+    eLineEndStyleAutoDetect,
+    eLineEndStyleUndefined, // only one line exists
+    eLineEndStyleConflict   // User must resolve manually
 };
 
 class Options
 {
-public:
+  public:
+    static boost::signals2::signal<void()> apply;
+    static boost::signals2::signal<void()> resetToDefaults;
+    static boost::signals2::signal<void()> setToCurrent;
+    static boost::signals2::signal<void(ValueMap*)> read;
+    static boost::signals2::signal<void(ValueMap*)> write;
+
+    static boost::signals2::signal<void()> preserve;
+    static boost::signals2::signal<void()> unpreserve;
+    static boost::signals2::signal<bool(const QString&, const QString&), find> accept;
+
     void init();
 
-    void apply();
-
-    void resetToDefaults();
-
-    void setToCurrent();
-
-    void saveOptions(const KSharedConfigPtr config);
     void readOptions(const KSharedConfigPtr config);
+    void saveOptions(const KSharedConfigPtr config);
 
     const QString parseOptions(const QStringList& optionList);
-    QString calcOptionHelp();
+    [[nodiscard]] QString calcOptionHelp();
 
-    void addOptionItem(OptionItemBase* inItem);
+    [[nodiscard]] const QSize& getGeometry() const { return m_geometry; }
+    [[nodiscard]] const QPoint& getPosition() const { return m_position; }
 
-    const QSize& getGeometry() const { return m_geometry; }
-    void setGeometry(const QSize& size) { m_geometry = size; }
+    [[nodiscard]] bool isFullScreen() const { return m_bFullScreen; };
+    void setFullScreen(const bool fullScreen) { m_bFullScreen = fullScreen; };
 
-    const QPoint& getPosition() const { return m_position; }
-    void setPosition(const QPoint& pos) { m_position = pos; }
+    [[nodiscard]] bool isMaximised() const { return m_bMaximised; };
+    void setMaximised(const bool maximised) { m_bMaximised = maximised; };
 
-    bool isFullScreen() const { return m_bFullScreen; };
-
-    void setFullScreen(const bool fullScreen) { m_bFullScreen = fullScreen;};
-
-    bool isMaximised() const { return m_bMaximised; };
-
-    void setMaximised(const bool maximised) { m_bMaximised = maximised;};
-
-
-    bool isToolBarVisable() const { return m_bShowToolBar; }
+    [[nodiscard]] bool isToolBarVisable() const { return m_bShowToolBar; }
     void setToolbarState(bool inShown) { m_bShowToolBar = inShown; }
 
-    bool isStatusBarVisable() const { return m_bShowStatusBar; }
+    [[nodiscard]] bool isStatusBarVisable() const { return m_bShowStatusBar; }
     void setStatusBarState(bool inShown) { m_bShowStatusBar = inShown; }
 
-    bool wordWrapOn() const { return m_bWordWrap; }
-    void setWordWrap(const bool enabled) { m_bWordWrap = enabled;}
+    [[nodiscard]] inline const QFont& defaultFont() { return mFont; };
+    [[nodiscard]] inline const QFont& appFont() { return mAppFont; };
+
+    [[nodiscard]] bool wordWrapOn() const { return m_bWordWrap; }
+    void setWordWrap(const bool enabled) { m_bWordWrap = enabled; }
 
     [[nodiscard]] bool ignoreComments() const { return m_bIgnoreComments; }
 
     [[nodiscard]] bool whiteSpaceIsEqual() const { return m_bDmWhiteSpaceEqual; }
+
+    [[nodiscard]] const QColor& forgroundColor() const { return m_fgColor; };
+    [[nodiscard]] const QColor& backgroundColor() const
+    {
+        if(mPrintMode) return mPrintBackground;
+        return m_bgColor;
+    };
+    [[nodiscard]] const QColor& diffBackgroundColor() const { return m_diffBgColor; };
+    [[nodiscard]] const QColor& aColor() const { return m_colorA; };
+    [[nodiscard]] const QColor& bColor() const { return m_colorB; };
+    [[nodiscard]] const QColor& cColor() const { return m_colorC; };
+
+    [[nodiscard]] const QColor& conflictColor() const { return m_colorForConflict; }
+
+    inline void beginPrint() { mPrintMode = true; }
+    inline void endPrint() { mPrintMode = false; }
+
   private:
-    std::list<OptionItemBase*> mOptionItemList;
+    void addOptionItem(std::shared_ptr<OptionItemBase> inItem);
+
+    friend class OptionDialog;
+    std::list<std::shared_ptr<OptionItemBase>> mOptionItemList;
+
+    bool mPrintMode = false;
+
+    const QColor mPrintBackground = Qt::white;
 
     // Some settings that are not available in the option dialog:
     QSize  m_geometry = QSize(600, 400);
@@ -86,12 +119,12 @@ public:
     bool   m_bMaximised = false;
     bool   m_bShowToolBar = true;
     bool   m_bShowStatusBar = true;
-  public:
 
     // These are the results of the option dialog.
-    QFont m_font;
+    QFont mFont;
+    QFont mAppFont;
+
     //bool m_bItalicForDeltas;
-    QFont m_appFont;
 
     QColor m_fgColor = Qt::black;
     QColor m_bgColor = Qt::white;
@@ -100,6 +133,8 @@ public:
     QColor m_colorB;
     QColor m_colorC;
     QColor m_colorForConflict = Qt::red;
+
+  public:
     QColor m_currentRangeBgColor;
     QColor m_currentRangeDiffBgColor;
     QColor m_oldestFileColor = qRgb(0xf0, 0, 0);
@@ -115,18 +150,17 @@ public:
     int  m_tabSize = 8;
     bool m_bAutoCopySelection = false;
     bool m_bSameEncoding = true;
-    QTextCodec*  m_pEncodingA = nullptr;
+    QTextCodec* m_pEncodingA = nullptr;
     bool m_bAutoDetectUnicodeA = true;
-    QTextCodec*  m_pEncodingB = nullptr;
+    QTextCodec* m_pEncodingB = nullptr;
     bool m_bAutoDetectUnicodeB = true;
-    QTextCodec*  m_pEncodingC = nullptr;
+    QTextCodec* m_pEncodingC = nullptr;
     bool m_bAutoDetectUnicodeC = true;
-    QTextCodec*  m_pEncodingOut = nullptr;
+    QTextCodec* m_pEncodingOut = nullptr;
     bool m_bAutoSelectOutEncoding = true;
-    QTextCodec*  m_pEncodingPP = nullptr;
-    e_LineEndStyle  m_lineEndStyle = eLineEndStyleAutoDetect;
+    QTextCodec* m_pEncodingPP = nullptr;
+    e_LineEndStyle m_lineEndStyle = eLineEndStyleAutoDetect;
 
-    bool m_bPreserveCarriageReturn = false;
     bool m_bTryHard = true;
     bool m_bShowWhiteSpaceCharacters = true;
     bool m_bShowWhiteSpace = true;
@@ -194,5 +228,4 @@ public:
     bool m_bEscapeKeyQuits = false;
 };
 
-
-#endif
\ No newline at end of file
+#endif
