@@ -1,3 +1,4 @@
+// clang-format off
 /*
  *  This file is part of KDiff3.
  *
@@ -5,14 +6,15 @@
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
 */
+// clang-format on
 
-
+#include "defmac.h"
 #include "difftextwindow.h"
 #include "DirectoryInfo.h"
 #include "directorymergewindow.h"
 #include "fileaccess.h"
-#include "Logging.h"
 #include "kdiff3.h"
+#include "Logging.h"
 #include "optiondialog.h"
 #include "progress.h"
 #include "Utils.h"
@@ -29,6 +31,7 @@
 #include <QComboBox>
 #include <QDialog>
 #include <QDir>
+#include <QDockWidget>
 #include <QEvent> // QKeyEvent, QDropEvent, QInputEvent
 #include <QFile>
 #include <QLayout>
@@ -45,6 +48,7 @@
 
 #include <KLocalizedString>
 #include <KMessageBox>
+#include <KParts/MainWindow>
 #include <KShortcutsDialog>
 
 // Function uses setMinSize( sizeHint ) before adding the widget.
@@ -62,7 +66,8 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
     ProgressProxy pp;
     QStringList errors;
     // When doing a full analysis in the directory-comparison, then the statistics-results
-    // will be stored in the given TotalDiffStatus. Otherwise it will be 0.
+    // will be stored in the given TotalDiffStatus. Otherwise pTotalDiffStatus will
+    // point to m_totalDiffStatus.
     bool bLoadFiles = inFlags & InitFlag::loadFiles;
     bool bUseCurrentEncoding = inFlags & InitFlag::useCurrentEncoding;
     bool bAutoSolve = inFlags & InitFlag::autoSolve;
@@ -76,8 +81,7 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
     if(m_pOptions->whiteSpaceIsEqual())
         eIgnoreFlags |= IgnoreFlag::ignoreWhiteSpace;
 
-    Q_ASSERT(pTotalDiffStatus != nullptr);
-    //bool bPreserveCarriageReturn = m_pOptions->m_bPreserveCarriageReturn;
+    assert(pTotalDiffStatus != nullptr);
 
     bool bVisibleMergeResultWindow = !m_outputFilename.isEmpty();
 
@@ -112,7 +116,7 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
     //insure merge result window never has stale iterators.
     if(m_pMergeResultWindow) m_pMergeResultWindow->clearMergeList();
     m_diff3LineList.clear();
-    m_diff3LineVector.clear();
+    mDiff3LineVector.clear();
 
     if(bLoadFiles)
     {
@@ -175,7 +179,6 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
                 pp.setInformation(i18n("Linediff: A <-> B"));
                 qCInfo(kdiffMain) << i18n("Linediff: A <-> B");
                 m_diff3LineList.calcDiff3LineListUsingAB(&m_diffList12);
-                m_diff3LineList.debugLineCheck(m_sd1->getSizeLines(), e_SrcSelector::A);
 
                 pTotalDiffStatus->setTextEqualAB(m_diff3LineList.fineDiff(e_SrcSelector::A, m_sd1->getLineDataForDisplay(), m_sd2->getLineDataForDisplay(), eIgnoreFlags));
                 if(m_sd1->getSizeBytes() == 0) pTotalDiffStatus->setTextEqualAB(false);
@@ -218,7 +221,6 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
                 m_diff3LineList.calcDiff3LineListUsingAB(&m_diffList12);
             }
             pp.step();
-            m_diff3LineList.debugLineCheck(m_sd1->getSizeLines(), e_SrcSelector::A);
 
             pp.setInformation(i18n("Diff: A <-> C"));
             qCInfo(kdiffMain) << i18n("Diff: A <-> C");
@@ -229,9 +231,7 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
                                              m_pOptionDialog->getOptions());
 
                 m_diff3LineList.calcDiff3LineListUsingAC(&m_diffList13);
-                //m_diff3LineList.dump();
                 m_diff3LineList.correctManualDiffAlignment(&m_manualDiffHelpList);
-                //m_diff3LineList.dump();
                 m_diff3LineList.calcDiff3LineListTrim(m_sd1->getLineDataForDiff(), m_sd2->getLineDataForDiff(), m_sd3->getLineDataForDiff(), &m_manualDiffHelpList);
             }
             pp.step();
@@ -299,8 +299,9 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
                 pTotalDiffStatus->setTextEqualAB(false);
                 pTotalDiffStatus->setTextEqualBC(false);
             }
+
+            errors.append(m_sd3->getErrors());
         }
-        errors.append(m_sd3->getErrors());
     }
     else
     {
@@ -309,43 +310,27 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
 
     if(errors.isEmpty() && m_sd1->isText() && m_sd2->isText())
     {
-        m_diffBufferInfo->init(&m_diff3LineList, &m_diff3LineVector,
-                               m_sd1->getLineDataForDiff(), m_sd1->getSizeLines(),
-                               m_sd2->getLineDataForDiff(), m_sd2->getSizeLines(),
-                               m_sd3->getLineDataForDiff(), m_sd3->getSizeLines());
-        Diff3Line::m_pDiffBufferInfo = m_diffBufferInfo;
+        Diff3Line::m_pDiffBufferInfo->init(&m_diff3LineList,
+                               m_sd1->getLineDataForDiff(),
+                               m_sd2->getLineDataForDiff(),
+                               m_sd3->getLineDataForDiff());
 
         m_diff3LineList.calcWhiteDiff3Lines(m_sd1->getLineDataForDiff(), m_sd2->getLineDataForDiff(), m_sd3->getLineDataForDiff(), m_pOptions->ignoreComments());
-        m_diff3LineList.calcDiff3LineVector(m_diff3LineVector);
+        m_diff3LineList.calcDiff3LineVector(mDiff3LineVector);
     }
 
     // Calc needed lines for display
-    m_neededLines = m_diff3LineList.size();
-
-    QList<int> oldHeights;
-    if(m_pDirectoryMergeSplitter->isVisible())
-        oldHeights = m_pMainSplitter->sizes();
+    if(m_diff3LineList.size() <= TYPE_MAX(QtNumberType))
+        m_neededLines = SafeInt<QtNumberType>(m_diff3LineList.size());
+    else
+    {
+        errors.append("Too many lines in diff. Skiping file.");
+    }
 
+    //initView does first time setup for ui. Why is called it here?
     initView();
     m_pMergeResultWindow->connectActions();
 
-    if(m_pDirectoryMergeSplitter->isVisible())
-    {
-        if(oldHeights.count() < 2)
-            oldHeights.append(0);
-        if(oldHeights[1] == 0) // Distribute the available space evenly between the two widgets.
-        {
-            oldHeights[1] = oldHeights[0] / 2;
-            oldHeights[0] -= oldHeights[1];
-        }
-        if(oldHeights[0] == 0 && oldHeights[1] == 0)
-        {
-            oldHeights[1] = 100;
-            oldHeights[0] = 100;
-        }
-        m_pMainSplitter->setSizes(oldHeights);
-    }
-
     m_pMainWidget->setVisible(bGUI);
 
     m_bTripleDiff = !m_sd3->isEmpty();
@@ -360,15 +345,15 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
     {
         const ManualDiffHelpList* pMDHL = &m_manualDiffHelpList;
         m_pDiffTextWindow1->init(m_sd1->getAliasName(), m_sd1->getEncoding(), m_sd1->getLineEndStyle(),
-                                 m_sd1->getLineDataForDisplay(), m_sd1->getSizeLines(), &m_diff3LineVector, pMDHL);
+                                 m_sd1->getLineDataForDisplay(), m_sd1->getSizeLines(), &mDiff3LineVector, pMDHL);
         m_pDiffTextWindowFrame1->init();
 
         m_pDiffTextWindow2->init(m_sd2->getAliasName(), m_sd2->getEncoding(), m_sd2->getLineEndStyle(),
-                                 m_sd2->getLineDataForDisplay(), m_sd2->getSizeLines(), &m_diff3LineVector, pMDHL);
+                                 m_sd2->getLineDataForDisplay(), m_sd2->getSizeLines(), &mDiff3LineVector, pMDHL);
         m_pDiffTextWindowFrame2->init();
 
         m_pDiffTextWindow3->init(m_sd3->getAliasName(), m_sd3->getEncoding(), m_sd3->getLineEndStyle(),
-                                 m_sd3->getLineDataForDisplay(), m_sd3->getSizeLines(), &m_diff3LineVector, pMDHL);
+                                 m_sd3->getLineDataForDisplay(), m_sd3->getSizeLines(), &mDiff3LineVector, pMDHL);
         m_pDiffTextWindowFrame3->init();
 
         m_pDiffTextWindowFrame3->setVisible(m_bTripleDiff);
@@ -502,7 +487,7 @@ void KDiff3App::resizeDiffTextWindowHeight(int newHeight)
 {
     m_DTWHeight = newHeight;
 
-    DiffTextWindow::mVScrollBar->setRange(0, std::max(0, m_neededLines + 1 - newHeight));
+    DiffTextWindow::mVScrollBar->setRange(0, std::max(0, QtNumberType(m_neededLines + 1 - newHeight)));
     DiffTextWindow::mVScrollBar->setPageStep(newHeight);
     m_pOverview->setRange(DiffTextWindow::mVScrollBar->value(), DiffTextWindow::mVScrollBar->pageStep());
 
@@ -553,8 +538,6 @@ void KDiff3App::initView()
 
     mInitCalled = true;
     //m_pMainWidget // Contains vertical splitter and horiz scrollbar
-    m_pMainSplitter->addWidget(m_pMainWidget);
-    m_pMainWidget->setObjectName("MainWidget");
     QVBoxLayout* pVLayout = new QVBoxLayout(m_pMainWidget);
     pVLayout->setContentsMargins(0, 0, 0, 0);
     pVLayout->setSpacing(0);
@@ -590,11 +573,11 @@ void KDiff3App::initView()
     chk_connect(this, &KDiff3App::showWhiteSpaceToggled, m_pOverview, &Overview::slotRedraw);
     chk_connect(this, &KDiff3App::changeOverViewMode, m_pOverview, &Overview::setOverviewMode);
 
-    m_pDiffTextWindowFrame1 = new DiffTextWindowFrame(m_pDiffWindowSplitter, m_pOptionDialog->getOptions(), e_SrcSelector::A, m_sd1);
+    m_pDiffTextWindowFrame1 = new DiffTextWindowFrame(m_pDiffWindowSplitter, m_pOptionDialog->getOptions(), e_SrcSelector::A, m_sd1, *this);
     m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame1);
-    m_pDiffTextWindowFrame2 = new DiffTextWindowFrame(m_pDiffWindowSplitter, m_pOptionDialog->getOptions(), e_SrcSelector::B, m_sd2);
+    m_pDiffTextWindowFrame2 = new DiffTextWindowFrame(m_pDiffWindowSplitter, m_pOptionDialog->getOptions(), e_SrcSelector::B, m_sd2, *this);
     m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame2);
-    m_pDiffTextWindowFrame3 = new DiffTextWindowFrame(m_pDiffWindowSplitter, m_pOptionDialog->getOptions(), e_SrcSelector::C, m_sd3);
+    m_pDiffTextWindowFrame3 = new DiffTextWindowFrame(m_pDiffWindowSplitter, m_pOptionDialog->getOptions(), e_SrcSelector::C, m_sd3, *this);
     m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame3);
     m_pDiffTextWindow1 = m_pDiffTextWindowFrame1->getDiffTextWindow();
     m_pDiffTextWindow2 = m_pDiffTextWindowFrame2->getDiffTextWindow();
@@ -623,8 +606,6 @@ void KDiff3App::initView()
     MergeResultWindow::mVScrollBar = new QScrollBar(Qt::Vertical, m_pMergeWindowFrame);
     pMergeHLayout->addWidget(MergeResultWindow::mVScrollBar);
 
-    m_pMainSplitter->addWidget(m_pMainWidget);
-
     autoAdvance->setEnabled(true);
 
     QList<int> sizes = pVSplitter->sizes();
@@ -635,8 +616,8 @@ void KDiff3App::initView()
     sizes[1] = total / 2;
     pVSplitter->setSizes(sizes);
 
-    QList<int> hSizes;
-    hSizes << 1 << 1 << 1;
+    QList<int> hSizes = {1, 1, 1};
+
     m_pDiffWindowSplitter->setSizes(hSizes);
 
     m_pMergeResultWindow->installEventFilter(m_pMergeResultWindowTitle); // for focus tracking
@@ -697,7 +678,7 @@ void KDiff3App::initView()
 // called after word wrap is complete
 void KDiff3App::slotFinishMainInit()
 {
-    Q_ASSERT(m_pDiffTextWindow1 != nullptr && DiffTextWindow::mVScrollBar != nullptr && m_pOverview != nullptr);
+    assert(m_pDiffTextWindow1 != nullptr && DiffTextWindow::mVScrollBar != nullptr && m_pOverview != nullptr);
 
     setHScrollBarRange();
 
@@ -705,13 +686,13 @@ void KDiff3App::slotFinishMainInit()
     /*int newWidth  = m_pDiffTextWindow1->getNofVisibleColumns();*/
     m_DTWHeight = newHeight;
 
-    DiffTextWindow::mVScrollBar->setRange(0, std::max(0, m_neededLines + 1 - newHeight));
+    DiffTextWindow::mVScrollBar->setRange(0, std::max(0, QtNumberType(m_neededLines + 1 - newHeight)));
     DiffTextWindow::mVScrollBar->setPageStep(newHeight);
     m_pOverview->setRange(DiffTextWindow::mVScrollBar->value(), DiffTextWindow::mVScrollBar->pageStep());
 
     int d3l = -1;
     if(!m_manualDiffHelpList.empty())
-        d3l = m_manualDiffHelpList.front().calcManualDiffFirstDiff3LineIdx(m_diff3LineVector);
+        d3l = m_manualDiffHelpList.front().calcManualDiffFirstDiff3LineIdx(mDiff3LineVector);
     if(d3l >= 0 && m_pDiffTextWindow1)
     {
         int line = m_pDiffTextWindow1->convertDiff3LineIdxToLine(d3l);
@@ -727,7 +708,7 @@ void KDiff3App::slotFinishMainInit()
     if(m_pCornerWidget)
         m_pCornerWidget->setFixedSize(DiffTextWindow::mVScrollBar->width(), m_pHScrollBar->height());
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
     setUpdatesEnabled(true);
 
     bool bVisibleMergeResultWindow = !m_outputFilename.isEmpty();
@@ -735,7 +716,7 @@ void KDiff3App::slotFinishMainInit()
     if(m_bLoadFiles)
     {
         if(bVisibleMergeResultWindow)
-            m_pMergeResultWindow->showNrOfConflicts();
+            m_pMergeResultWindow->showNumberOfConflicts();
         else if(
             // Avoid showing this message during startup without parameters.
             !(m_sd1->getAliasName().isEmpty() && m_sd2->getAliasName().isEmpty() && m_sd3->getAliasName().isEmpty()) &&
@@ -802,7 +783,7 @@ void KDiff3App::slotFinishMainInit()
 
 void KDiff3App::resizeEvent(QResizeEvent* e)
 {
-    QSplitter::resizeEvent(e);
+    QMainWindow::resizeEvent(e);
     if(m_pCornerWidget)
         m_pCornerWidget->setFixedSize(DiffTextWindow::mVScrollBar->width(), m_pHScrollBar->height());
 }
@@ -825,7 +806,7 @@ void KDiff3App::keyPressEvent(QKeyEvent* keyEvent)
         return;
     }
 
-    bool bCtrl = (keyEvent->QInputEvent::modifiers() & Qt::ControlModifier) != 0;
+    bool bCtrl = (keyEvent->modifiers() & Qt::ControlModifier) != 0;
 
     switch(keyEvent->key())
     {
@@ -868,13 +849,13 @@ void KDiff3App::slotFinishDrop()
 
 void KDiff3App::slotFileOpen()
 {
-    if(!shouldContinue()) return;
+    if(!canContinue()) return;
 
     if(m_pDirectoryMergeWindow->isDirectoryMergeInProgress())
     {
         int result = KMessageBox::warningYesNo(this,
                                                i18n("You are currently doing a folder merge. Are you sure, you want to abort?"),
-                                               i18n("Warning"),
+                                               i18nc("Error dialog caption", "Warning"),
                                                KGuiItem(i18n("Abort")),
                                                KGuiItem(i18n("Continue Merging")));
         if(result != KMessageBox::Yes)
@@ -922,7 +903,8 @@ void KDiff3App::slotFileOpen()
                 bool bSuccess = doDirectoryCompare(false);
                 if(bSuccess)
                 {
-                    m_pDirectoryMergeSplitter->show();
+                    m_pDirectoryMergeDock->show();
+                    m_pDirectoryMergeInfoDock->show();
                     m_pMainWidget->hide();
                     break;
                 }
@@ -931,7 +913,8 @@ void KDiff3App::slotFileOpen()
             {
                 improveFilenames();
 
-                m_pDirectoryMergeSplitter->hide();
+                m_pDirectoryMergeDock->hide();
+                m_pDirectoryMergeInfoDock->hide();
                 mainInit(m_totalDiffStatus);
 
                 if((!m_sd1->getErrors().isEmpty()) ||
@@ -947,7 +930,7 @@ void KDiff3App::slotFileOpen()
                     if(!m_sd3->getErrors().isEmpty())
                         text += " - " + m_sd3->getAliasName() + '\n' + m_sd3->getErrors().join('\n') + '\n';
 
-                    KMessageBox::sorry(this, text, i18n("File open error"));
+                    KMessageBox::error(this, text, i18n("File open error"));
 
                     continue;
                 }
@@ -956,14 +939,14 @@ void KDiff3App::slotFileOpen()
         break;
     }
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
     slotStatusMsg(i18n("Ready."));
 }
 
-void KDiff3App::slotFileOpen2(QStringList &errors, const QString& fn1, const QString& fn2, const QString& fn3, const QString& ofn,
+void KDiff3App::slotFileOpen2(QStringList& errors, const QString& fn1, const QString& fn2, const QString& fn3, const QString& ofn,
                               const QString& an1, const QString& an2, const QString& an3, TotalDiffStatus* pTotalDiffStatus)
 {
-    if(!shouldContinue()) return;
+    if(!canContinue()) return;
 
     if(fn1.isEmpty() && fn2.isEmpty() && fn3.isEmpty() && ofn.isEmpty())
     {
@@ -1036,6 +1019,7 @@ void KDiff3App::slotFileNameChanged(const QString& fileName, e_SrcSelector winId
     QString an2 = m_sd2->getAliasName();
     QString fn3 = m_sd3->getFilename();
     QString an3 = m_sd3->getAliasName();
+
     if(winIdx == e_SrcSelector::A)
     {
         fn1 = fileName;
@@ -1081,7 +1065,7 @@ void KDiff3App::slotEditPaste()
     }
     else
     {
-        if(shouldContinue())
+        if(canContinue())
         {
             QString error;
             bool do_init = false;
@@ -1164,6 +1148,37 @@ void KDiff3App::slotGoNextDelta()
     Q_EMIT goNextDelta();
 }
 
+void KDiff3App::slotGoToLine()
+{
+    QDialog pDialog;
+    QVBoxLayout* l = new QVBoxLayout(&pDialog);
+
+    QLineEdit* pLineNumEdit = new QLineEdit();
+    //Limit imput to valid 1 based line numbers
+    pLineNumEdit->setValidator(new QIntValidator(1, DiffTextWindow::mVScrollBar->maximum(), pLineNumEdit));
+
+    QPushButton* pOkButton = new QPushButton(i18n("Ok"));
+    l->addWidget(pLineNumEdit);
+    l->addWidget(pOkButton);
+
+    chk_connect(pOkButton, &QPushButton::clicked, &pDialog,
+                ([&pDialog, pLineNumEdit]() {
+                    if(pLineNumEdit->text() != "")
+                    {
+                        int lineNum = pLineNumEdit->text().toInt();
+                        lineNum = qMax(lineNum - 2, 0);
+                        //No need for anything else here setValue triggers a valueChanged signal internally.
+                        DiffTextWindow::mVScrollBar->setValue(lineNum);
+                    }
+                    pDialog.close();
+                }));
+
+    pDialog.setWindowTitle(i18n("Go to Line"));
+    pDialog.setWindowFlag(Qt::WindowContextHelpButtonHint, false);
+    pDialog.setFixedSize(260, 110);
+    pDialog.exec();
+}
+
 void KDiff3App::choose(e_SrcSelector choice)
 {
     if(!m_bTimerBlock)
@@ -1207,7 +1222,7 @@ void KDiff3App::slotAutoSolve()
 {
     Q_EMIT autoSolve();
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::slotUnsolve()
@@ -1296,7 +1311,7 @@ void KDiff3App::slotConfigureKeys()
 
 void KDiff3App::slotRefresh()
 {
-    QApplication::setFont(m_pOptions->m_appFont);
+    QApplication::setFont(m_pOptions->appFont());
 
     Q_EMIT doRefresh();
 
@@ -1341,7 +1356,7 @@ void KDiff3App::slotSelectionEnd()
         }
     }
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::slotClipboardChanged()
@@ -1364,7 +1379,7 @@ void KDiff3App::slotOutputModified(bool bModified)
     if(bModified && !m_bOutputModified)
     {
         m_bOutputModified = true;
-        slotUpdateAvailabilities();
+        Q_EMIT updateAvailabilities();
     }
 }
 
@@ -1454,7 +1469,7 @@ void KDiff3App::recalcWordWrap(int visibleTextWidthForPrinting)
         }
         else
         {
-            m_neededLines = m_diff3LineVector.size();
+            m_neededLines = m_diff3LineList.size();
             if(m_pDiffTextWindow1)
                 m_pDiffTextWindow1->recalcWordWrap(false, 0, 0);
             if(m_pDiffTextWindow2)
@@ -1462,8 +1477,8 @@ void KDiff3App::recalcWordWrap(int visibleTextWidthForPrinting)
             if(m_pDiffTextWindow3)
                 m_pDiffTextWindow3->recalcWordWrap(false, 0, 0);
         }
-        bool bRunnablesStarted = DiffTextWindow::startRunnables();
-        if(!bRunnablesStarted)
+        mRunnablesStarted = DiffTextWindow::startRunnables();
+        if(!mRunnablesStarted)
             slotFinishRecalcWordWrap(visibleTextWidthForPrinting);
         else
         {
@@ -1482,7 +1497,11 @@ void KDiff3App::recalcWordWrap(int visibleTextWidthForPrinting)
 
 void KDiff3App::slotFinishRecalcWordWrap(int visibleTextWidthForPrinting)
 {
-    g_pProgressDialog->pop();
+    if(mRunnablesStarted)
+    {
+        ProgressProxy::endBackgroundTask();
+        mRunnablesStarted = false;
+    }
 
     if(m_pOptions->wordWrapOn() && g_pProgressDialog->wasCancelled())
     {
@@ -1537,7 +1556,7 @@ void KDiff3App::slotFinishRecalcWordWrap(int visibleTextWidthForPrinting)
         if(m_pOverview)
             m_pOverview->slotRedraw();
         if(DiffTextWindow::mVScrollBar)
-            DiffTextWindow::mVScrollBar->setRange(0, std::max(0, m_neededLines + 1 - m_DTWHeight));
+            DiffTextWindow::mVScrollBar->setRange(0, std::max(0, QtNumberType(m_neededLines + 1 - m_DTWHeight)));
         if(m_pDiffTextWindow1)
         {
             if(DiffTextWindow::mVScrollBar)
@@ -1584,7 +1603,7 @@ bool KDiff3App::doDirectoryCompare(const bool bCreateNewInstance)
     FileAccess f3(m_sd3->getFilename());
     FileAccess f4(m_outputFilename);
 
-    Q_ASSERT(f1.isDir());
+    assert(f1.isDir());
 
     if(bCreateNewInstance)
     {
@@ -1593,21 +1612,22 @@ bool KDiff3App::doDirectoryCompare(const bool bCreateNewInstance)
     else
     {
         //Only a debugging aid now. Used to insure m_bDirCompare is not changed
-        const bool bDirCompare = m_bDirCompare;
+        [[maybe_unused]] const bool bDirCompare = m_bDirCompare;
 
         FileAccess destDir;
 
         if(!m_bDefaultFilename) destDir = f4;
-        m_pDirectoryMergeSplitter->show();
+        m_pDirectoryMergeDock->show();
+        m_pDirectoryMergeInfoDock->show();
         m_pMainWidget->hide();
         setUpdatesEnabled(true);
 
         (*gDirInfo) = DirectoryInfo(f1, f2, f3, destDir);
+
         bool bSuccess = m_pDirectoryMergeWindow->init(
-            gDirInfo,
             !m_outputFilename.isEmpty());
         //This is a bug if it still happens.
-        Q_ASSERT(m_bDirCompare == bDirCompare);
+        assert(m_bDirCompare == bDirCompare);
 
         if(bSuccess)
         {
@@ -1630,13 +1650,12 @@ bool KDiff3App::doDirectoryCompare(const bool bCreateNewInstance)
                 m_pDiffTextWindowFrame3->init();
             }
         }
-        slotUpdateAvailabilities();
+        Q_EMIT updateAvailabilities();
         return bSuccess;
     }
 
     return true;
 }
-
 /*
     If A is targetted to an existing file and the paths point to directories attempt to find that file in the corrisponding
     directory. If it exists then the filename from A will be appended to the path.
@@ -1673,7 +1692,7 @@ void KDiff3App::improveFilenames()
 
 void KDiff3App::slotReload()
 {
-    if(!shouldContinue()) return;
+    if(!canContinue()) return;
 
     mainInit(m_totalDiffStatus);
 }
@@ -1685,7 +1704,7 @@ bool KDiff3App::canContinue()
     {
         int result = KMessageBox::warningYesNoCancel(this,
                                                      i18n("The merge result has not been saved."),
-                                                     i18n("Warning"),
+                                                     i18nc("Error dialog caption", "Warning"),
                                                      KGuiItem(i18n("Save && Continue")),
                                                      KGuiItem(i18n("Continue Without Saving")));
         if(result == KMessageBox::Cancel)
@@ -1695,7 +1714,7 @@ bool KDiff3App::canContinue()
             slotFileSave();
             if(m_bOutputModified)
             {
-                KMessageBox::sorry(this, i18n("Saving the merge result failed."), i18n("Warning"));
+                KMessageBox::error(this, i18n("Saving the merge result failed."), i18nc("Error dialog caption", "Warning"));
                 return false;
             }
         }
@@ -1709,8 +1728,10 @@ void KDiff3App::slotDirShowBoth()
 {
     if(dirShowBoth->isChecked())
     {
-        if(m_pDirectoryMergeSplitter)
-            m_pDirectoryMergeSplitter->setVisible(m_bDirCompare);
+        if(m_pDirectoryMergeDock)
+            m_pDirectoryMergeDock->setVisible(m_bDirCompare);
+        if(m_pDirectoryMergeInfoDock)
+            m_pDirectoryMergeInfoDock->setVisible(m_bDirCompare);
 
         m_pMainWidget->show();
     }
@@ -1720,33 +1741,37 @@ void KDiff3App::slotDirShowBoth()
         if(bTextDataAvailable)
         {
             m_pMainWidget->show();
-            m_pDirectoryMergeSplitter->hide();
+            m_pDirectoryMergeDock->hide();
+            m_pDirectoryMergeInfoDock->hide();
         }
         else if(m_bDirCompare)
         {
-            m_pDirectoryMergeSplitter->show();
+            m_pDirectoryMergeDock->show();
+            m_pDirectoryMergeInfoDock->show();
         }
     }
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::slotDirViewToggle()
 {
     if(m_bDirCompare)
     {
-        if(!m_pDirectoryMergeSplitter->isVisible())
+        if(!m_pDirectoryMergeDock->isVisible())
         {
-            m_pDirectoryMergeSplitter->show();
+            m_pDirectoryMergeDock->show();
+            m_pDirectoryMergeInfoDock->show();
             m_pMainWidget->hide();
         }
         else
         {
-            m_pDirectoryMergeSplitter->hide();
+            m_pDirectoryMergeDock->hide();
+            m_pDirectoryMergeInfoDock->hide();
             m_pMainWidget->show();
         }
     }
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::slotShowWindowAToggled()
@@ -1754,7 +1779,7 @@ void KDiff3App::slotShowWindowAToggled()
     if(m_pDiffTextWindow1 != nullptr)
     {
         m_pDiffTextWindowFrame1->setVisible(showWindowA->isChecked());
-        slotUpdateAvailabilities();
+        Q_EMIT updateAvailabilities();
     }
 }
 
@@ -1763,7 +1788,7 @@ void KDiff3App::slotShowWindowBToggled()
     if(m_pDiffTextWindow2 != nullptr)
     {
         m_pDiffTextWindowFrame2->setVisible(showWindowB->isChecked());
-        slotUpdateAvailabilities();
+        Q_EMIT updateAvailabilities();
     }
 }
 
@@ -1772,7 +1797,7 @@ void KDiff3App::slotShowWindowCToggled()
     if(m_pDiffTextWindow3 != nullptr)
     {
         m_pDiffTextWindowFrame3->setVisible(showWindowC->isChecked());
-        slotUpdateAvailabilities();
+        Q_EMIT updateAvailabilities();
     }
 }
 
@@ -1891,7 +1916,7 @@ void KDiff3App::slotMergeCurrentFile()
     }
     else if(m_pMainWidget->isVisible())
     {
-        if(!shouldContinue()) return;
+        if(!canContinue()) return;
 
         if(m_outputFilename.isEmpty())
         {
@@ -1992,28 +2017,28 @@ void KDiff3App::slotOverviewNormal()
 {
     Q_EMIT changeOverViewMode(e_OverviewMode::eOMNormal);
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::slotOverviewAB()
 {
     Q_EMIT changeOverViewMode(e_OverviewMode::eOMAvsB);
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::slotOverviewAC()
 {
     Q_EMIT changeOverViewMode(e_OverviewMode::eOMAvsC);
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::slotOverviewBC()
 {
     Q_EMIT changeOverViewMode(e_OverviewMode::eOMBvsC);
 
-    slotUpdateAvailabilities();
+    Q_EMIT updateAvailabilities();
 }
 
 void KDiff3App::slotNoRelevantChangesDetected()
@@ -2077,31 +2102,32 @@ void KDiff3App::slotClearManualDiffHelpList()
     slotRefresh();
 }
 
-void KDiff3App::slotEncodingChanged(QTextCodec* c)
+void KDiff3App::slotEncodingChanged(QTextCodec*)
 {
-    encodingChanged(c);
-    mainInit(m_totalDiffStatus, InitFlag::loadFiles | InitFlag::useCurrentEncoding | InitFlag::autoSolve ); // Init with reload
+    mainInit(m_totalDiffStatus, InitFlag::loadFiles | InitFlag::useCurrentEncoding | InitFlag::autoSolve); // Init with reload
     slotRefresh();
 }
 
 void KDiff3App::slotUpdateAvailabilities()
 {
-    if(m_pMainSplitter == nullptr || m_pDiffTextWindow2 == nullptr || m_pDiffTextWindow1 == nullptr || m_pDiffTextWindow3 == nullptr)
+    if(m_pDiffTextWindow2 == nullptr || m_pDiffTextWindow1 == nullptr || m_pDiffTextWindow3 == nullptr)
         return;
 
     bool bTextDataAvailable = (m_sd1->hasData() || m_sd2->hasData() || m_sd3->hasData());
 
     if(dirShowBoth->isChecked())
     {
-        if(m_pDirectoryMergeSplitter != nullptr)
-            m_pDirectoryMergeSplitter->setVisible(m_bDirCompare);
+        if(m_pDirectoryMergeDock != nullptr)
+            m_pDirectoryMergeDock->setVisible(m_bDirCompare);
+        if(m_pDirectoryMergeInfoDock != nullptr)
+            m_pDirectoryMergeInfoDock->setVisible(m_bDirCompare);
 
-        if( !m_pMainWidget->isVisible() &&
+        if(!m_pMainWidget->isVisible() &&
            bTextDataAvailable && !m_pDirectoryMergeWindow->isScanning())
             m_pMainWidget->show();
     }
 
-    bool bDiffWindowVisible =  m_pMainWidget->isVisible();
+    bool bDiffWindowVisible = m_pMainWidget->isVisible();
     bool bMergeEditorVisible = m_pMergeWindowFrame != nullptr && m_pMergeWindowFrame->isVisible() && m_pMergeResultWindow != nullptr;
 
     m_pDirectoryMergeWindow->updateAvailabilities(bMergeEditorVisible, m_bDirCompare, bDiffWindowVisible, chooseA, chooseB, chooseC);
@@ -2109,23 +2135,19 @@ void KDiff3App::slotUpdateAvailabilities()
     dirShowBoth->setEnabled(m_bDirCompare);
     dirViewToggle->setEnabled(
         m_bDirCompare &&
-        ( m_pDirectoryMergeSplitter != nullptr &&
-         ((!m_pDirectoryMergeSplitter->isVisible() && m_pMainWidget->isVisible()) ||
-          (m_pDirectoryMergeSplitter->isVisible() && !m_pMainWidget->isVisible() && bTextDataAvailable))));
+        (m_pDirectoryMergeDock != nullptr && m_pDirectoryMergeInfoDock != nullptr &&
+         ((!m_pDirectoryMergeDock->isVisible() && m_pMainWidget->isVisible()) ||
+          (m_pDirectoryMergeDock->isVisible() && !m_pMainWidget->isVisible() && bTextDataAvailable))));
 
     showWhiteSpaceCharacters->setEnabled(bDiffWindowVisible);
     autoAdvance->setEnabled(bMergeEditorVisible);
     mAutoSolve->setEnabled(bMergeEditorVisible && m_bTripleDiff);
     mUnsolve->setEnabled(bMergeEditorVisible);
 
+    editUndo->setEnabled(false); //Not yet implemented but planned.
     editCut->setEnabled(allowCut());
     editCopy->setEnabled(allowCopy());
 
-    if(m_pMergeResultWindow != nullptr)
-    {
-        m_pMergeResultWindow->slotUpdateAvailabilities();
-    }
-
     mMergeHistory->setEnabled(bMergeEditorVisible);
     mergeRegExp->setEnabled(bMergeEditorVisible);
     showWindowA->setEnabled(bDiffWindowVisible && (m_pDiffTextWindow2->isVisible() || m_pDiffTextWindow3->isVisible()));
@@ -2136,7 +2158,7 @@ void KDiff3App::slotUpdateAvailabilities()
     m_pFindDialog->m_pSearchInC->setEnabled(m_bTripleDiff);
     m_pFindDialog->m_pSearchInOutput->setEnabled(bMergeEditorVisible);
 
-    bool bSavable = bMergeEditorVisible && m_pMergeResultWindow->getNrOfUnsolvedConflicts() == 0;
+    bool bSavable = bMergeEditorVisible && m_pMergeResultWindow->getNumberOfUnsolvedConflicts() == 0;
     fileSave->setEnabled(m_bOutputModified && bSavable);
     fileSaveAs->setEnabled(bSavable);
 
