@@ -4,9 +4,11 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 #include "progress.h"
+
 #include "common.h"
 #include "defmac.h"
 
@@ -21,46 +23,25 @@
 #include <QStatusBar>
 #include <QThread>
 #include <QTimer>
+#include <QTimerEvent>
 #include <QVBoxLayout>
 
 #include <KIO/Job>
 #include <KLocalizedString>
 
+namespace placeholders = boost::placeholders;
+
 ProgressDialog* g_pProgressDialog = nullptr;
 
 ProgressDialog::ProgressDialog(QWidget* pParent, QStatusBar* pStatusBar)
     : QDialog(pParent), m_pStatusBar(pStatusBar)
 {
-    m_pGuiThread = QThread::currentThread();
-
-    setObjectName("ProgressDialog");
-    m_bStayHidden = false;
+    dialogUi.setupUi(this);
     setModal(true);
-    QVBoxLayout* layout = new QVBoxLayout(this);
+    //Abort if verticalLayout is not the immediate child of the dialog. This interferes with re-sizing.
+    assert(dialogUi.layout->parent() == this);
 
-    m_pInformation = new QLabel(" ", this);
-    layout->addWidget(m_pInformation);
-
-    m_pProgressBar = new QProgressBar();
-    m_pProgressBar->setRange(0, 1000);
-    layout->addWidget(m_pProgressBar);
-
-    m_pSubInformation = new QLabel(" ", this);
-    layout->addWidget(m_pSubInformation);
-
-    m_pSubProgressBar = new QProgressBar();
-    m_pSubProgressBar->setRange(0, 1000);
-    layout->addWidget(m_pSubProgressBar);
-
-    m_pSlowJobInfo = new QLabel(" ", this);
-    layout->addWidget(m_pSlowJobInfo);
-
-    QHBoxLayout* hlayout = new QHBoxLayout();
-    layout->addLayout(hlayout);
-    hlayout->addStretch(1);
-    m_pAbortButton = new QPushButton(i18n("&Cancel"), this);
-    hlayout->addWidget(m_pAbortButton);
-    chk_connect(m_pAbortButton, &QPushButton::clicked, this, &ProgressDialog::slotAbort);
+    chk_connect(dialogUi.abortButton, &QPushButton::clicked, this, &ProgressDialog::slotAbort);
     if(m_pStatusBar != nullptr)
     {
         m_pStatusBarWidget = new QWidget;
@@ -78,23 +59,42 @@ ProgressDialog::ProgressDialog(QWidget* pParent, QStatusBar* pStatusBar)
         m_pStatusBarWidget->setFixedHeight(m_pStatusBar->height());
         m_pStatusBarWidget->hide();
     }
-    else
-    {
-        m_pStatusProgressBar = nullptr;
-        m_pStatusAbortButton = nullptr;
-    }
 
-    m_progressDelayTimer = 0;
-    m_delayedHideTimer = 0;
-    m_delayedHideStatusBarWidgetTimer = 0;
     resize(400, 100);
 #ifndef AUTOTEST
     m_t1.start();
     m_t2.start();
 #endif
-    m_bWasCancelled = false;
-    m_eCancelReason = eUserAbort;
-    m_pJob = nullptr;
+
+    initConnections();
+}
+
+void ProgressDialog::initConnections()
+{
+    connections.push_back(ProgressProxy::startBackgroundTask.connect(boost::bind(&ProgressDialog::beginBackgroundTask, this)));
+    connections.push_back(ProgressProxy::endBackgroundTask.connect(boost::bind(&ProgressDialog::endBackgroundTask, this)));
+
+    connections.push_back(ProgressProxy::push.connect(boost::bind(&ProgressDialog::push, this)));
+    connections.push_back(ProgressProxy::pop.connect(boost::bind(&ProgressDialog::pop, this, placeholders::_1)));
+    connections.push_back(ProgressProxy::clearSig.connect(boost::bind(&ProgressDialog::clear, this)));
+
+    connections.push_back(ProgressProxy::enterEventLoop.connect(boost::bind(&ProgressDialog::enterEventLoop, this, placeholders::_1, placeholders::_2)));
+    connections.push_back(ProgressProxy::exitEventLoop.connect(boost::bind(&ProgressDialog::exitEventLoop, this)));
+
+    connections.push_back(ProgressProxy::setCurrentSig.connect(boost::bind(&ProgressDialog::setCurrent, this, placeholders::_1, placeholders::_2)));
+    connections.push_back(ProgressProxy::addNofStepsSig.connect(boost::bind(&ProgressDialog::addNofSteps, this, placeholders::_1)));
+    connections.push_back(ProgressProxy::setMaxNofStepsSig.connect(boost::bind(&ProgressDialog::setMaxNofSteps, this, placeholders::_1)));
+    connections.push_back(ProgressProxy::stepSig.connect(boost::bind(&ProgressDialog::step, this, placeholders::_1)));
+
+    connections.push_back(ProgressProxy::setRangeTransformationSig.connect(boost::bind(&ProgressDialog::setRangeTransformation, this, placeholders::_1, placeholders::_2)));
+    connections.push_back(ProgressProxy::setSubRangeTransformationSig.connect(boost::bind(&ProgressDialog::setSubRangeTransformation, this, placeholders::_1, placeholders::_2)));
+
+    connections.push_back(ProgressProxy::wasCancelledSig.connect(boost::bind(&ProgressDialog::wasCancelled, this)));
+
+    connections.push_back(ProgressProxy::setInformationSig.connect(boost::bind(
+        static_cast<void (ProgressDialog::*)(const QString&, bool)>(&ProgressDialog::setInformation),
+        this,
+        placeholders::_1, placeholders::_2)));
 }
 
 void ProgressDialog::setStayHidden(bool bStayHidden)
@@ -127,7 +127,6 @@ void ProgressDialog::setStayHidden(bool bStayHidden)
 
 void ProgressDialog::push()
 {
-#ifndef AUTOTEST
     ProgressLevelData pld;
     if(!m_progressStack.empty())
     {
@@ -137,19 +136,45 @@ void ProgressDialog::push()
     else
     {
         m_bWasCancelled = false;
+#ifndef AUTOTEST
         m_t1.restart();
         m_t2.restart();
+#endif
         if(!m_bStayHidden)
             show();
     }
 
     m_progressStack.push_back(pld);
+}
+
+void ProgressDialog::beginBackgroundTask()
+{
+    if(backgroundTaskCount > 0)
+    {
+#ifndef AUTOTEST
+        m_t1.restart();
+        m_t2.restart();
 #endif
+    }
+    backgroundTaskCount++;
+    if(!m_bStayHidden)
+        show();
+}
+
+void ProgressDialog::endBackgroundTask()
+{
+    if(backgroundTaskCount > 0)
+    {
+        backgroundTaskCount--;
+        if(backgroundTaskCount == 0)
+        {
+            hide();
+        }
+    }
 }
 
 void ProgressDialog::pop(bool bRedrawUpdate)
 {
-#ifndef AUTOTEST
     if(!m_progressStack.empty())
     {
         m_progressStack.pop_back();
@@ -160,124 +185,96 @@ void ProgressDialog::pop(bool bRedrawUpdate)
         else
             recalc(bRedrawUpdate);
     }
-#else
-    Q_UNUSED(bRedrawUpdate);
-#endif
 }
 
 void ProgressDialog::setInformation(const QString& info, int current, bool bRedrawUpdate)
 {
-#ifndef AUTOTEST
     if(m_progressStack.empty())
         return;
-    ProgressLevelData& pld = m_progressStack.back();
-    pld.m_current = current;
-    int level = m_progressStack.size();
-    if(level == 1)
-    {
-        m_pInformation->setText(info);
-        m_pSubInformation->setText("");
-        if(m_pStatusBar != nullptr && m_bStayHidden)
-            m_pStatusBar->showMessage(info);
-    }
-    else if(level == 2)
-    {
-        m_pSubInformation->setText(info);
-    }
+
+    setCurrent(current, false);
+    setInformationImp(info);
     recalc(bRedrawUpdate);
-#else
-    Q_UNUSED(info);
-    Q_UNUSED(bRedrawUpdate);
-    Q_UNUSED(current);
-#endif
 }
 
 void ProgressDialog::setInformation(const QString& info, bool bRedrawUpdate)
 {
-#ifndef AUTOTEST
     if(m_progressStack.empty())
         return;
-    //ProgressLevelData& pld = m_progressStack.back();
+
+    setInformationImp(info);
+    recalc(bRedrawUpdate);
+}
+
+void ProgressDialog::setMaxNofSteps(const qint64 maxNofSteps)
+{
+    if(m_progressStack.empty() || maxNofSteps == 0)
+        return;
+
+    ProgressLevelData& pld = m_progressStack.back();
+    pld.m_maxNofSteps = maxNofSteps;
+    pld.m_current = 0;
+}
+
+void ProgressDialog::setInformationImp(const QString& info)
+{
+#ifndef AUTOTEST
+    assert(!m_progressStack.empty());
+
     int level = m_progressStack.size();
     if(level == 1)
     {
-        m_pInformation->setText(info);
-        m_pSubInformation->setText("");
+        dialogUi.information->setText(info);
+        dialogUi.subInformation->setText("");
         if(m_pStatusBar && m_bStayHidden)
             m_pStatusBar->showMessage(info);
     }
     else if(level == 2)
     {
-        m_pSubInformation->setText(info);
+        dialogUi.subInformation->setText(info);
     }
-    recalc(bRedrawUpdate);
 #else
     Q_UNUSED(info);
-    Q_UNUSED(bRedrawUpdate);
-#endif
-}
-
-void ProgressDialog::setMaxNofSteps(const qint64 maxNofSteps)
-{
-#ifndef AUTOTEST
-    if(m_progressStack.empty() || maxNofSteps == 0)
-        return;
-    ProgressLevelData& pld = m_progressStack.back();
-    pld.m_maxNofSteps = maxNofSteps;
-    pld.m_current = 0;
-#else
-    Q_UNUSED(maxNofSteps);
 #endif
 }
 
 void ProgressDialog::addNofSteps(const qint64 nofSteps)
 {
-#ifndef AUTOTEST
     if(m_progressStack.empty())
         return;
+
     ProgressLevelData& pld = m_progressStack.back();
     pld.m_maxNofSteps.fetchAndAddRelaxed(nofSteps);
-#else
-    Q_UNUSED(nofSteps);
-#endif
 }
 
 void ProgressDialog::step(bool bRedrawUpdate)
 {
-#ifndef AUTOTEST
     if(m_progressStack.empty())
         return;
+
     ProgressLevelData& pld = m_progressStack.back();
     pld.m_current.fetchAndAddRelaxed(1);
     recalc(bRedrawUpdate);
-#else
-    Q_UNUSED(bRedrawUpdate);
-#endif
 }
 
 void ProgressDialog::setCurrent(qint64 subCurrent, bool bRedrawUpdate)
 {
-#ifndef AUTOTEST
     if(m_progressStack.empty())
         return;
+
     ProgressLevelData& pld = m_progressStack.back();
     pld.m_current = subCurrent;
+
     recalc(bRedrawUpdate);
-#else
-    Q_UNUSED(subCurrent);
-    Q_UNUSED(bRedrawUpdate);
-#endif
 }
 
 void ProgressDialog::clear()
 {
-#ifndef AUTOTEST
     if(m_progressStack.isEmpty())
         return;
 
     ProgressLevelData& pld = m_progressStack.back();
     setCurrent(pld.m_maxNofSteps);
-#endif
 }
 
 // The progressbar goes from 0 to 1 usually.
@@ -286,31 +283,23 @@ void ProgressDialog::clear()
 // Requirement: 0 < dMin < dMax < 1
 void ProgressDialog::setRangeTransformation(double dMin, double dMax)
 {
-#ifndef AUTOTEST
     if(m_progressStack.empty())
         return;
+
     ProgressLevelData& pld = m_progressStack.back();
     pld.m_dRangeMin = dMin;
     pld.m_dRangeMax = dMax;
     pld.m_current = 0;
-#else
-    Q_UNUSED(dMin);
-    Q_UNUSED(dMax);
-#endif
 }
 
 void ProgressDialog::setSubRangeTransformation(double dMin, double dMax)
 {
-#ifndef AUTOTEST
     if(m_progressStack.empty())
         return;
+
     ProgressLevelData& pld = m_progressStack.back();
     pld.m_dSubRangeMin = dMin;
     pld.m_dSubRangeMax = dMax;
-#else
-    Q_UNUSED(dMin);
-    Q_UNUSED(dMax);
-#endif
 }
 
 void ProgressDialog::enterEventLoop(KJob* pJob, const QString& jobInfo)
@@ -318,7 +307,7 @@ void ProgressDialog::enterEventLoop(KJob* pJob, const QString& jobInfo)
 #ifndef AUTOTEST
     m_pJob = pJob;
     m_currentJobInfo = jobInfo;
-    m_pSlowJobInfo->setText(m_currentJobInfo);
+    dialogUi.slowJobInfo->setText(m_currentJobInfo);
     if(m_progressDelayTimer)
         killTimer(m_progressDelayTimer);
     m_progressDelayTimer = startTimer(3000); /* 3 s delay */
@@ -375,22 +364,22 @@ void ProgressDialog::recalc(bool bUpdate)
             {
                 if(m_progressStack.empty())
                 {
-                    m_pProgressBar->setValue(0);
-                    m_pSubProgressBar->setValue(0);
+                    dialogUi.progressBar->setValue(0);
+                    dialogUi.subProgressBar->setValue(0);
                 }
                 else
                 {
                     QList<ProgressLevelData>::iterator i = m_progressStack.begin();
                     int value = int(1000.0 * (getAtomic(i->m_current) * (i->m_dRangeMax - i->m_dRangeMin) / getAtomic(i->m_maxNofSteps) + i->m_dRangeMin));
-                    m_pProgressBar->setValue(value);
+                    dialogUi.progressBar->setValue(value);
                     if(m_bStayHidden && m_pStatusProgressBar)
                         m_pStatusProgressBar->setValue(value);
 
                     ++i;
                     if(i != m_progressStack.end())
-                        m_pSubProgressBar->setValue(int(1000.0 * (getAtomic(i->m_current) * (i->m_dRangeMax - i->m_dRangeMin) / getAtomic(i->m_maxNofSteps) + i->m_dRangeMin)));
+                        dialogUi.subProgressBar->setValue((int)lround(1000.0 * (getAtomic(i->m_current) * (i->m_dRangeMax - i->m_dRangeMin) / getAtomic(i->m_maxNofSteps) + i->m_dRangeMin)));
                     else
-                        m_pSubProgressBar->setValue(int(1000.0 * m_progressStack.front().m_dSubRangeMin));
+                        dialogUi.subProgressBar->setValue((int)lround(1000.0 * m_progressStack.front().m_dSubRangeMin));
                 }
 
                 if(!m_bStayHidden && !isVisible())
@@ -447,14 +436,14 @@ void ProgressDialog::delayedHide()
         m_pJob = nullptr;
     }
     QDialog::hide();
-    m_pInformation->setText("");
+    dialogUi.information->setText("");
 
     //m_progressStack.clear();
 
-    m_pProgressBar->setValue(0);
-    m_pSubProgressBar->setValue(0);
-    m_pSubInformation->setText("");
-    m_pSlowJobInfo->setText("");
+    dialogUi.progressBar->setValue(0);
+    dialogUi.subProgressBar->setValue(0);
+    dialogUi.subInformation->setText("");
+    dialogUi.slowJobInfo->setText("");
 #endif
 }
 
@@ -550,7 +539,7 @@ void ProgressDialog::timerEvent(QTimerEvent* te)
         {
             show();
         }
-        m_pSlowJobInfo->setText(m_currentJobInfo);
+        dialogUi.slowJobInfo->setText(m_currentJobInfo);
     }
     else if(te->timerId() == m_delayedHideTimer)
     {
@@ -568,145 +557,3 @@ void ProgressDialog::timerEvent(QTimerEvent* te)
     Q_UNUSED(te);
 #endif
 }
-#ifndef AUTOTEST
-
-ProgressProxy::ProgressProxy()
-{
-    g_pProgressDialog->push();
-}
-#else
-ProgressProxy::ProgressProxy()=default;
-#endif
-
-#ifndef AUTOTEST
-ProgressProxy::~ProgressProxy()
-{
-    g_pProgressDialog->pop(false);
-}
-
-#else
-ProgressProxy::~ProgressProxy() = default;
-#endif
-
-void ProgressProxy::enterEventLoop(KJob* pJob, const QString& jobInfo)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->enterEventLoop(pJob, jobInfo);
-#else
-    Q_UNUSED(pJob);
-    Q_UNUSED(jobInfo);
-#endif
-}
-
-void ProgressProxy::exitEventLoop()
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->exitEventLoop();
-#endif
-}
-
-QDialog* ProgressProxy::getDialog()
-{
-    return g_pProgressDialog;
-}
-
-void ProgressProxy::setInformation(const QString& info, bool bRedrawUpdate)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->setInformation(info, bRedrawUpdate);
-#else
-    Q_UNUSED(info);
-    Q_UNUSED(bRedrawUpdate);
-#endif
-}
-
-void ProgressProxy::setInformation(const QString& info, int current, bool bRedrawUpdate)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->setInformation(info, current, bRedrawUpdate);
-#else
-    Q_UNUSED(info);
-    Q_UNUSED(bRedrawUpdate);
-    Q_UNUSED(current);
-#endif
-}
-
-void ProgressProxy::setCurrent(qint64 current, bool bRedrawUpdate)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->setCurrent(current, bRedrawUpdate);
-#else
-    Q_UNUSED(bRedrawUpdate);
-    Q_UNUSED(current);
-#endif
-}
-
-void ProgressProxy::step(bool bRedrawUpdate)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->step(bRedrawUpdate);
-#else
-    Q_UNUSED(bRedrawUpdate);
-#endif
-}
-
-void ProgressProxy::clear()
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->clear();
-#endif
-}
-
-void ProgressProxy::setMaxNofSteps(const qint64 maxNofSteps)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->setMaxNofSteps(maxNofSteps);
-#else
-    Q_UNUSED(maxNofSteps);
-#endif
-}
-
-void ProgressProxy::addNofSteps(const qint64 nofSteps)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->addNofSteps(nofSteps);
-#else
-    Q_UNUSED(nofSteps)
-#endif
-}
-
-bool ProgressProxy::wasCancelled()
-{
-#ifndef AUTOTEST
-    return g_pProgressDialog->wasCancelled();
-#else
-    return false;
-#endif
-}
-
-void ProgressProxy::setRangeTransformation(double dMin, double dMax)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->setRangeTransformation(dMin, dMax);
-#else
-    Q_UNUSED(dMin);
-    Q_UNUSED(dMax);
-#endif
-}
-
-void ProgressProxy::setSubRangeTransformation(double dMin, double dMax)
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->setSubRangeTransformation(dMin, dMax);
-#else
-    Q_UNUSED(dMin);
-    Q_UNUSED(dMax);
-#endif
-}
-
-void ProgressProxy::recalc()
-{
-#ifndef AUTOTEST
-    g_pProgressDialog->recalc(true);
-#endif
-}
