@@ -4,24 +4,25 @@
  * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
-*/
+ */
+// clang-format on
 
 #include "selection.h"
-
 #include "TypeUtils.h"
 
-#include <utility>   // for swap
-
 #include <QtGlobal>
 
-int Selection::firstPosInLine(LineRef l) const
+#include <utility>   // for swap
+
+
+qint32 Selection::firstPosInLine(LineRef l) const
 {
-    Q_ASSERT(firstLine.isValid());
+    assert(firstLine.isValid());
 
     LineRef l1 = firstLine;
     LineRef l2 = lastLine;
-    int p1 = firstPos;
-    int p2 = lastPos;
+    qint32 p1 = firstPos;
+    qint32 p2 = lastPos;
     if(l1 > l2)
     {
         std::swap(l1, l2);
@@ -38,14 +39,14 @@ int Selection::firstPosInLine(LineRef l) const
     return 0;
 }
 
-int Selection::lastPosInLine(LineRef l) const
+qint32 Selection::lastPosInLine(LineRef l) const
 {
-    Q_ASSERT(firstLine.isValid());
+    assert(firstLine.isValid());
 
     LineRef l1 = firstLine;
     LineRef l2 = lastLine;
-    int p1 = firstPos;
-    int p2 = lastPos;
+    qint32 p1 = firstPos;
+    qint32 p2 = lastPos;
 
     if(l1 > l2)
     {
@@ -60,10 +61,10 @@ int Selection::lastPosInLine(LineRef l) const
     if(l == l2)
         return p2;
 
-    return TYPE_MAX(int);
+    return TYPE_MAX(qint32);
 }
 
-bool Selection::within(LineRef l, LineRef p) const
+bool Selection::within(LineRef l, qint32 p) const
 {
     if(!firstLine.isValid())
         return false;
